@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2016, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -57,7 +57,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
     i.e. no encapsulation
   3. There's no explicit set of dependencies of a plugin, thus it's
     hard to initialize them properly
-  4. Plusing require a running server to operate.
+  4. Plugins require a running server to operate.
 
   The component infrastructure aims at allowing the MySQL server subsystems
   to be encapsulated into a set of logical components. Additional components
@@ -164,7 +164,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
   which accepts a path to a dynamic library to load components
   from using the OS dynamic library APIs. This dynamic loader scheme
   implementation does not rely on any server functionality and hence can
-  be placed into the component infrastructure core (a.k.a. the mimimal
+  be placed into the component infrastructure core (a.k.a. the minimal
   chassis).
 
   An additional service implementation, the path filter dynamic Loader
@@ -232,7 +232,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
   Note: minimal chassis support the native implementation of rwlocks and
   trylocks and it doesn't support the prlock because the prlock native
-  implementaion depends on mysql server code.
+  implementation depends on mysql server code.
 
   @section sect_components_layering_server_component The Server Component
 
@@ -276,7 +276,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
   Plugins do implement plugin APIs and may choose to call plugin service
   APIs exposed by the server. But in reality they have access to all of
   the server binary global symbols. So most if not all plugins choose to
-  use these instead of confining themselves into the (admitedly limited
+  use these instead of confining themselves into the (admittedly limited
   set of) plugin APIs available.
 
   The above makes plugins an integral part of the server codebase, more
@@ -356,7 +356,14 @@ void mysql_dynamic_loader_imp::deinit() {
            vector iterator invalidation. So, we need to make a copy
            of the vector and pass it to below function */
         auto components_to_unload = (*it);
-        unload_do_topological_order(components_to_unload);
+        if (unload_do_topological_order(components_to_unload)) {
+          /* since there is a error in the deinit function, report
+             the error and clear the vector elements(i.e components in that
+             group and then remove the component group node */
+          it->clear();
+          // removes the forward_list node
+          mysql_dynamic_loader_imp::urns_with_gen_list.remove(*it);
+        }
         /* Updating the iterator because unload_do_unload_components
           removes the mysql_dynamic_loader_imp::urns_with_gen_list node */
         it = mysql_dynamic_loader_imp::urns_with_gen_list.begin();
@@ -504,14 +511,6 @@ DEFINE_BOOL_METHOD(mysql_dynamic_loader_imp::iterator_create,
   return true;
 }
 
-/**
-  Releases Component iterator. Releases read lock on dynamic loader.
-
-  @param iterator Component iterator handle.
-  @return Status of performed operation
-  @retval false success
-  @retval true failure
-*/
 DEFINE_METHOD(void, mysql_dynamic_loader_imp::iterator_release,
               (my_h_component_iterator iterator)) {
   try {
@@ -847,10 +846,8 @@ bool mysql_dynamic_loader_imp::load_do_resolve_dependencies(
   /* List of services acquired for component dependencies. */
   std::vector<my_h_service *> acquired_services;
   auto guard = create_scope_guard([&acquired_services]() {
-    for (my_h_service *service_storage : acquired_services) {
+    for (my_h_service *service_storage : acquired_services)
       mysql_registry_imp::release(*service_storage);
-      *service_storage = nullptr;
-    }
   });
 
   /* Acquire services to meet component dependencies. */
@@ -1129,7 +1126,7 @@ bool mysql_dynamic_loader_imp::unload_do_lock_provided_services(
    */
   minimal_chassis::rwlock_scoped_lock lock =
       mysql_registry_imp::lock_registry_for_write();
-  return mysql_dynamic_loader_imp ::
+  return mysql_dynamic_loader_imp::
       unload_do_check_provided_services_reference_count(
           components_to_unload, dependency_graph, scheme_services);
 }
@@ -1150,7 +1147,7 @@ bool mysql_dynamic_loader_imp::unload_do_lock_provided_services(
   @retval false success
   @retval true failure
 */
-bool mysql_dynamic_loader_imp ::
+bool mysql_dynamic_loader_imp::
     unload_do_check_provided_services_reference_count(
         const std::vector<mysql_component *> &components_to_unload,
         const std::map<const void *, std::vector<mysql_component *>>
