@@ -1,4 +1,4 @@
--- Copyright (c) 2008, 2021, Oracle and/or its affiliates.
+-- Copyright (c) 2008, 2023, Oracle and/or its affiliates.
 --
 -- This program is free software; you can redistribute it and/or modify
 -- it under the terms of the GNU General Public License, version 2.0,
@@ -28,7 +28,7 @@ DELIMITER $$
 
 CREATE DEFINER=root@localhost PROCEDURE check_testcase_perfschema()
 BEGIN
-  IF ((SELECT count(*) from information_schema.engines
+  IF ((SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ count(*) from information_schema.engines
        where engine='PERFORMANCE_SCHEMA' and support='YES') = 1) THEN
   BEGIN
 
@@ -39,27 +39,27 @@ BEGIN
       END;
 
       -- Leave the instruments in the same state
-      SELECT * from performance_schema.setup_instruments
+      SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ * from performance_schema.setup_instruments
         where enabled='NO' order by NAME;
     END;
 
     -- Leave the consumers in the same state
-    SELECT * from performance_schema.setup_consumers
+    SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ * from performance_schema.setup_consumers
       order by NAME;
 
     -- Leave the actors setup in the same state
-    SELECT * from performance_schema.setup_actors
+    SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ * from performance_schema.setup_actors
       order by USER, HOST;
 
     -- Leave the objects setup in the same state
-    SELECT * from performance_schema.setup_objects
+    SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ * from performance_schema.setup_objects
       order by OBJECT_TYPE, OBJECT_SCHEMA, OBJECT_NAME;
 
     -- Leave the prepared statement instances in the same state
-    SELECT * from performance_schema.prepared_statements_instances;
+    SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ * from performance_schema.prepared_statements_instances;
 
     -- Leave the user defined functions in the same state
-    SELECT * from performance_schema.user_defined_functions
+    SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ * from performance_schema.user_defined_functions
       ORDER BY UDF_NAME;
   END;
   END IF;
@@ -68,27 +68,32 @@ END$$
 CREATE DEFINER=root@localhost PROCEDURE check_testcase_ndb()
 BEGIN
   -- Check NDB only if the ndbcluster plugin is enabled
-  IF ((SELECT count(*) FROM information_schema.engines
+  IF ((SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ count(*) FROM information_schema.engines
        WHERE engine='ndbcluster' AND support IN ('YES', 'DEFAULT')) = 1) THEN
 
     -- Check only if connection information is available
-    IF (SELECT LENGTH(VARIABLE_VALUE) > 0
+    IF (SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ LENGTH(VARIABLE_VALUE) > 0
           FROM performance_schema.global_variables
              WHERE VARIABLE_NAME='ndb_connectstring') THEN
 
-      -- List dict objects in NDB, skip objects created by ndbcluster plugin
-      -- and HashMap's since those can't be dropped after having been
-      -- created by test
-      SELECT id, indented_name FROM ndbinfo.dict_obj_tree
-        WHERE root_type != 24 AND  -- HashMap
-              root_name NOT IN ( 'mysql/def/ndb_schema',
-                                 'mysql/def/ndb_schema_result',
-                                 'mysql/def/ndb_index_stat_head',
-                                 'mysql/def/ndb_index_stat_sample',
-                                 'mysql/def/ndb_apply_status',
-                                 'mysql/def/ndb_sql_metadata')
-        ORDER BY path;
+      -- Check ndbinfo engine is enabled, else ndbinfo schema will not exists
+      IF ((SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ count(*) FROM information_schema.engines
+           WHERE engine='ndbinfo' AND support IN ('YES', 'DEFAULT')) = 1) THEN
 
+        -- List dict objects in NDB, skip objects created by ndbcluster plugin
+        -- and HashMap's since those can't be dropped after having been
+        -- created by test
+        SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ id, indented_name FROM ndbinfo.dict_obj_tree
+          WHERE root_type != 24 AND  -- HashMap
+                root_name NOT IN ( 'mysql/def/ndb_schema',
+                                   'mysql/def/ndb_schema_result',
+                                   'mysql/def/ndb_index_stat_head',
+                                   'mysql/def/ndb_index_stat_sample',
+                                   'mysql/def/ndb_apply_status',
+                                   'mysql/def/ndb_sql_metadata')
+          ORDER BY path;
+
+      END IF;
     END IF;
   END IF;
 END$$
@@ -105,7 +110,7 @@ BEGIN
 
   -- Dump all global variables except those that may change.
   -- timestamp changes if time passes. server_uuid changes if server restarts.
-  SELECT * FROM performance_schema.global_variables
+  SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ * FROM performance_schema.global_variables
     WHERE variable_name NOT IN ('timestamp', 'server_uuid',
                                 'gtid_executed', 'gtid_purged',
                                 'group_replication_group_name',
@@ -114,28 +119,28 @@ BEGIN
   ORDER BY VARIABLE_NAME;
 
   -- Dump all persisted variables, those that may change.
-  SELECT * FROM performance_schema.persisted_variables
+  SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ * FROM performance_schema.persisted_variables
     ORDER BY VARIABLE_NAME;
 
   -- Dump all databases, there should be none
   -- except those that was created during bootstrap
-  SELECT * FROM INFORMATION_SCHEMA.SCHEMATA ORDER BY SCHEMA_NAME;
+  SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ * FROM INFORMATION_SCHEMA.SCHEMATA ORDER BY SCHEMA_NAME;
 
   -- Dump all tablespaces, there should be none
-  SELECT FILE_NAME, FILE_TYPE, TABLESPACE_NAME, ENGINE FROM INFORMATION_SCHEMA.FILES
+  SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ FILE_NAME, FILE_TYPE, TABLESPACE_NAME, ENGINE FROM INFORMATION_SCHEMA.FILES
     WHERE FILE_TYPE !='TEMPORARY' ORDER BY FILE_NAME;
 
   -- The test database should not contain any tables
-  SELECT table_name AS tables_in_test FROM INFORMATION_SCHEMA.TABLES
+  SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ table_name AS tables_in_test FROM INFORMATION_SCHEMA.TABLES
     WHERE table_schema='test'
       ORDER BY TABLE_NAME;
 
   -- Show "mysql" database, tables and columns
-  SELECT CONCAT(table_schema, '.', table_name) AS tables_in_mysql
+  SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ CONCAT(table_schema, '.', table_name) AS tables_in_mysql
     FROM INFORMATION_SCHEMA.TABLES
       WHERE table_schema='mysql' AND table_name != 'ndb_apply_status'
         ORDER BY tables_in_mysql;
-  SELECT CONCAT(table_schema, '.', table_name) AS columns_in_mysql,
+  SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ CONCAT(table_schema, '.', table_name) AS columns_in_mysql,
        column_name, ordinal_position, column_default, is_nullable,
        data_type, character_maximum_length, character_octet_length,
        numeric_precision, numeric_scale, character_set_name,
@@ -145,11 +150,11 @@ BEGIN
         ORDER BY columns_in_mysql, column_name;
 
   -- Dump all events, there should be none
-  SELECT * FROM INFORMATION_SCHEMA.EVENTS;
+  SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ * FROM INFORMATION_SCHEMA.EVENTS;
 
   -- Dump all triggers except mtr internals, only those in the sys schema should exist
   -- do not select the CREATED column however, as tests like mysqldump.test / mysql_ugprade.test update this
-  SELECT TRIGGER_CATALOG, TRIGGER_SCHEMA, TRIGGER_NAME, EVENT_MANIPULATION,
+  SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ TRIGGER_CATALOG, TRIGGER_SCHEMA, TRIGGER_NAME, EVENT_MANIPULATION,
          EVENT_OBJECT_CATALOG, EVENT_OBJECT_SCHEMA, EVENT_OBJECT_TABLE, ACTION_ORDER, ACTION_CONDITION,
          ACTION_STATEMENT, ACTION_ORIENTATION, ACTION_TIMING ACTION_REFERENCE_OLD_TABLE, ACTION_REFERENCE_NEW_TABLE,
          ACTION_REFERENCE_OLD_ROW, ACTION_REFERENCE_NEW_ROW, SQL_MODE, DEFINER CHARACTER_SET_CLIENT,
@@ -160,7 +165,7 @@ BEGIN
 
   -- Dump all created procedures, only those in the sys schema should exist
   -- do not select the CREATED or LAST_ALTERED columns however, as tests like mysqldump.test / mysql_ugprade.test update this
-  SELECT SPECIFIC_NAME,ROUTINE_CATALOG,ROUTINE_SCHEMA,ROUTINE_NAME,ROUTINE_TYPE,DATA_TYPE,CHARACTER_MAXIMUM_LENGTH,
+  SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ SPECIFIC_NAME,ROUTINE_CATALOG,ROUTINE_SCHEMA,ROUTINE_NAME,ROUTINE_TYPE,DATA_TYPE,CHARACTER_MAXIMUM_LENGTH,
          CHARACTER_OCTET_LENGTH,NUMERIC_PRECISION,NUMERIC_SCALE,DATETIME_PRECISION,CHARACTER_SET_NAME,COLLATION_NAME,
          DTD_IDENTIFIER,ROUTINE_BODY,ROUTINE_DEFINITION,EXTERNAL_NAME,EXTERNAL_LANGUAGE,PARAMETER_STYLE,
          IS_DETERMINISTIC,SQL_DATA_ACCESS,SQL_PATH,SECURITY_TYPE,SQL_MODE,ROUTINE_COMMENT,DEFINER,
@@ -168,18 +173,18 @@ BEGIN
     FROM INFORMATION_SCHEMA.ROUTINES ORDER BY ROUTINE_SCHEMA, ROUTINE_NAME, ROUTINE_TYPE;
 
   -- Dump all views, only those in the sys schema should exist
-  SELECT * FROM INFORMATION_SCHEMA.VIEWS
+  SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ * FROM INFORMATION_SCHEMA.VIEWS
     ORDER BY TABLE_SCHEMA, TABLE_NAME;
 
   -- Dump all plugins, loaded with plugin-loading options or through
   -- INSTALL/UNINSTALL command
-  SELECT * FROM INFORMATION_SCHEMA.PLUGINS;
+  SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ * FROM INFORMATION_SCHEMA.PLUGINS;
 
   -- Leave InnoDB metrics in the same state
-  SELECT name, status FROM INFORMATION_SCHEMA.INNODB_METRICS
+  SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ name, status FROM INFORMATION_SCHEMA.INNODB_METRICS
     ORDER BY name;
 
-  SHOW GLOBAL STATUS LIKE 'slave_open_temp_tables';
+  SHOW GLOBAL STATUS LIKE 'replica_open_temp_tables';
 
   -- Check for number of active connections before & after the test run.
 
@@ -190,8 +195,11 @@ BEGIN
   --
   -- For "unauthenticated user", see Bug#30035699 "UNAUTHENTICATED USER" SHOWS UP IN CHECK-TESTCASE
   --
-  SELECT USER, HOST, DB, COMMAND, INFO FROM INFORMATION_SCHEMA.PROCESSLIST
-    WHERE COMMAND NOT IN ('Sleep')
+  -- Also (in similar fashion as above) exclude all 'Daemon' threads, they will
+  -- not give consistent result either.
+  --
+  SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ USER, HOST, DB, COMMAND, INFO FROM INFORMATION_SCHEMA.PROCESSLIST
+    WHERE COMMAND NOT IN ('Sleep', 'Daemon')
       AND USER NOT IN ('unauthenticated user','mysql.session', 'event_scheduler')
         ORDER BY COMMAND;
 
@@ -209,6 +217,7 @@ BEGIN
     mysql.default_roles,
     mysql.db,
     mysql.func,
+    mysql.general_log,
     mysql.global_grants,
     mysql.help_category,
     mysql.help_keyword,
@@ -218,7 +227,10 @@ BEGIN
     mysql.proxies_priv,
     mysql.replication_asynchronous_connection_failover,
     mysql.replication_asynchronous_connection_failover_managed,
+    mysql.replication_group_configuration_version,
+    mysql.replication_group_member_actions,
     mysql.role_edges,
+    mysql.slow_log,
     mysql.tables_priv,
     mysql.time_zone,
     mysql.time_zone_leap_second,
@@ -229,7 +241,7 @@ BEGIN
 
   -- Check that Replica IO Monitor thread state is the same before
   -- and after the test run, which is not running.
-  SELECT * FROM performance_schema.threads WHERE NAME="thread/sql/replica_monitor";
+  SELECT /*+SET_VAR(use_secondary_engine=OFF)*/ * FROM performance_schema.threads WHERE NAME="thread/sql/replica_monitor";
 
 END$$
 
