@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -33,6 +33,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include "m_string.h"
 #include "my_aes.h"
 #include "my_aes_impl.h"
+#include "mysys/my_kdf.h"
 
 /*
   xplugin needs BIO_new_bio_pair, but the server does not.
@@ -113,10 +114,40 @@ static const EVP_CIPHER *aes_evp_type(const my_aes_opmode mode) {
   }
 }
 
+/**
+  Creates required length of AES key,
+  Input key size can be smaller or bigger in length, we need exact AES key
+  size. If KDF options are valid and given, use KDF functionality. otherwise
+  use previously used method.
+
+  @param [out] rkey Output key
+  @param key Input key
+  @param key_length input key length
+  @param mode AES mode
+  @param kdf_options  KDF function options
+
+  @return 0 on success and 1 on failure
+*/
+int my_create_key(unsigned char *rkey, const unsigned char *key,
+                  uint32 key_length, enum my_aes_opmode mode,
+                  vector<string> *kdf_options) {
+  if (kdf_options) {
+    if (kdf_options->size() < 1) {
+      return 1;
+    }
+    const uint key_size = my_aes_opmode_key_sizes[mode] / 8;
+    return create_kdf_key(key, key_length, rkey, key_size, kdf_options);
+  }
+
+  my_aes_create_key(key, key_length, rkey, mode);
+  return 0;
+}
+
 int my_aes_encrypt(const unsigned char *source, uint32 source_length,
                    unsigned char *dest, const unsigned char *key,
                    uint32 key_length, enum my_aes_opmode mode,
-                   const unsigned char *iv, bool padding) {
+                   const unsigned char *iv, bool padding,
+                   vector<string> *kdf_options) {
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
   EVP_CIPHER_CTX stack_ctx;
   EVP_CIPHER_CTX *ctx = &stack_ctx;
@@ -127,8 +158,10 @@ int my_aes_encrypt(const unsigned char *source, uint32 source_length,
   int u_len, f_len;
   /* The real key to be used for encryption */
   unsigned char rkey[MAX_AES_KEY_LENGTH / 8];
-  my_aes_create_key(key, key_length, rkey, mode);
 
+  if (my_create_key(rkey, key, key_length, mode, kdf_options)) {
+    return MY_AES_BAD_DATA;
+  }
   if (!ctx || !cipher || (EVP_CIPHER_iv_length(cipher) > 0 && !iv))
     return MY_AES_BAD_DATA;
 
@@ -160,7 +193,8 @@ aes_error:
 int my_aes_decrypt(const unsigned char *source, uint32 source_length,
                    unsigned char *dest, const unsigned char *key,
                    uint32 key_length, enum my_aes_opmode mode,
-                   const unsigned char *iv, bool padding) {
+                   const unsigned char *iv, bool padding,
+                   vector<string> *kdf_options) {
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
   EVP_CIPHER_CTX stack_ctx;
   EVP_CIPHER_CTX *ctx = &stack_ctx;
@@ -173,7 +207,10 @@ int my_aes_decrypt(const unsigned char *source, uint32 source_length,
   /* The real key to be used for decryption */
   unsigned char rkey[MAX_AES_KEY_LENGTH / 8];
 
-  my_aes_create_key(key, key_length, rkey, mode);
+  if (my_create_key(rkey, key, key_length, mode, kdf_options)) {
+    return MY_AES_BAD_DATA;
+  }
+
   if (!ctx || !cipher || (EVP_CIPHER_iv_length(cipher) > 0 && !iv))
     return MY_AES_BAD_DATA;
 
@@ -204,20 +241,20 @@ aes_error:
   return MY_AES_BAD_DATA;
 }
 
-int my_aes_get_size(uint32 source_length, my_aes_opmode opmode) {
+longlong my_aes_get_size(uint32 source_length, my_aes_opmode opmode) {
   const EVP_CIPHER *cipher = aes_evp_type(opmode);
   size_t block_size;
 
   block_size = EVP_CIPHER_block_size(cipher);
 
-  return block_size > 1 ? block_size * (source_length / block_size) + block_size
-                        : source_length;
+  if (block_size <= 1) return source_length;
+  return block_size * (static_cast<ulonglong>(source_length) / block_size) +
+         block_size;
 }
 
 bool my_aes_needs_iv(my_aes_opmode opmode) {
   const EVP_CIPHER *cipher = aes_evp_type(opmode);
   int iv_length;
-
   iv_length = EVP_CIPHER_iv_length(cipher);
   assert(iv_length == 0 || iv_length == MY_AES_IV_SIZE);
   return iv_length != 0 ? true : false;
