@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -40,7 +39,6 @@
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/gcs_xcom_proxy.h"
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/gcs_xcom_state_exchange.h"
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/gcs_xcom_statistics_interface.h"
-#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/gcs_xcom_statistics_manager.h"
 
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/network/include/network_management_interface.h"
 
@@ -221,7 +219,7 @@ class Gcs_xcom_communication : public Gcs_xcom_communication_interface {
   */
 
   explicit Gcs_xcom_communication(
-      Gcs_xcom_statistics_manager_interface *stats, Gcs_xcom_proxy *proxy,
+      Gcs_xcom_statistics_updater *stats, Gcs_xcom_proxy *proxy,
       Gcs_xcom_view_change_control_interface *view_control,
       Gcs_xcom_engine *gcs_engine, Gcs_group_identifier const &group_id,
       std::unique_ptr<Network_provider_management_interface> comms_mgmt);
@@ -305,7 +303,7 @@ class Gcs_xcom_communication : public Gcs_xcom_communication_interface {
   std::map<int, const Gcs_communication_event_listener &> event_listeners;
 
   // Reference to the stats updater interface
-  Gcs_xcom_statistics_manager_interface *m_stats;
+  Gcs_xcom_statistics_updater *stats;
 
   // Reference to the xcom proxy interface
   Gcs_xcom_proxy *m_xcom_proxy;
