@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -42,6 +42,7 @@
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/gcs_xcom_networking.h"
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/gcs_xcom_notification.h"
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/gcs_xcom_state_exchange.h"
+#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/network/include/network_management_interface.h"
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/node_list.h"
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/node_set.h"
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/server_struct.h"
@@ -384,6 +385,8 @@ class Gcs_xcom_control : public Gcs_control_interface {
     @param[in] view_control View change control interface reference
     @param[in] boot Whether the node will be used to bootstrap the group
     @param[in] socket_util Reference to a socket utility
+    @param[in] comms_operation_interface an unique_ptr to a
+                                          Network_provider_operations_interface
   */
 
   explicit Gcs_xcom_control(
@@ -394,7 +397,9 @@ class Gcs_xcom_control : public Gcs_control_interface {
       Gcs_xcom_engine *gcs_engine,
       Gcs_xcom_state_exchange_interface *state_exchange,
       Gcs_xcom_view_change_control_interface *view_control, bool boot,
-      My_xp_socket_util *socket_util);
+      My_xp_socket_util *socket_util,
+      std::unique_ptr<Network_provider_operations_interface>
+          comms_operation_interface);
 
   ~Gcs_xcom_control() override;
 
@@ -452,12 +457,14 @@ class Gcs_xcom_control : public Gcs_control_interface {
     @param[in] message_id the message that conveys the View Change
     @param[in] xcom_nodes Set of nodes that participated in the consensus
                             to deliver the message
-    @param[in] same_view  Whether this global view was already delivered.
+    @param[in] do_not_deliver_to_client  Whether to filter this view from being
+                                         delivered to the client
     @param[in] max_synode XCom max synode
   */
 
   bool xcom_receive_global_view(synode_no const config_id, synode_no message_id,
-                                Gcs_xcom_nodes *xcom_nodes, bool same_view,
+                                Gcs_xcom_nodes *xcom_nodes,
+                                bool do_not_deliver_to_client,
                                 synode_no max_synode);
 
   /*
@@ -775,10 +782,13 @@ class Gcs_xcom_control : public Gcs_control_interface {
   // Proxy to GCS Sock Probe
   Gcs_sock_probe_interface *m_sock_probe_interface;
 
+  std::unique_ptr<Network_provider_operations_interface>
+      m_comms_operation_interface;
+
  protected:
   /*
     Whether the XCOM was left running or not meaning that the join
-    operation was successfuly executed. Note, however, that this
+    operation was successfully executed. Note, however, that this
     does not mean that any view was delivered yet.
 
     This flag is only updated by the MySQL GCS engine when the join
