@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -32,7 +32,7 @@ Keys_iterator::Keys_iterator(ILogger *logger) : logger(logger) {}
 
 /**
   This function initiates the internal pointers to point to
-  begining of the first element of keys metadata.
+  beginning of the first element of keys metadata.
 */
 void Keys_iterator::init() {
   key_metadata_list = keys->get_keys_metadata();
@@ -63,7 +63,7 @@ bool Keys_iterator::get_key(Key_metadata **km) {
     key_meta->user = key_metadata_list_iterator->user;
     *km = key_meta.release();
   }
-  key_metadata_list_iterator++;
+  ++key_metadata_list_iterator;
   return false;
 }
 
@@ -75,6 +75,6 @@ void Keys_iterator::deinit() {
   this->key_metadata_list.clear();
 }
 
-Keys_iterator::~Keys_iterator() {}
+Keys_iterator::~Keys_iterator() = default;
 
 }  // namespace keyring
