@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -55,7 +55,7 @@ bool is_super_user() {
   return has_super_privilege;
 }
 
-File File_io::open(PSI_file_key file_data_key MY_ATTRIBUTE((unused)),
+File File_io::open(PSI_file_key file_data_key [[maybe_unused]],
                    const char *filename, int flags, myf myFlags) {
   File file = mysql_file_open(file_data_key, filename, flags, MYF(0));
   if (file < 0 && (myFlags & MY_WME)) {
