@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2018, 2021, Oracle and/or its affiliates.
+  Copyright (c) 2018, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -55,7 +55,7 @@ enum class TlsVersion { AUTO, SSL_3, TLS_1_0, TLS_1_1, TLS_1_2, TLS_1_3 };
 /**
  * Verification of Cerifiticates.
  *
- * NONE no certifcate is verified
+ * NONE no certificate is verified
  * PEER verify the cert of the peer
  */
 enum class TlsVerify { NONE, PEER };
@@ -63,6 +63,11 @@ enum class TlsVerify { NONE, PEER };
 class HARNESS_TLS_EXPORT TlsLibraryContext {
  public:
   TlsLibraryContext();
+  TlsLibraryContext(const TlsLibraryContext &) = delete;
+  TlsLibraryContext(TlsLibraryContext &&) = delete;
+  TlsLibraryContext &operator=(const TlsLibraryContext &) = delete;
+  TlsLibraryContext &operator=(TlsLibraryContext &&) = delete;
+  ~TlsLibraryContext();
 };
 
 /**
@@ -72,7 +77,6 @@ class HARNESS_TLS_EXPORT TlsLibraryContext {
  * - SSL_CTX_set_session_cache_mode()
  * - SSL_CTX_set_alpn_select_cb()
  * - SSL_CTX_set_tlsext_ticket_key_cb()
- * - SSL_CTX_set_session_id_context()
  * - SSL_CTX_set_tlsext_servername_callback() for SNI
  * - SSL_CTX_set_cert_verify_callback() vs. SSL_CTX_set_verify()
  *
@@ -101,17 +105,6 @@ class HARNESS_TLS_EXPORT TlsContext {
     return OPENSSL_VERSION_NUMBER >= 0x1010100f;
   }
 
-  /**
-   * if TLS context allows getting cipher-lists.
-   *
-   * @returns if cipher_list() is supported.
-   * @retval false cipher_list() is not supported
-   */
-  static constexpr bool has_get_cipher_list() {
-    // 1.1.0 and later
-    return OPENSSL_VERSION_NUMBER >= 0x1010000f;
-  }
-
   /**
    * construct a TlsContext based on the SSL_METHODs provided by openssl.
    */
@@ -191,9 +184,6 @@ class HARNESS_TLS_EXPORT TlsContext {
 
   /**
    * get current cipher-list.
-   *
-   * @throws std::invalid_argument if API isn't supported
-   * @see has_get_cipher_list()
    */
   std::vector<std::string> cipher_list() const;
 
@@ -214,6 +204,11 @@ class HARNESS_TLS_EXPORT TlsContext {
    */
   int security_level() const;
 
+  /**
+   * get session reuse cache hits number
+   */
+  long session_cache_hits() const;
+
  protected:
   std::unique_ptr<SSL_CTX, decltype(&SSL_CTX_free)> ssl_ctx_{nullptr,
                                                              &SSL_CTX_free};
