@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -47,7 +47,9 @@
 #include <unistd.h>
 #endif
 
-#include "common.h"
+#include "exit_status.h"
+#include "mysql/harness/stdx/expected.h"
+#include "scope_guard.h"
 
 using namespace std::chrono_literals;
 using namespace std::string_literals;
@@ -104,7 +106,10 @@ std::error_code ProcessLauncher::send_shutdown_event(
       ok = GenerateConsoleCtrlEvent(CTRL_BREAK_EVENT, pi.dwProcessId);
       break;
     case ShutdownEvent::KILL:
-      ok = TerminateProcess(pi.hProcess, 0);
+      ok = TerminateProcess(pi.hProcess, 0x0);
+      break;
+    case ShutdownEvent::ABRT:
+      ok = TerminateProcess(pi.hProcess, STATUS_TIMEOUT);
       break;
   }
 #else
@@ -115,6 +120,9 @@ std::error_code ProcessLauncher::send_shutdown_event(
     case ShutdownEvent::KILL:
       ok = ::kill(childpid, SIGKILL) == 0;
       break;
+    case ShutdownEvent::ABRT:
+      ok = ::kill(childpid, SIGABRT) == 0;
+      break;
   }
 #endif
 
@@ -241,7 +249,7 @@ void ProcessLauncher::start() {
         last_error_code(),
         "Failed to HeapAlloc() when launchin a process " + arguments);
   }
-  mysql_harness::ScopeGuard clean_attribute_list_guard(
+  Scope_guard clean_attribute_list_guard(
       [&]() { DeleteProcThreadAttributeList(attribute_list); });
 
   if (InitializeProcThreadAttributeList(attribute_list, 1, 0, &size) == FALSE) {
@@ -303,41 +311,64 @@ void ProcessLauncher::start() {
   // res1 = WaitForSingleObject(pi.hThread, 100);
 }
 
-uint64_t ProcessLauncher::get_pid() const { return (uint64_t)pi.hProcess; }
-
-int ProcessLauncher::wait(std::chrono::milliseconds timeout) {
-  DWORD dwExit = 0;
-  BOOL get_ret{FALSE};
-  if ((get_ret = GetExitCodeProcess(pi.hProcess, &dwExit))) {
-    if (dwExit == STILL_ACTIVE) {
-      auto wait_ret = WaitForSingleObject(pi.hProcess, timeout.count());
-      switch (wait_ret) {
-        case WAIT_OBJECT_0:
-          get_ret = GetExitCodeProcess(pi.hProcess, &dwExit);
-          break;
-        case WAIT_TIMEOUT:
-          throw std::system_error(std::make_error_code(std::errc::timed_out));
-        case WAIT_FAILED:
-          throw std::system_error(last_error_code());
-        default:
-          throw std::runtime_error(
-              "Unexpected error while waiting for the process '" +
-              executable_path + "' to finish: " + std::to_string(wait_ret));
-      }
-    }
-  }
-  if (get_ret == FALSE) {
-    auto ec = last_error_code();
-    if (ec != std::error_code(ERROR_INVALID_HANDLE,
-                              std::system_category()))  // not closed already?
-      throw std::system_error(ec);
-    else
-      dwExit = 128;  // Invalid handle
-  }
-  return dwExit;
+ProcessLauncher::process_id_type ProcessLauncher::get_pid() const {
+  return pi.dwProcessId;
 }
 
-int ProcessLauncher::close() {
+ProcessLauncher::process_handle_type ProcessLauncher::get_process_handle()
+    const {
+  return pi.hProcess;
+}
+
+stdx::expected<ProcessLauncher::exit_status_type, std::error_code>
+ProcessLauncher::exit_code() {
+  DWORD dwExit = 0;
+
+  const BOOL ret = GetExitCodeProcess(pi.hProcess, &dwExit);
+  if (ret == 0) {
+    return stdx::make_unexpected(last_error_code());
+  } else if (dwExit == STILL_ACTIVE) {
+    return stdx::make_unexpected(std::make_error_code(std::errc::timed_out));
+  }
+
+  return {std::in_place, exit_status_type::native_t{}, dwExit};
+}
+
+ProcessLauncher::exit_status_type ProcessLauncher::native_wait(
+    std::chrono::milliseconds timeout) {
+  auto exit_res = exit_code();
+  if (exit_res) return *exit_res;
+
+  auto ec = exit_res.error();
+  if (ec != std::errc::timed_out) throw std::system_error(ec);
+
+  auto wait_ret = WaitForSingleObject(pi.hProcess, timeout.count());
+  switch (wait_ret) {
+    case WAIT_OBJECT_0:
+      break;
+    case WAIT_TIMEOUT:
+      throw std::system_error(std::make_error_code(std::errc::timed_out));
+    case WAIT_FAILED:
+      throw std::system_error(last_error_code());
+    default:
+      throw std::runtime_error(
+          "Unexpected error while waiting for the process '" + executable_path +
+          "' to finish: " + std::to_string(wait_ret));
+  }
+
+  // try again.
+  exit_res = exit_code();
+  if (exit_res) return *exit_res;
+
+  ec = exit_res.error();
+  if (ec == std::error_code(ERROR_INVALID_HANDLE, std::system_category())) {
+    return 128;  // Invalid handle
+  }
+
+  throw std::system_error(ec);
+}
+
+ProcessLauncher::exit_status_type ProcessLauncher::close() {
   DWORD dwExit;
   if (GetExitCodeProcess(pi.hProcess, &dwExit)) {
     if (dwExit == STILL_ACTIVE) {
@@ -373,6 +404,17 @@ int ProcessLauncher::close() {
   return 0;
 }
 
+static void throw_system_error(const std::error_code ec) {
+  if (ec == std::error_code(ERROR_INVALID_FUNCTION, std::system_category()) ||
+      ec == std::error_code(ERROR_INVALID_HANDLE, std::system_category()) ||
+      ec == std::error_code(ERROR_BAD_PIPE, std::system_category())) {
+    throw std::system_error(
+        std::error_code(ERROR_NOT_READY, std::system_category()));
+  }
+
+  throw std::system_error(ec);
+}
+
 int ProcessLauncher::read(char *buf, size_t count,
                           std::chrono::milliseconds timeout) {
   DWORD dwBytesRead;
@@ -392,7 +434,7 @@ int ProcessLauncher::read(char *buf, size_t count,
           ec == std::error_code(ERROR_BROKEN_PIPE, std::system_category())) {
         return EOF;
       } else {
-        throw std::system_error(last_error_code());
+        throw_system_error(ec);
       }
     }
 
@@ -422,7 +464,7 @@ int ProcessLauncher::read(char *buf, size_t count,
         ec == std::error_code(ERROR_BROKEN_PIPE, std::system_category())) {
       return EOF;
     } else {
-      throw std::system_error(ec);
+      throw_system_error(ec);
     }
   }
 
@@ -452,10 +494,6 @@ void ProcessLauncher::end_of_write() {
   child_in_wr_closed = true;
 }
 
-uint64_t ProcessLauncher::get_fd_write() const { return (uint64_t)child_in_wr; }
-
-uint64_t ProcessLauncher::get_fd_read() const { return (uint64_t)child_out_rd; }
-
 #else
 
 static std::vector<const char *> get_params(
@@ -606,8 +644,8 @@ void ProcessLauncher::start() {
   }
 }
 
-int ProcessLauncher::close() {
-  int result = 0;
+ProcessLauncher::exit_status_type ProcessLauncher::close() {
+  exit_status_type result = 0;
   if (is_alive) {
     // only try to kill the pid, if we started it. Not that we hurt someone
     // else.
@@ -618,19 +656,31 @@ int ProcessLauncher::close() {
     } else {
       try {
         // wait for it shutdown before using the big hammer
-        result = wait(kTerminateWaitInterval);
+        result = native_wait(kTerminateWaitInterval);
       } catch (const std::system_error &e) {
         if (e.code() != std::errc::no_such_process) {
-          std::error_code ec2 = send_shutdown_event(ShutdownEvent::KILL);
+          std::error_code ec2 = send_shutdown_event(ShutdownEvent::ABRT);
           if (ec2 != std::errc::no_such_process) {
             throw std::system_error(ec2);
           }
+          try {
+            // wait for it shutdown before using the big hammer
+            result = native_wait(kTerminateWaitInterval);
+          } catch (const std::system_error &e) {
+            std::error_code ec2 = send_shutdown_event(ShutdownEvent::KILL);
+            if (ec2 != std::errc::no_such_process) {
+              throw std::system_error(ec2);
+            }
+          }
+          result = native_wait();
         }
-        result = wait();
       }
     }
   }
 
+  std::lock_guard<std::mutex> fd_in_lock(fd_in_mtx_);
+  std::lock_guard<std::mutex> fd_out_lock(fd_out_mtx_);
+
   if (fd_out[0] != -1) ::close(fd_out[0]);
   if (fd_in[1] != -1) ::close(fd_in[1]);
 
@@ -642,12 +692,17 @@ int ProcessLauncher::close() {
 }
 
 void ProcessLauncher::end_of_write() {
+  std::lock_guard<std::mutex> fd_in_lock(fd_in_mtx_);
+
   if (fd_in[1] != -1) ::close(fd_in[1]);
   fd_in[1] = -1;
 }
 
 int ProcessLauncher::read(char *buf, size_t count,
                           std::chrono::milliseconds timeout) {
+  std::lock_guard<std::mutex> fd_out_lock(fd_out_mtx_);
+  if (fd_out[0] == -1) return 0;
+
   int n;
   fd_set set;
   struct timeval timeout_tv;
@@ -671,6 +726,10 @@ int ProcessLauncher::read(char *buf, size_t count,
 
 int ProcessLauncher::write(const char *buf, size_t count) {
   int n;
+
+  std::lock_guard<std::mutex> fd_in_lock(fd_in_mtx_);
+  if (fd_in[1] == -1) return 0;
+
   if ((n = (int)::write(fd_in[1], buf, count)) >= 0) return n;
 
   auto ec = last_error_code();
@@ -679,43 +738,59 @@ int ProcessLauncher::write(const char *buf, size_t count) {
   throw std::system_error(ec, "write");
 }
 
-uint64_t ProcessLauncher::get_pid() const {
+ProcessLauncher::process_handle_type ProcessLauncher::get_pid() const {
   static_assert(sizeof(pid_t) <= sizeof(uint64_t),
                 "sizeof(pid_t) > sizeof(uint64_t)");
-  return (uint64_t)childpid;
+  return childpid;
 }
 
-int ProcessLauncher::wait(const std::chrono::milliseconds timeout) {
+ProcessLauncher::process_handle_type ProcessLauncher::get_process_handle()
+    const {
+  return get_pid();
+}
+
+stdx::expected<ProcessLauncher::exit_status_type, std::error_code>
+ProcessLauncher::exit_code() {
+  int status;
+
+  const pid_t ret = ::waitpid(childpid, &status, WNOHANG);
+  if (ret == 0) {
+    return stdx::make_unexpected(std::make_error_code(std::errc::timed_out));
+  } else if (ret == -1) {
+    return stdx::make_unexpected(last_error_code());
+  }
+
+  return {std::in_place, exit_status_type::native_t{}, status};
+}
+
+ProcessLauncher::exit_status_type ProcessLauncher::native_wait(
+    const std::chrono::milliseconds timeout) {
   using namespace std::chrono_literals;
-  auto wait_time = timeout;
+
+  auto end_time = std::chrono::steady_clock::now() + timeout;
+
   do {
-    int status;
+    const auto wait_res = exit_code();
 
-    pid_t ret = ::waitpid(childpid, &status, WNOHANG);
+    if (wait_res) {
+      auto status = wait_res.value();
 
-    if (ret == 0) {
-      auto sleep_for = std::min(wait_time, kWaitPidCheckInterval);
-      if (sleep_for.count() > 0) {
-        std::this_thread::sleep_for(std::chrono::milliseconds(sleep_for));
-        wait_time -= sleep_for;
-      } else {
-        throw std::system_error(std::make_error_code(std::errc::timed_out));
-      }
-    } else if (ret == -1) {
-      throw std::system_error(last_error_code());
-    } else {
-      if (WIFEXITED(status)) {
-        return WEXITSTATUS(status);
-      } else if (WIFSIGNALED(status)) {
+      if (status.exited()) {
+        return status;
+      } else if (auto sig = status.terminated()) {
+        auto signum = *sig;
+
+        // drain the stdout|stderr.
         std::string msg;
         std::array<char, 1024> b;
         int n;
         while ((n = read(b.data(), b.size(), 100ms)) > 0) {
           msg.append(b.data(), n);
         }
+
         throw std::runtime_error(std::string("Process '" + executable_path +
                                              "' got signal " +
-                                             std::to_string(WTERMSIG(status))) +
+                                             std::to_string(signum)) +
                                  ":\n" + msg);
       } else {
         // it neither exited, not received a signal.
@@ -723,15 +798,32 @@ int ProcessLauncher::wait(const std::chrono::milliseconds timeout) {
             std::string("Process '" + executable_path + "' ... no idea"));
       }
     }
+
+    const auto ec = wait_res.error();
+
+    if (ec != std::errc::timed_out) throw std::system_error(ec);
+
+    const auto now = std::chrono::steady_clock::now();
+    if (now > end_time) break;
+
+    std::this_thread::sleep_until(
+        std::min(now + kWaitPidCheckInterval, end_time));
   } while (true);
+
+  throw std::system_error(std::make_error_code(std::errc::timed_out));
 }
-
-uint64_t ProcessLauncher::get_fd_write() const { return (uint64_t)fd_in[1]; }
-
-uint64_t ProcessLauncher::get_fd_read() const { return (uint64_t)fd_out[0]; }
-
 #endif
 
-int ProcessLauncher::kill() { return close(); }
+int ProcessLauncher::wait(const std::chrono::milliseconds timeout) {
+  auto wait_res = native_wait(timeout);
+
+  if (auto code = wait_res.exited()) {
+    return *code;
+  }
+
+  throw std::runtime_error("terminated?");
+}
+
+ProcessLauncher::exit_status_type ProcessLauncher::kill() { return close(); }
 
 }  // end of namespace mysql_harness
