@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2018, 2021, Oracle and/or its affiliates.
+  Copyright (c) 2018, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -36,9 +36,10 @@
 
 #include "mysql/harness/filesystem.h"
 #include "mysql/harness/utility/string.h"  // mysql_harness::join
-#include "mysqlrouter/utils.h"
-#include "print_version.h"             // build_version
-#include "welcome_copyright_notice.h"  // ORACLE_WELCOME_COPYRIGHT_NOTICE
+#include "mysqlrouter/utils.h"             // set_prompt_password
+#include "print_version.h"                 // build_version
+#include "router_config.h"                 // MYSQL_ROUTER_PACKAGE_NAME
+#include "welcome_copyright_notice.h"      // ORACLE_WELCOME_COPYRIGHT_NOTICE
 
 constexpr const char kAppExeFileName[]{"mysqlrouter_passwd"};
 
@@ -210,7 +211,7 @@ static std::string version_builder() {
 const std::string kHelpText(help_builder(cmdline_opts));
 const std::string kVersionText(version_builder());
 
-// placehoder in the opts to replace by the temp-filename
+// placeholder in the opts to replace by the temp-filename
 const std::string kPasswdPlaceholder("@passwdfile@");
 
 struct PasswdFrontendTestParam {
