@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2019, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -24,16 +24,20 @@
 
 #include "gr_notifications_listener.h"
 
-#include "common.h"  // rename_thread()
+#include <algorithm>
+#include <list>
+#include <map>
+#include <system_error>
+#include <thread>
+
+#include "my_thread.h"  // my_thread_self_setname
 #include "mysql/harness/logging/logging.h"
+#include "mysql/harness/net_ts/impl/poll.h"
+#include "mysql/harness/net_ts/impl/socket.h"
+#include "mysql/harness/net_ts/impl/socket_constants.h"
 #include "mysqld_error.h"
 #include "mysqlx_error.h"
 #include "mysqlxclient/xsession.h"
-#include "socket_operations.h"
-
-#include <algorithm>
-#include <map>
-#include <thread>
 
 IMPORT_LOG_FUNCTIONS()
 
@@ -51,13 +55,9 @@ const auto kXSesssionPingTimeout = kXSesssionWaitTimeout / 2;
 
 namespace {
 struct NodeId {
-#ifdef _WIN32
-  using native_handle_type = SOCKET;
-  static const native_handle_type kInvalidSocket{INVALID_SOCKET};
-#else
-  using native_handle_type = int;
-  static const native_handle_type kInvalidSocket{-1};
-#endif
+  using native_handle_type = net::impl::socket::native_handle_type;
+  static const native_handle_type kInvalidSocket{
+      net::impl::socket::kInvalidSocket};
 
   std::string host;
   uint16_t port;
@@ -86,24 +86,22 @@ using NodeSession = std::shared_ptr<xcl::XSession>;
 using namespace std::chrono_literals;
 
 struct GRNotificationListener::Impl {
-  std::string user_name;
-  std::string password;
+  mysqlrouter::UserCredentials user_credentials;
 
   std::map<NodeId, NodeSession> sessions_;
   bool sessions_changed_{false};
   std::mutex configuration_data_mtx_;
-  bool mysqlx_wait_timeout_set_{false};
+  std::atomic<bool> mysqlx_wait_timeout_set_{false};
 
   std::unique_ptr<std::thread> listener_thread;
   std::atomic<bool> terminate{false};
   NotificationClb notification_callback;
-  std::string last_view_id;
 
   std::chrono::steady_clock::time_point last_ping_timepoint =
       std::chrono::steady_clock::now();
 
-  Impl(const std::string &auth_user_name, const std::string &auth_password)
-      : user_name(auth_user_name), password(auth_password) {}
+  Impl(const mysqlrouter::UserCredentials &auth_user_credentials)
+      : user_credentials(auth_user_credentials) {}
 
   ~Impl();
 
@@ -111,17 +109,16 @@ struct GRNotificationListener::Impl {
   void listener_thread_func();
   bool read_from_session(const NodeId &node_id, NodeSession &session);
 
-  xcl::XError enable_notices(xcl::XSession &session,
-                             const NodeId &node_id) noexcept;
+  xcl::XError enable_notices(xcl::XSession &session, const NodeId &node_id,
+                             const std::string &cluster_name) noexcept;
   void set_mysqlx_wait_timeout(xcl::XSession &session,
                                const NodeId &node_id) noexcept;
   void check_mysqlx_wait_timeout();
   xcl::XError ping(xcl::XSession &session) noexcept;
   void remove_node_session(const NodeId &node) noexcept;
 
-  void reconfigure(
-      const std::vector<metadata_cache::ManagedInstance> &instances,
-      const NotificationClb &notification_clb);
+  void reconfigure(const metadata_cache::ClusterTopology &cluster_topology,
+                   const NotificationClb &notification_clb);
 
   // handles the notice from the session
   xcl::Handler_result notice_handler(const xcl::XProtocol *,
@@ -139,20 +136,12 @@ xcl::Handler_result GRNotificationListener::Impl::notice_handler(
       Mysqlx::Notice::Frame::Type::Frame_Type_GROUP_REPLICATION_STATE_CHANGED) {
     Mysqlx::Notice::GroupReplicationStateChanged change;
     change.ParseFromArray(payload, static_cast<int>(payload_size));
-    log_debug("Got notification from the cluster. type=%d; view_id=%s; ",
-              change.type(), change.view_id().c_str());
+    log_debug(
+        "Got notification from the cluster. type=%d; view_id=%s; Refreshing "
+        "metadata.",
+        change.type(), change.view_id().c_str());
 
-    const bool view_id_changed =
-        change.view_id().empty() || (change.view_id() != last_view_id);
-    if (view_id_changed) {
-      log_debug(
-          "Cluster notification: new view_id='%s'; previous view_id='%s'. "
-          "Refreshing metadata.",
-          change.view_id().c_str(), last_view_id.c_str());
-
-      notify = true;
-      last_view_id = change.view_id();
-    }
+    notify = true;
   }
 
   if (notify && notification_callback) {
@@ -190,8 +179,9 @@ xcl::XError GRNotificationListener::Impl::connect(NodeSession &session,
 
   log_debug("Connecting GR Notices listener on %s:%d", node_id.host.c_str(),
             node_id.port);
-  err = session->connect(node_id.host.c_str(), node_id.port, user_name.c_str(),
-                         password.c_str(), "");
+  err = session->connect(node_id.host.c_str(), node_id.port,
+                         user_credentials.username.c_str(),
+                         user_credentials.password.c_str(), "");
   if (err) {
     log_warning(
         "Failed connecting GR Notices listener on %s:%d; (err_code=%d; "
@@ -213,9 +203,10 @@ void GRNotificationListener::Impl::listener_thread_func() {
   size_t sessions_qty{0};
   std::unique_ptr<pollfd[]> fds;
 
-  mysql_harness::rename_thread("GR Notify");
+  my_thread_self_setname("GR Notify");
 
   while (!terminate) {
+    std::list<NodeSession> used_sessions;
     // first check if the set of fds did not change and we shouldn't reconfigure
     {
       std::lock_guard<std::mutex> lock(configuration_data_mtx_);
@@ -234,6 +225,15 @@ void GRNotificationListener::Impl::listener_thread_func() {
         }
         sessions_changed_ = false;
       }
+
+      // we use the fds so we need to keep the session objects alive to prevent
+      // the fds being released to the OS and reused while the poll is using
+      // those fds. For that we copy the sessions shared pointers to our list,
+      // it will be cleared at the end of the loop
+      std::for_each(sessions_.begin(), sessions_.end(),
+                    [&used_sessions](const auto &s) {
+                      used_sessions.push_back(s.second);
+                    });
     }
 
     if (sessions_qty == 0) {
@@ -247,21 +247,21 @@ void GRNotificationListener::Impl::listener_thread_func() {
       check_mysqlx_wait_timeout();
     }
 
-    const int poll_res = mysql_harness::SocketOperations::instance()->poll(
-        fds.get(), sessions_qty, kPollTimeout);
-    if (poll_res <= 0) {
-      // poll has timed out or failed
-      const int err_no =
-          mysql_harness::SocketOperations::instance()->get_errno();
-      // if this is timeout or EINTR just sleep and go to the next iteration
-      if (poll_res == 0 || EINTR == err_no) {
+    const auto poll_res =
+        net::impl::poll::poll(fds.get(), sessions_qty, kPollTimeout);
+    if (!poll_res) {
+      // poll has failed
+      if (poll_res.error() == make_error_condition(std::errc::interrupted)) {
+        // got interrupted. Sleep a bit more
         std::this_thread::sleep_for(kPollTimeout);
+      } else if (poll_res.error() == make_error_code(std::errc::timed_out)) {
+        // poll has timed out, sleep time already passed.
       } else {
         // any other error is fatal
         log_error(
             "poll() failed with error: %d, clearing all the sessions in the GR "
             "Notice thread",
-            err_no);
+            poll_res.error().value());
         sessions_.clear();
         sessions_changed_ = true;
       }
@@ -401,8 +401,10 @@ GRNotificationListener::Impl::~Impl() {
  * - state_changed
  */
 xcl::XError GRNotificationListener::Impl::enable_notices(
-    xcl::XSession &session, const NodeId &node_id) noexcept {
-  log_info("Enabling notices for cluster changes");
+    xcl::XSession &session, const NodeId &node_id,
+    const std::string &cluster_name) noexcept {
+  log_info("Enabling GR notices for cluster '%s' changes on node %s:%u",
+           cluster_name.c_str(), node_id.host.c_str(), node_id.port);
   xcl::XError err;
 
   xcl::Argument_value::Object arg_obj;
@@ -423,16 +425,17 @@ xcl::XError GRNotificationListener::Impl::enable_notices(
                                           {xcl::Argument_value(arg_obj)}, &err);
 
   if (!err) {
-    log_debug("Enabled notices for cluster changes on connection to node %s:%d",
-              node_id.host.c_str(), node_id.port);
+    log_debug(
+        "Enabled GR notices for cluster changes on connection to node %s:%d",
+        node_id.host.c_str(), node_id.port);
   } else if (err.error() == ER_X_BAD_NOTICE) {
     log_warning(
-        "Failed enabling notices on the node %s:%d. This MySQL server "
+        "Failed enabling GR notices on the node %s:%d. This MySQL server "
         "version does not support GR notifications (err_code=%d; err_msg='%s')",
         node_id.host.c_str(), node_id.port, err.error(), err.what());
   } else {
     log_warning(
-        "Failed enabling notices on the node %s:%d; (err_code=%d; "
+        "Failed enabling GR notices on the node %s:%d; (err_code=%d; "
         "err_msg='%s')",
         node_id.host.c_str(), node_id.port, err.error(), err.what());
   }
@@ -472,20 +475,23 @@ xcl::XError GRNotificationListener::Impl::ping(
 }
 
 void GRNotificationListener::Impl::reconfigure(
-    const std::vector<metadata_cache::ManagedInstance> &instances,
+    const metadata_cache::ClusterTopology &cluster_topology,
     const NotificationClb &notification_clb) {
   std::lock_guard<std::mutex> lock(configuration_data_mtx_);
 
   notification_callback = notification_clb;
 
+  const auto all_nodes = cluster_topology.get_all_members();
   // if there are connections to the nodes that are no longer required, remove
   // them first
   for (auto it = sessions_.cbegin(); it != sessions_.cend();) {
-    if (std::find_if(instances.begin(), instances.end(),
+    if (std::find_if(all_nodes.begin(), all_nodes.end(),
                      [&it](const metadata_cache::ManagedInstance &i) {
                        return it->first.host == i.host &&
                               it->first.port == i.xport;
-                     }) != instances.end()) {
+                     }) == all_nodes.end()) {
+      log_info("Removing unused GR notification session to '%s:%d'",
+               it->first.host.c_str(), it->first.port);
       sessions_.erase(it++);
       sessions_changed_ = true;
     } else {
@@ -494,33 +500,36 @@ void GRNotificationListener::Impl::reconfigure(
   }
 
   // check if there are some new nodes that we should connect to
-  for (const auto &instance : instances) {
-    NodeId node_id{instance.host, instance.xport, NodeId::kInvalidSocket};
-    if (std::find_if(
-            sessions_.begin(), sessions_.end(),
-            [&node_id](const std::pair<const NodeId, NodeSession> &node) {
-              return node.first.host == node_id.host &&
-                     node.first.port == node_id.port;
-            }) == sessions_.end()) {
+  for (const auto &cluster : cluster_topology.clusters_data) {
+    for (const auto &instance : cluster.members) {
       NodeId node_id{instance.host, instance.xport, NodeId::kInvalidSocket};
-      NodeSession session;
-      // If we could not connect it's not fatal, we only log it and live with
-      // the node not being monitored for GR notifications.
-      if (connect(session, node_id)) continue;
+      if (std::find_if(
+              sessions_.begin(), sessions_.end(),
+              [&node_id](const std::pair<const NodeId, NodeSession> &node) {
+                return node.first.host == node_id.host &&
+                       node.first.port == node_id.port;
+              }) == sessions_.end()) {
+        NodeId node_id{instance.host, instance.xport, NodeId::kInvalidSocket};
+        NodeSession session;
+        // If we could not connect it's not fatal, we only log it and live with
+        // the node not being monitored for GR notifications.
+        if (connect(session, node_id)) continue;
 
-      set_mysqlx_wait_timeout(*session, node_id);
+        set_mysqlx_wait_timeout(*session, node_id);
 
-      if (enable_notices(*session, node_id)) continue;
+        if (enable_notices(*session, node_id, cluster.name)) continue;
 
-      session->get_protocol().add_notice_handler(
-          [this](const xcl::XProtocol *protocol, const bool is_global,
-                 const Mysqlx::Notice::Frame::Type type, const char *data,
-                 const uint32_t data_length) -> xcl::Handler_result {
-            return notice_handler(protocol, is_global, type, data, data_length);
-          });
+        session->get_protocol().add_notice_handler(
+            [this](const xcl::XProtocol *protocol, const bool is_global,
+                   const Mysqlx::Notice::Frame::Type type, const char *data,
+                   const uint32_t data_length) -> xcl::Handler_result {
+              return notice_handler(protocol, is_global, type, data,
+                                    data_length);
+            });
 
-      sessions_[node_id] = std::move(session);
-      sessions_changed_ = true;
+        sessions_[node_id] = std::move(session);
+        sessions_changed_ = true;
+      }
     }
   }
 
@@ -529,14 +538,14 @@ void GRNotificationListener::Impl::reconfigure(
         new std::thread([this]() { listener_thread_func(); }));
 }
 
-GRNotificationListener::GRNotificationListener(const std::string &user_name,
-                                               const std::string &password)
-    : impl_(new Impl(user_name, password)) {}
+GRNotificationListener::GRNotificationListener(
+    const mysqlrouter::UserCredentials &user_credentials)
+    : impl_(new Impl(user_credentials)) {}
 
 GRNotificationListener::~GRNotificationListener() = default;
 
 void GRNotificationListener::setup(
-    const std::vector<metadata_cache::ManagedInstance> &instances,
+    const metadata_cache::ClusterTopology &cluster_topology,
     const NotificationClb &notification_clb) {
-  impl_->reconfigure(instances, notification_clb);
+  impl_->reconfigure(cluster_topology, notification_clb);
 }
