@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2020, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2020, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -104,12 +103,14 @@ stdx::expected<OsslUniquePtr<RSA>, std::error_code> generate_rsa(
   OsslUniquePtr<RSA> rsa{RSA_new()};
   OsslUniquePtr<BIGNUM> bignum{BN_new()};
   if (!rsa || !bignum) {
-    return stdx::unexpected(make_error_code(cert_errc::rsa_generation_failed));
+    return stdx::make_unexpected(
+        make_error_code(cert_errc::rsa_generation_failed));
   }
 
   if (!BN_set_word(bignum.get(), exponent) ||
       !RSA_generate_key_ex(rsa.get(), key_size, bignum.get(), nullptr)) {
-    return stdx::unexpected(make_error_code(cert_errc::rsa_generation_failed));
+    return stdx::make_unexpected(
+        make_error_code(cert_errc::rsa_generation_failed));
   }
 
   return {std::move(rsa)};
@@ -163,7 +164,7 @@ CertificateGenerator::generate_evp_pkey() {
   const unsigned int exponent = RSA_F4;
 
   auto rsa_res = generate_rsa(key_size, exponent);
-  if (!rsa_res) return stdx::unexpected(rsa_res.error());
+  if (!rsa_res) return stdx::make_unexpected(rsa_res.error());
 
   auto rsa = std::move(*rsa_res);
 
@@ -171,7 +172,7 @@ CertificateGenerator::generate_evp_pkey() {
   // pkey takes control over the rsa lifetime, assign_RSA function guarantees
   // that rsa will be freed on pkey destruction
   if (!EVP_PKEY_assign_RSA(pkey.get(), rsa.get())) {
-    return stdx::unexpected(
+    return stdx::make_unexpected(
         make_error_code(cert_errc::evp_pkey_generation_failed));
   }
   (void)rsa.release();
@@ -226,50 +227,54 @@ stdx::expected<X509Cert, std::error_code> CertificateGenerator::generate_x509(
 
   X509Cert cert{X509_new()};
   if (!cert) {
-    return stdx::unexpected(make_error_code(cert_errc::cert_alloc_failed));
+    return stdx::make_unexpected(make_error_code(cert_errc::cert_alloc_failed));
   }
 
   // Set certificate version
   if (!X509_set_version(cert.get(), 2)) {
-    return stdx::unexpected(
+    return stdx::make_unexpected(
         make_error_code(cert_errc::cert_set_version_failed));
   }
 
   // Set serial number
   if (!ASN1_INTEGER_set(X509_get_serialNumber(cert.get()), serial)) {
-    return stdx::unexpected(make_error_code(cert_errc::cert_set_serial_failed));
+    return stdx::make_unexpected(
+        make_error_code(cert_errc::cert_set_serial_failed));
   }
 
   // Set certificate validity
   if (!X509_gmtime_adj(X509_get_notBefore(cert.get()), notbefore) ||
       !X509_gmtime_adj(X509_get_notAfter(cert.get()), notafter)) {
-    return stdx::unexpected(
+    return stdx::make_unexpected(
         make_error_code(cert_errc::cert_set_validity_failed));
   }
 
   // Set public key
   if (!X509_set_pubkey(cert.get(), pkey)) {
-    return stdx::unexpected(
+    return stdx::make_unexpected(
         make_error_code(cert_errc::cert_set_public_key_failed));
   }
 
   // Set CN value in subject
   auto name = X509_get_subject_name(cert.get());
   if (!name) {
-    return stdx::unexpected(make_error_code(cert_errc::cert_set_cn_failed));
+    return stdx::make_unexpected(
+        make_error_code(cert_errc::cert_set_cn_failed));
   }
 
   if (!X509_NAME_add_entry_by_txt(
           name, "CN", MBSTRING_ASC,
           reinterpret_cast<const unsigned char *>(common_name.c_str()), -1, -1,
           0)) {
-    return stdx::unexpected(make_error_code(cert_errc::cert_set_cn_failed));
+    return stdx::make_unexpected(
+        make_error_code(cert_errc::cert_set_cn_failed));
   }
 
   // Set Issuer
   if (!X509_set_issuer_name(cert.get(),
                             ca_cert ? X509_get_subject_name(ca_cert) : name)) {
-    return stdx::unexpected(make_error_code(cert_errc::cert_set_issuer_failed));
+    return stdx::make_unexpected(
+        make_error_code(cert_errc::cert_set_issuer_failed));
   }
 
   // Add X509v3 extensions
@@ -283,14 +288,14 @@ stdx::expected<X509Cert, std::error_code> CertificateGenerator::generate_x509(
                           ca_cert ? const_cast<char *>("critical,CA:FALSE")
                                   : const_cast<char *>("critical,CA:TRUE"))};
   if (!ext) {
-    return stdx::unexpected(
+    return stdx::make_unexpected(
         make_error_code(cert_errc::cert_set_v3_extensions_failed));
   }
   X509_add_ext(cert.get(), ext.get(), -1);
 
   // Sign using SHA256
   if (!X509_sign(cert.get(), ca_cert ? ca_pkey : pkey, EVP_sha256())) {
-    return stdx::unexpected(
+    return stdx::make_unexpected(
         make_error_code(cert_errc::cert_could_not_be_signed));
   }
 
