@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2020, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2020, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,6 +37,7 @@
 
 #include <openssl/ssl.h>  // SSL_CTX
 
+#include "channel.h"
 #include "connection.h"
 #include "mysql/harness/net_ts/buffer.h"
 #include "mysql/harness/net_ts/impl/socket.h"
@@ -49,13 +49,17 @@
 #include "mysql/harness/tls_context.h"
 #include "mysql/harness/tls_error.h"
 #include "mysql/harness/tls_server_context.h"
-#include "mysqlrouter/base_protocol.h"
-#include "mysqlrouter/channel.h"
-#include "mysqlrouter/ssl_mode.h"
 #include "openssl_version.h"
-#include "stdx_expected_no_error.h"
+#include "protocol/base_protocol.h"
+#include "ssl_mode.h"
 #include "test/helpers.h"  // init_test_logger
 
+#define ASSERT_NO_ERROR(x) \
+  ASSERT_THAT((x), ::testing::Truly([](auto const &v) { return (bool)v; }))
+
+#define EXPECT_NO_ERROR(x) \
+  EXPECT_THAT((x), ::testing::Truly([](auto const &v) { return (bool)v; }))
+
 using namespace std::chrono_literals;
 #if 0
 class ClassicProtocolSplicerTest : public ::testing::Test {
@@ -572,7 +576,7 @@ class ProtocolSplicerTest
 
       EXPECT_THAT(connect_res,
                   ::testing::AnyOf(stdx::expected<void, std::error_code>{},
-                                   stdx::unexpected(
+                                   stdx::make_unexpected(
                                        make_error_code(TlsErrc::kWantRead))))
           << connect_res.error().message();
 
