@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2013, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2013, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -88,6 +88,9 @@ extern "C" {
 
 using std::max;
 
+// Test accept this many times
+static constexpr const uint MAX_ACCEPT_RETRY{10};
+
 /** Number of connection errors when selecting on the listening port */
 static std::atomic<ulong> connection_errors_query_block{0};
 
@@ -148,7 +151,9 @@ class Channel_info_local_socket : public Channel_info {
         mysql_socket_vio_new(m_connect_sock, VIO_TYPE_SOCKET, VIO_LOCALHOST);
 #ifdef USE_PPOLL_IN_VIO
     if (vio != nullptr) {
-      vio->thread_id = my_thread_self();
+      // Unset thread_id, to ensure that all shutdowns explicitly set the
+      // current real_id from the THD.
+      vio->thread_id.reset();
       vio->signal_mask = mysqld_signal_mask;
     }
 #endif
@@ -211,7 +216,7 @@ class Channel_info_tcpip_socket : public Channel_info {
     Vio *vio = mysql_socket_vio_new(m_connect_sock, VIO_TYPE_TCPIP, 0);
 #ifdef USE_PPOLL_IN_VIO
     if (vio != nullptr) {
-      vio->thread_id = my_thread_self();
+      vio->thread_id.reset();
       vio->signal_mask = mysqld_signal_mask;
     }
 #endif
@@ -616,12 +621,12 @@ class TCP_socket {
 */
 class Unix_socket {
   std::string m_unix_sockname;  // pathname for socket to bind to.
-  uint m_backlog;  // backlog specifying lenght of pending queue connection.
+  uint m_backlog;  // backlog specifying length of pending queue connection.
   /**
     Create a lockfile which contains the pid of the mysqld instance started
     and pathname as name of unix socket pathname appended with .lock
 
-    @retval   false if lockfile creation is successful else true if lockfile
+    @retval   False if lockfile creation is successful else true if lockfile
               file could not be created.
 
   */
@@ -656,7 +661,9 @@ class Unix_socket {
       return MYSQL_INVALID_SOCKET;
     }
 
-    if (create_lockfile()) {
+    if (create_lockfile() ||
+        DBUG_EVALUATE_IF("simulate_create_socket_lockfile_error", true,
+                         false)) {
       LogErr(ERROR_LEVEL, ER_CONN_UNIX_LOCK_FILE_FAIL);
       return MYSQL_INVALID_SOCKET;
     }
@@ -1321,13 +1328,13 @@ const Listen_socket *Mysqld_socket_listener::get_listen_socket() const {
   if (!m_admin_bind_address.address.empty() &&
       !m_use_separate_thread_for_admin &&
       FD_ISSET(mysql_socket_getfd(m_admin_interface_listen_socket),
-               &m_select_info.m_read_fds)) {
+               const_cast<fd_set *>(&m_select_info.m_read_fds))) {
     return &m_socket_vector[0];
   }
 
   for (const auto &socket_element : m_socket_vector) {
     if (FD_ISSET(mysql_socket_getfd(socket_element.m_socket),
-                 &m_select_info.m_read_fds)) {
+                 const_cast<fd_set *>(&m_select_info.m_read_fds))) {
       return &socket_element;
     }
   }
@@ -1444,28 +1451,36 @@ void Mysqld_socket_listener::close_listener() {
     referenced by the data member m_admin_interface_listen_socket.
   */
   if (m_use_separate_thread_for_admin) {
+    if (admin_socket_thread_id != my_thread_handle{}) {
 #ifdef _WIN32
-    /*
-      For Windows, first close the socket referenced by the data member
-      m_admin_interface_listen_socket. It results in return from select()
-      API call running from a separate thread.
-    */
-    (void)mysql_socket_close(m_admin_interface_listen_socket);
-    my_thread_join(&admin_socket_thread_id, nullptr);
+      /*
+        For Windows, first close the socket referenced by the data member
+        m_admin_interface_listen_socket. It results in return from select()
+        API call running from a separate thread.
+      */
+      (void)mysql_socket_close(m_admin_interface_listen_socket);
+      m_admin_interface_listen_socket.fd = INVALID_SOCKET;
+      my_thread_join(&admin_socket_thread_id, nullptr);
 #else
-    // First, finish listening thread.
-    pthread_kill(admin_socket_thread_id.thread, SIGALRM);
-    my_thread_join(&admin_socket_thread_id, nullptr);
+      // First, finish listening thread.
+      pthread_kill(admin_socket_thread_id.thread, SIGALRM);
+      my_thread_join(&admin_socket_thread_id, nullptr);
+
+#endif
+      mysql_mutex_destroy(&LOCK_start_admin_thread);
+      mysql_cond_destroy(&COND_start_admin_thread);
+    }  // if (admin_socket_thread_id != my_thread_handle{})
     /*
       After a thread listening on admin interface finished, it is safe
-      to close listening socket.
+      to close listening socket. But socket is opened before thread is
+      spawned, so we may need to close the socket even if
+      admin_socket_thread_id has not been assigned.
     */
-    (void)mysql_socket_close(m_admin_interface_listen_socket);
-#endif
-
-    mysql_mutex_destroy(&LOCK_start_admin_thread);
-    mysql_cond_destroy(&COND_start_admin_thread);
-  }
+    if (m_admin_interface_listen_socket.fd != INVALID_SOCKET) {
+      (void)mysql_socket_close(m_admin_interface_listen_socket);
+      m_admin_interface_listen_socket.fd = INVALID_SOCKET;
+    }
+  }  // use_separate_thread_for_admin
 
 #if defined(HAVE_SYS_UN_H)
   if (m_unix_sockname != "" && m_unlink_sockname) {
