@@ -94,6 +94,16 @@ DD_properties::DD_properties() : m_properties() {
                                 upgrade.
       MYSQLD_VERSION_UPGRADED   The server version of the last
                                 completed successful upgrade.
+      MYSQL_VERSION_STABILITY   Stability type of the GA release of the
+                                server version MYSQLD_VERSION. Should be
+                                "LTS" or "INNOVATION". In the source code,
+                                we take a specific action if the value is
+                                "LTS". There is no validation of the value
+                                that is set.
+      SERVER_DOWNGRADE_THRESHOLD Limit for how far back within an LTS
+                                release series we can downgrade.
+      SERVER_UPGRADE_THRESHOLD  The lowest innovation release we can upgrade
+                                to from an LTS release.
   */
   m_property_desc = {
       {"DD_VERSION", Property_type::UNSIGNED_INT_32},
@@ -109,7 +119,10 @@ DD_properties::DD_properties() : m_properties() {
       {"SYSTEM_TABLES", Property_type::PROPERTIES},
       {"UPGRADE_TARGET_SCHEMA", Property_type::CHARACTER_STRING},
       {"UPGRADE_ACTUAL_SCHEMA", Property_type::CHARACTER_STRING},
-      {"MYSQLD_VERSION_UPGRADED", Property_type::UNSIGNED_INT_32}};
+      {"MYSQLD_VERSION_UPGRADED", Property_type::UNSIGNED_INT_32},
+      {"MYSQL_VERSION_STABILITY", Property_type::CHARACTER_STRING},
+      {"SERVER_DOWNGRADE_THRESHOLD", Property_type::UNSIGNED_INT_32},
+      {"SERVER_UPGRADE_THRESHOLD", Property_type::UNSIGNED_INT_32}};
 }
 
 // Read all properties from disk and populate the cache.
