@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -44,11 +44,11 @@ class Weak_object {
   virtual void debug_print(String_type &outb) const = 0;
 
  public:
-  Weak_object() {}
+  Weak_object() = default;
 
   Weak_object(const Weak_object &) = default;
 
-  virtual ~Weak_object() {}
+  virtual ~Weak_object() = default;
 };
 
 ///////////////////////////////////////////////////////////////////////////
