@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,13 +27,12 @@
 #include <math.h>
 #include <string.h>
 #include <algorithm>
-#include <memory>
 
+#include "m_ctype.h"
 #include "my_bitmap.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_table_map.h"
-#include "mysql/strings/m_ctype.h"
 #include "sql/current_thd.h"
 #include "sql/field.h"
 #include "sql/handler.h"
@@ -115,50 +113,6 @@ static void cost_skip_scan(TABLE *table, uint key, uint distinct_key_parts,
                            Cost_estimate *cost_est, ha_rows *records,
                            Item *where_cond, Opt_trace_object *trace_idx);
 
-static bool find_skip_scans(
-    THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
-    enum_order order_direction, bool skip_records_in_range,
-    bool find_all_skip_scans,
-    Mem_root_array<IndexSkipScanParameters *> *skip_scan_info_list);
-
-static AccessPath *make_skip_scan_path(RANGE_OPT_PARAM *param,
-                                       bool force_skip_scan,
-                                       IndexSkipScanParameters *skip_scan_info);
-
-/**
-  Test if skip scan is applicable and if so, construct a new AccessPath for each
-  candidate index skip scan.
-
-  @param  thd       Thread info
-  @param  param     Parameter from test_quick_select
-  @param  tree      Range tree generated by get_mm_tree
-  @param  order_direction The sort order the range access method must be able
-                    to provide. Three-value logic: asc/desc/don't care
-  @param  skip_records_in_range Same value as JOIN_TAB::skip_records_in_range()
-  @param  force_skip_scan  TRUE if skip scan is forced by optimizer hint
-
-  @retval Mem_root_array of candidate INDEX_SKIP_SCAN AccessPaths.
-*/
-Mem_root_array<AccessPath *> get_all_skip_scans(THD *thd,
-                                                RANGE_OPT_PARAM *param,
-                                                SEL_TREE *tree,
-                                                enum_order order_direction,
-                                                bool skip_records_in_range,
-                                                bool force_skip_scan) {
-  Mem_root_array<AccessPath *> skip_scan_paths(param->temp_mem_root);
-  Mem_root_array<IndexSkipScanParameters *> skip_scan_info_list(
-      param->temp_mem_root);
-  AccessPath *cur_path = nullptr;
-  if (!find_skip_scans(thd, param, tree, order_direction, skip_records_in_range,
-                       /*get_all_skip_scans*/ true, &skip_scan_info_list)) {
-    for (IndexSkipScanParameters *iskip_scan : skip_scan_info_list) {
-      cur_path = make_skip_scan_path(param, force_skip_scan, iskip_scan);
-      skip_scan_paths.push_back(cur_path);
-    }
-  }
-  return skip_scan_paths;
-}
-
 /**
   Test if skip scan is applicable and if so, construct a new AccessPath.
 
@@ -197,7 +151,6 @@ Mem_root_array<AccessPath *> get_all_skip_scans(THD *thd,
     AccessPath object, that is later used to construct a new
     IndexSkipScanIterator.
 
-  @param  thd       Thread info
   @param  param     Parameter from test_quick_select
   @param  tree      Range tree generated by get_mm_tree
   @param  order_direction The sort order the range access method must be able
@@ -206,124 +159,13 @@ Mem_root_array<AccessPath *> get_all_skip_scans(THD *thd,
   @param  force_skip_scan  TRUE if skip scan is forced by optimizer hint
 
   @retval NULL, if skip index scan not applicable,
-          otherwise index skip scan access path.
+          otherwise skip index scan table read plan.
 */
+
 AccessPath *get_best_skip_scan(THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
                                enum_order order_direction,
                                bool skip_records_in_range,
                                bool force_skip_scan) {
-  Mem_root_array<IndexSkipScanParameters *> skip_scan_info_list(
-      param->temp_mem_root);
-  AccessPath *skip_scan_path = nullptr;
-  if (!find_skip_scans(thd, param, tree, order_direction, skip_records_in_range,
-                       /*get_all_skip_scans*/ false, &skip_scan_info_list)) {
-    assert(skip_scan_info_list.size() <= 1);
-    for (const IndexSkipScanParameters *iskip_scan : skip_scan_info_list) {
-      IndexSkipScanParameters *skip_scan_info =
-          const_cast<IndexSkipScanParameters *>(iskip_scan);
-      skip_scan_path =
-          make_skip_scan_path(param, force_skip_scan, skip_scan_info);
-    }
-  }
-  return skip_scan_path;
-}
-
-/**
-  Set up ranges for index skip scan
-  - Set flags in skip scan info for null conditions and equality ranges based
-    on the range condition
-  - Alloc storage and set values of min/max range keys
-
-  @param  range_sel_arg  Range condition info
-  @param  param          Parameter from test_quick_select
-  @param  skip_scan_info Info about a single index skip scan
-
-  @retval true on error, otherwise false
-*/
-static bool setup_range_for_skip_scan(SEL_ARG *range_sel_arg,
-                                      RANGE_OPT_PARAM *param,
-                                      IndexSkipScanParameters *skip_scan_info) {
-  skip_scan_info->range_cond_flag = 0;
-  // Setup fields that hold the range condition on key part C.
-  // This is only the suffix of the whole key, that we use
-  // to append to the prefix to get the full key later on.
-  if (!(range_sel_arg->min_flag & NO_MIN_RANGE) &&
-      !(range_sel_arg->max_flag & NO_MAX_RANGE)) {
-    // IS NULL condition
-    if (range_sel_arg->maybe_null() && range_sel_arg->min_value[0] &&
-        range_sel_arg->max_value[0])
-      skip_scan_info->range_cond_flag |= NULL_RANGE;
-    // equality condition
-    else if (memcmp(range_sel_arg->min_value, range_sel_arg->max_value,
-                    skip_scan_info->range_key_part->store_length) == 0)
-      skip_scan_info->range_cond_flag |= EQ_RANGE;
-  }
-
-  uchar *min_value, *max_value;
-  if (range_sel_arg->is_ascending) {
-    skip_scan_info->range_cond_flag |=
-        range_sel_arg->min_flag | range_sel_arg->max_flag;
-    min_value = range_sel_arg->min_value;
-    max_value = range_sel_arg->max_value;
-  } else {
-    skip_scan_info->range_cond_flag |=
-        invert_min_flag(range_sel_arg->min_flag) |
-        invert_max_flag(range_sel_arg->max_flag);
-    min_value = range_sel_arg->max_value;
-    max_value = range_sel_arg->min_value;
-  }
-
-  unsigned range_key_len = skip_scan_info->range_key_part->store_length;
-
-  // Allocate storage for min/max key if they exist.
-  unsigned max_used_key_length = 0;
-  for (uint i = 0; i < skip_scan_info->used_key_parts; ++i) {
-    max_used_key_length += skip_scan_info->index_info->key_part[i].store_length;
-  }
-
-  MEM_ROOT *const mem_root = param->return_mem_root;
-  if (!(skip_scan_info->range_cond_flag & NO_MIN_RANGE)) {
-    skip_scan_info->min_range_key = mem_root->ArrayAlloc<uchar>(range_key_len);
-    if (!skip_scan_info->min_range_key) return true;
-    skip_scan_info->min_search_key =
-        mem_root->ArrayAlloc<uchar>(max_used_key_length);
-    if (!skip_scan_info->min_search_key) return true;
-
-    memcpy(skip_scan_info->min_range_key, min_value, range_key_len);
-  }
-  if (!(skip_scan_info->range_cond_flag & NO_MAX_RANGE)) {
-    skip_scan_info->max_range_key = mem_root->ArrayAlloc<uchar>(range_key_len);
-    if (!skip_scan_info->max_range_key) return true;
-    skip_scan_info->max_search_key =
-        mem_root->ArrayAlloc<uchar>(max_used_key_length);
-    if (!skip_scan_info->max_search_key) return true;
-
-    memcpy(skip_scan_info->max_range_key, max_value, range_key_len);
-  }
-  return false;
-}
-
-/**
-  Collect all possible index skip scans and store them in the
-  skip_scan_info_list.
-
-  @param  thd       Thread info
-  @param  param     Parameter from test_quick_select
-  @param  tree      Range tree generated by get_mm_tree
-  @param  order_direction The sort order the range access method must be able
-                    to provide. Three-value logic: asc/desc/don't care
-  @param  skip_records_in_range Same value as JOIN_TAB::skip_records_in_range()
-  @param  find_all_skip_scans TRUE if every possible skip scan should be added
-  to skip scan list, FALSE if the best skip scan should be added
-  @param[out]  skip_scan_info_list List of index skip scans
-
-  @retval true on error, otherwise false.
-*/
-bool find_skip_scans(
-    THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
-    enum_order order_direction, bool skip_records_in_range,
-    bool find_all_skip_scans,
-    Mem_root_array<IndexSkipScanParameters *> *skip_scan_info_list) {
   JOIN *join = param->query_block->join;
   TABLE *table = param->table;
   const char *cause = nullptr;
@@ -338,8 +180,7 @@ bool find_skip_scans(
 
   if (!join)
     cause = "no_join";
-  else if (param->query_block->leaf_table_count !=
-           1) /* Query must reference one table. */
+  else if (join->primary_tables != 1) /* Query must reference one table. */
     cause = "not_single_table";
   else if (table->s->keys == 0) /* There are no indexes to use. */
     cause = "no_index";
@@ -354,17 +195,17 @@ bool find_skip_scans(
 
   if (cause != nullptr) {
     trace_group.add("chosen", false).add_alnum("cause", cause);
-    return true;
+    return nullptr;
   }
 
-  KEY *best_index_info = nullptr; /* The index chosen for data access. */
-  uint best_index = 0;
-  KEY_PART_INFO *best_range_key_part = nullptr;
-  SEL_ROOT *best_index_range_tree = nullptr;
-  SEL_ARG *best_range_sel_arg = nullptr;
-  uint best_eq_prefix_len = 0;
-  uint best_eq_prefix_key_parts = 0;
-  uint best_used_key_parts = 0;
+  KEY *index_info = nullptr; /* The index chosen for data access. */
+  uint index = 0;
+  KEY_PART_INFO *range_key_part = nullptr;
+  SEL_ROOT *index_range_tree = nullptr;
+  SEL_ARG *range_sel_arg = nullptr;
+  uint eq_prefix_len = 0;
+  uint eq_prefix_key_parts = 0;
+  uint used_key_parts = 0;
   Cost_estimate min_diff_cost;
   Opt_trace_array trace_indices(trace, "potential_skip_scan_indexes");
 
@@ -381,7 +222,7 @@ bool find_skip_scans(
           min_max_item->sum_func() == Item_sum::AVG_DISTINCT_FUNC) {
         trace_group.add("chosen", false)
             .add_alnum("cause", "has_aggregate_distinct");
-        return true;
+        return nullptr;
       }
     }
     has_aggregate_function = true;
@@ -523,44 +364,19 @@ bool find_skip_scans(
 
     min_diff_cost = cur_read_cost;
     min_diff_cost.multiply(DBL_EPSILON);
-
-    if (find_all_skip_scans) {
-      IndexSkipScanParameters *cur_skip_scan_info =
-          new (param->return_mem_root) IndexSkipScanParameters;
-      if (cur_skip_scan_info == nullptr) return true;
-
-      cur_skip_scan_info->index = cur_index;
-      cur_skip_scan_info->index_info = cur_index_info;
-      cur_skip_scan_info->eq_prefix_len = cur_eq_prefix_len;
-      cur_skip_scan_info->eq_prefix_key_parts = cur_eq_prefix_key_parts;
-      cur_skip_scan_info->range_key_part = cur_range_key_part;
-      cur_skip_scan_info->used_key_parts = cur_used_key_parts;
-      cur_skip_scan_info->num_output_rows = cur_records;
-      cur_skip_scan_info->range_key_len = cur_range_key_part->store_length;
-      cur_skip_scan_info->range_part_tracing_only = cur_range_sel_arg->first();
-      cur_skip_scan_info->index_range_tree = cur_index_range_tree;
-      cur_skip_scan_info->has_aggregate_function = has_aggregate_function;
-      cur_skip_scan_info->read_cost = cur_read_cost.total_cost();
-
-      if (!setup_range_for_skip_scan(cur_range_sel_arg, param,
-                                     cur_skip_scan_info)) {
-        skip_scan_info_list->push_back(cur_skip_scan_info);
-      } else {
-        ::destroy_at(cur_skip_scan_info);
-      }
-    } else if (cur_read_cost < (best_read_cost - min_diff_cost)) {
-      best_index = cur_index;
-      best_index_info = cur_index_info;
+    if (cur_read_cost < (best_read_cost - min_diff_cost)) {
+      index_info = cur_index_info;
+      index = cur_index;
       best_read_cost = cur_read_cost;
       best_records = cur_records;
 
-      best_eq_prefix_len = cur_eq_prefix_len;
-      best_eq_prefix_key_parts = cur_eq_prefix_key_parts;
-      best_index_range_tree = cur_index_range_tree;
+      eq_prefix_len = cur_eq_prefix_len;
+      eq_prefix_key_parts = cur_eq_prefix_key_parts;
+      index_range_tree = cur_index_range_tree;
 
-      best_range_sel_arg = cur_range_sel_arg;
-      best_range_key_part = cur_range_key_part;
-      best_used_key_parts = cur_used_key_parts;
+      range_sel_arg = cur_range_sel_arg;
+      range_key_part = cur_range_key_part;
+      used_key_parts = cur_used_key_parts;
     }
   next_index:
     if (cause) {
@@ -568,78 +384,101 @@ bool find_skip_scans(
       cause = nullptr;
     }
   }
-  if (!find_all_skip_scans && best_index_info) {  // save only the best scan
-    IndexSkipScanParameters *best_skip_scan_info =
-        new (param->return_mem_root) IndexSkipScanParameters;
-    if (best_skip_scan_info == nullptr) return true;
-
-    best_skip_scan_info->index = best_index;
-    best_skip_scan_info->index_info = best_index_info;
-    best_skip_scan_info->eq_prefix_len = best_eq_prefix_len;
-    best_skip_scan_info->eq_prefix_key_parts = best_eq_prefix_key_parts;
-    best_skip_scan_info->range_key_part = best_range_key_part;
-    best_skip_scan_info->used_key_parts = best_used_key_parts;
-    best_skip_scan_info->num_output_rows = best_records;
-    best_skip_scan_info->range_key_len = best_range_key_part->store_length;
-    best_skip_scan_info->range_part_tracing_only = best_range_sel_arg->first();
-    best_skip_scan_info->index_range_tree = best_index_range_tree;
-    best_skip_scan_info->has_aggregate_function = has_aggregate_function;
-    best_skip_scan_info->read_cost = best_read_cost.total_cost();
-
-    if (!setup_range_for_skip_scan(best_range_sel_arg, param,
-                                   best_skip_scan_info)) {
-      skip_scan_info_list->push_back(best_skip_scan_info);
-    } else {
-      ::destroy_at(best_skip_scan_info);
-    }
-  }
   trace_indices.end();
-  return false;
-}
 
-/**
-  Construct a new INDEX_SKIP_SCAN AccessPath from the IndexSkipScanParameters
+  if (!index_info) /* No usable index found. */
+    return nullptr;
 
-  @param  param     Parameter from test_quick_select
-  @param  force_skip_scan  TRUE if skip scan is forced by optimizer hint
-  @param  skip_scan_info Parameters of index skip scan
-  @retval INDEX_SKIP_SCAN AccessPath
-*/
-AccessPath *make_skip_scan_path(RANGE_OPT_PARAM *param, bool force_skip_scan,
-                                IndexSkipScanParameters *skip_scan_info) {
-  // The query passes all tests, so construct a new AccessPath.
+  // Setup fields that hold the range condition on key part C.
+  //
+  // This is only the suffix of the whole key, that we use
+  // to append to the prefix to get the full key later on.
+  uint range_cond_flag = 0;
+  if (!(range_sel_arg->min_flag & NO_MIN_RANGE) &&
+      !(range_sel_arg->max_flag & NO_MAX_RANGE)) {
+    // IS NULL condition
+    if (range_sel_arg->maybe_null() && range_sel_arg->min_value[0] &&
+        range_sel_arg->max_value[0])
+      range_cond_flag |= NULL_RANGE;
+    // equality condition
+    else if (memcmp(range_sel_arg->min_value, range_sel_arg->max_value,
+                    range_key_part->store_length) == 0)
+      range_cond_flag |= EQ_RANGE;
+  }
+
+  uchar *min_value, *max_value;
+  if (range_sel_arg->is_ascending) {
+    range_cond_flag |= range_sel_arg->min_flag | range_sel_arg->max_flag;
+    min_value = range_sel_arg->min_value;
+    max_value = range_sel_arg->max_value;
+  } else {
+    range_cond_flag |= invert_min_flag(range_sel_arg->min_flag) |
+                       invert_max_flag(range_sel_arg->max_flag);
+    min_value = range_sel_arg->max_value;
+    max_value = range_sel_arg->min_value;
+  }
+
+  unsigned range_key_len = range_key_part->store_length;
+
+  // Allocate storage for min/max key if they exist.
+  unsigned max_used_key_length = 0;
+  for (uint i = 0; i < used_key_parts; ++i) {
+    max_used_key_length += index_info->key_part[i].store_length;
+  }
+
+  MEM_ROOT *const mem_root = param->return_mem_root;
+  uchar *min_range_key = nullptr;
+  uchar *min_search_key = nullptr;
+  if (!(range_cond_flag & NO_MIN_RANGE)) {
+    if (!(min_range_key = mem_root->ArrayAlloc<uchar>(range_key_len)))
+      return nullptr;
+    if (!(min_search_key = mem_root->ArrayAlloc<uchar>(max_used_key_length)))
+      return nullptr;
+
+    memcpy(min_range_key, min_value, range_key_len);
+  }
+  uchar *max_range_key = nullptr;
+  uchar *max_search_key = nullptr;
+  if (!(range_cond_flag & NO_MAX_RANGE)) {
+    if (!(max_range_key = mem_root->ArrayAlloc<uchar>(range_key_len)))
+      return nullptr;
+    if (!(max_search_key = mem_root->ArrayAlloc<uchar>(max_used_key_length)))
+      return nullptr;
+
+    memcpy(max_range_key, max_value, range_key_len);
+  }
+
+  /* The query passes all tests, so construct a new AccessPath. */
   EQPrefix *eq_prefixes = nullptr;
-  if (skip_scan_info->eq_prefix_key_parts > 0) {
-    eq_prefixes = param->return_mem_root->ArrayAlloc<EQPrefix>(
-        skip_scan_info->eq_prefix_key_parts);
+  if (eq_prefix_key_parts > 0) {
+    eq_prefixes = mem_root->ArrayAlloc<EQPrefix>(eq_prefix_key_parts);
     if (eq_prefixes == nullptr) {
       return nullptr;
     }
 
-    SEL_ROOT *index_range_tree = skip_scan_info->index_range_tree;
     const SEL_ARG *cur_range = index_range_tree->root->first();
     const SEL_ARG *first_range = nullptr;
     const SEL_ROOT *cur_root = index_range_tree;
-    for (uint i = 0; i < skip_scan_info->eq_prefix_key_parts;
+    for (uint i = 0; i < eq_prefix_key_parts;
          i++, cur_range = cur_range->next_key_part->root) {
       eq_prefixes[i].cur_eq_prefix = 0;
       unsigned num_elements = cur_root->elements;
       cur_root = cur_range->next_key_part;
       assert(num_elements > 0);
-      eq_prefixes[i].eq_key_prefixes = Bounds_checked_array<uchar *>::Alloc(
-          param->return_mem_root, num_elements);
+      eq_prefixes[i].eq_key_prefixes =
+          Bounds_checked_array<uchar *>::Alloc(mem_root, num_elements);
 
       uint j = 0;
       first_range = cur_range->first();
       for (cur_range = first_range; cur_range;
            j++, cur_range = cur_range->next) {
-        KEY_PART_INFO *keypart = skip_scan_info->index_info->key_part + i;
+        KEY_PART_INFO *keypart = index_info->key_part + i;
         size_t field_length = keypart->store_length;
         //  Store ranges in the reverse order if key part is descending.
         uint pos = cur_range->is_ascending ? j : num_elements - j - 1;
 
         if (!(eq_prefixes[i].eq_key_prefixes[pos] =
-                  param->return_mem_root->ArrayAlloc<uchar>(field_length)))
+                  mem_root->ArrayAlloc<uchar>(field_length)))
           return nullptr;
 
         if (cur_range->maybe_null() && cur_range->min_value[0] &&
@@ -660,18 +499,31 @@ AccessPath *make_skip_scan_path(RANGE_OPT_PARAM *param, bool force_skip_scan,
 
   AccessPath *path = new (param->return_mem_root) AccessPath;
   path->type = AccessPath::INDEX_SKIP_SCAN;
-  path->set_cost(skip_scan_info->read_cost);
-  path->set_init_cost(0.0);
-  path->set_num_output_rows(skip_scan_info->num_output_rows);
+  path->cost = best_read_cost.total_cost();
+  path->set_num_output_rows(best_records);
 
-  path->index_skip_scan().table = param->table;
-  path->index_skip_scan().index = skip_scan_info->index;
-  path->index_skip_scan().num_used_key_parts = skip_scan_info->used_key_parts;
+  IndexSkipScanParameters *ext =
+      new (param->return_mem_root) IndexSkipScanParameters;
+  ext->index_info = index_info;
+  ext->eq_prefix_len = eq_prefix_len;
+  ext->eq_prefix_key_parts = eq_prefix_key_parts;
+  ext->eq_prefixes = eq_prefixes;
+  ext->range_key_part = range_key_part;
+  ext->min_range_key = min_range_key;
+  ext->max_range_key = max_range_key;
+  ext->min_search_key = min_search_key;
+  ext->max_search_key = max_search_key;
+  ext->range_cond_flag = range_cond_flag;
+  ext->range_key_len = range_key_len;
+  ext->range_part_tracing_only = range_sel_arg->first();
+  ext->index_range_tree = index_range_tree;
+  ext->has_aggregate_function = has_aggregate_function;
+
+  path->index_skip_scan().table = table;
+  path->index_skip_scan().index = index;
+  path->index_skip_scan().num_used_key_parts = used_key_parts;
   path->index_skip_scan().forced_by_hint = force_skip_scan;
-
-  skip_scan_info->eq_prefixes = eq_prefixes;
-  path->index_skip_scan().param =
-      const_cast<IndexSkipScanParameters *>(skip_scan_info);
+  path->index_skip_scan().param = ext;
   return path;
 }
 
