@@ -1,4 +1,4 @@
-/* Copyright (c) 2020, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2020, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -29,7 +29,8 @@
 class Rpl_async_conn_failover_add_managed : public Udf_service_impl {
  private:
   Udf_charset_service m_charset_service;
-  static const std::string m_udf_name;
+  static constexpr const char *m_udf_name =
+      "asynchronous_connection_failover_add_managed";
   bool m_initialized{false};
 
  public:
@@ -85,7 +86,7 @@ class Rpl_async_conn_failover_add_managed : public Udf_service_impl {
                                char *message);
 
   /**
-    Deinitialize variables initalized during init function.
+    Deinitialize variables initialized during init function.
 
     @param[in] init_id     UDF_INIT structure
   */
