@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -387,15 +387,17 @@ int Rpl_info_table::do_reset_info(uint nparam, const char *param_schema,
     table->field[fieldnr]->store(channel_name, strlen(channel_name),
                                  &my_charset_bin);
     uint key_len = key_info->key_part[0].store_length;
-    uchar *key_buf = table->field[fieldnr]->field_ptr();
 
+    uchar key[MAX_KEY_LENGTH];
+    key_copy(key, table->record[0], table->key_info,
+             table->key_info->key_length);
     if (!(handler_error = table->file->ha_index_read_map(
-              table->record[0], key_buf, (key_part_map)1, HA_READ_KEY_EXACT))) {
+              table->record[0], key, (key_part_map)1, HA_READ_KEY_EXACT))) {
       do {
         if ((handler_error = table->file->ha_delete_row(table->record[0])))
           break;
       } while (!(handler_error = table->file->ha_index_next_same(
-                     table->record[0], key_buf, key_len)));
+                     table->record[0], key, key_len)));
       if (handler_error != HA_ERR_END_OF_FILE) error = 1;
     } else {
       /*
@@ -527,7 +529,7 @@ end:
 bool Rpl_info_table::do_count_info(uint nparam, const char *param_schema,
                                    const char *param_table,
                                    MY_BITMAP const *nullable_bitmap,
-                                   uint *counter) {
+                                   ulonglong *counter) {
   int error = 1;
   TABLE *table = nullptr;
   sql_mode_t saved_mode;
@@ -580,6 +582,54 @@ end:
   return error;
 }
 
+std::pair<bool, bool> Rpl_info_table::table_in_use(
+    uint nparam, const char *param_schema, const char *param_table,
+    MY_BITMAP const *nullable_bitmap) {
+  bool error{false};
+  bool empty{false};
+
+  TABLE *table = nullptr;
+  Open_tables_backup backup;
+  Rpl_info_table *info = nullptr;
+
+  DBUG_TRACE;
+
+  if (!(info = new Rpl_info_table(nparam, param_schema, param_table, 0, nullptr,
+                                  nullable_bitmap)))
+    return std::make_pair(true, false);
+
+  THD *thd = info->access->create_thd();
+  sql_mode_t saved_mode = thd->variables.sql_mode;
+
+  /*
+    Opens and locks the rpl_info table before accessing it.
+  */
+  if (!info->access->open_table(thd, to_lex_cstring(info->str_schema),
+                                to_lex_cstring(info->str_table),
+                                info->get_number_info(), TL_READ, &table,
+                                &backup)) {
+    /*
+      Check if table is in use
+    */
+    std::tie(error, empty) = info->access->is_table_in_use(table);
+    if (error) {
+      LogErr(WARNING_LEVEL, ER_RPL_CANT_SCAN_INFO_TABLE, info->str_schema.str,
+             info->str_table.str);
+    }
+  }
+  /* else:
+    We cannot simply print out a warning message at this
+    point because this may represent a bootstrap.
+  */
+
+  // Unlocks and closes the rpl_info table.
+  error = info->access->close_table(thd, table, &backup, error) || error;
+  thd->variables.sql_mode = saved_mode;
+  info->access->drop_thd(thd);
+  delete info;
+  return std::make_pair(error != 0, empty);
+}
+
 void Rpl_info_table::do_end_info() {}
 
 int Rpl_info_table::do_prepare_info_for_read() {
@@ -675,7 +725,7 @@ Rpl_info_handler::enum_field_get_status Rpl_info_table::do_get_info(
 
 Rpl_info_handler::enum_field_get_status Rpl_info_table::do_get_info(
     const int pos, uchar *value, const size_t size,
-    const uchar *default_value MY_ATTRIBUTE((unused))) {
+    const uchar *default_value [[maybe_unused]]) {
   if (bitmap_is_set(&field_values->is_null, pos)) {
     return Rpl_info_handler::enum_field_get_status::FIELD_VALUE_IS_NULL;
   } else {
@@ -740,7 +790,7 @@ Rpl_info_handler::enum_field_get_status Rpl_info_table::do_get_info(
 
 Rpl_info_handler::enum_field_get_status Rpl_info_table::do_get_info(
     const int pos, Server_ids *value,
-    const Server_ids *default_value MY_ATTRIBUTE((unused))) {
+    const Server_ids *default_value [[maybe_unused]]) {
   if (value->unpack_dynamic_ids(field_values->value[pos].c_ptr_safe()))
     return Rpl_info_handler::enum_field_get_status::FAILURE;
 
@@ -758,7 +808,6 @@ bool Rpl_info_table::do_update_is_transactional() {
   Open_tables_backup backup;
 
   DBUG_TRACE;
-  DBUG_EXECUTE_IF("simulate_update_is_transactional_error", { return true; });
 
   THD *thd = access->create_thd();
   saved_mode = thd->variables.sql_mode;
