@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -28,4 +28,4 @@
 #include "sql/select_lex_visitor.h"
 
 /// Gcc demands an implementation for a virtual destructor.
-Select_lex_visitor::~Select_lex_visitor() {}
+Select_lex_visitor::~Select_lex_visitor() = default;
