@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -122,7 +122,7 @@ void comma_maybe(String *str, bool *comma) {
 }
 
 /**
-  Append a key/value pair to a string, with an optional preceeding comma.
+  Append a key/value pair to a string, with an optional preceding comma.
   For numeric values.
 
   @param[in,out]   str                  The string to append to
@@ -190,6 +190,24 @@ void append_auth_id(const THD *thd, const LEX_USER *user, bool comma,
   str->append(STRING_WITH_LEN("@"));
   append_query_string(thd, system_charset_info, &from_host, str);
 }
+
+/**
+  Append the authorization id for the user. This quotes auth_id with ` or "
+  based on the sql_mode set.
+
+  @param [in]       thd     The THD to find the SQL mode
+  @param [in]       user    LEX User to retrieve the plugin string
+  @param [in]       comma   Separator to be prefixed before adding user info
+  @param [in, out]  str     The string in which authID is suffixed
+*/
+void append_auth_id_identifier(const THD *thd, const LEX_USER *user, bool comma,
+                               String *str) {
+  assert(thd);
+  if (comma) str->append(',');
+  append_auth_id_string(thd, user->user.str, user->user.length, user->host.str,
+                        user->host.length, str);
+}
+
 /**
   Used with List<>::sort for alphabetic sorting of LEX_USER records
   using user,host as keys.
@@ -214,7 +232,7 @@ int lex_user_comp(LEX_USER *l1, LEX_USER *l2) {
     return (key > 0 ? 1 : 0);
 }
 /**
-  Util method which does the real rewrite of the SQL statment.
+  Util method which does the real rewrite of the SQL statement.
   If a Rewriter is available for the specified SQL command then
   the rewritten query will be stored in the String rlb; otherwise,
   the string will just be cleared.
@@ -228,7 +246,7 @@ int lex_user_comp(LEX_USER *l1, LEX_USER *l2) {
   @retval        true    If the Query is re-written.
   @retval        false   Otherwise
 */
-bool rewrite_query(THD *thd, Consumer_type type, Rewrite_params *params,
+bool rewrite_query(THD *thd, Consumer_type type, const Rewrite_params *params,
                    String &rlb) {
   DBUG_TRACE;
   std::unique_ptr<I_rewriter> rw = nullptr;
@@ -331,7 +349,7 @@ bool rewrite_query(THD *thd, Consumer_type type, Rewrite_params *params,
                       rewriter.
 */
 void mysql_rewrite_query(THD *thd, Consumer_type type /*= Consumer_type::LOG */,
-                         Rewrite_params *params /*= nullptr*/) {
+                         const Rewrite_params *params /*= nullptr*/) {
   String rlb;
 
   DBUG_TRACE;
@@ -340,7 +358,8 @@ void mysql_rewrite_query(THD *thd, Consumer_type type /*= Consumer_type::LOG */,
   // We should not come through here twice for the same query.
   assert(thd->rewritten_query().length() == 0);
 
-  if (thd->lex->contains_plaintext_password) {
+  if (thd->lex->contains_plaintext_password ||
+      thd->lex->is_rewrite_required()) {
     rewrite_query(thd, type, params, rlb);
     if (rlb.length() > 0) thd->swap_rewritten_query(rlb);
     // The previous rewritten query is in rlb now, which now goes out of scope.
@@ -367,7 +386,7 @@ void mysql_rewrite_query(THD *thd, Consumer_type type /*= Consumer_type::LOG */,
                           If instrumented, the previous
 */
 void mysql_rewrite_acl_query(THD *thd, String &rlb, Consumer_type type,
-                             Rewrite_params *params /* = nullptr */,
+                             const Rewrite_params *params /* = nullptr */,
                              bool do_ps_instrument /* = true */) {
   if (rewrite_query(thd, type, params, rlb) && (rlb.length() > 0) &&
       do_ps_instrument) {
@@ -439,20 +458,22 @@ void Rewriter_user::rewrite_in_memory_user_application_user_metadata(
     const LEX *lex, String *str) const {
   if (lex->alter_user_attribute ==
       enum_alter_user_attribute::ALTER_USER_ATTRIBUTE) {
-    str->append(" ATTRIBUTE '");
+    str->append(" ATTRIBUTE ");
   } else if (lex->alter_user_attribute ==
              enum_alter_user_attribute::ALTER_USER_COMMENT) {
-    str->append(" COMMENT '");
+    str->append(" COMMENT ");
   }
   if (lex->alter_user_attribute !=
       enum_alter_user_attribute::ALTER_USER_COMMENT_NOT_USED) {
-    str->append(lex->alter_user_comment_text);
-    str->append("'");
+    String comment_text(lex->alter_user_comment_text.str,
+                        lex->alter_user_comment_text.length,
+                        system_charset_info);
+    append_query_string(m_thd, system_charset_info, &comment_text, str);
   }
 }
 
 /**
-  Default implementaiton of the the rewriter for user applicatiton
+  Default implementation of the the rewriter for user applicatiton
   user metadata.
   @param [in]       lex    LEX struct to know if the clause was specified
   @param [in, out]  str    The string in which the clause is suffixed
@@ -463,7 +484,7 @@ void Rewriter_create_user::rewrite_user_application_user_metadata(
 }
 
 /**
-  Default implementaiton of the the rewriter for user applicatiton
+  Default implementation of the the rewriter for user applicatiton
   user metadata.
   @param [in]       lex    LEX struct to know if the clause was specified
   @param [in, out]  str    The string in which the clause is suffixed
@@ -489,7 +510,9 @@ void Rewriter_user::append_literal_secret(String *str) const {
 */
 
 void Rewriter_user::append_auth_str(LEX_USER *user, String *str) const {
-  String from_auth(user->auth.str, user->auth.length, system_charset_info);
+  String from_auth(user->first_factor_auth_info.auth.str,
+                   user->first_factor_auth_info.auth.length,
+                   system_charset_info);
   append_query_string(m_thd, system_charset_info, &from_auth, str);
 }
 /**
@@ -655,8 +678,9 @@ void Rewriter_user::rewrite_account_lock_state(LEX *lex, String *str) const {
 void Rewriter_user::append_plugin_name(const LEX_USER *user,
                                        String *str) const {
   str->append(STRING_WITH_LEN(" WITH "));
-  if (user->plugin.length > 0) {
-    String from_plugin(user->plugin.str, user->plugin.length,
+  if (user->first_factor_auth_info.plugin.length > 0) {
+    String from_plugin(user->first_factor_auth_info.plugin.str,
+                       user->first_factor_auth_info.plugin.length,
                        system_charset_info);
     append_query_string(m_thd, system_charset_info, &from_plugin, str);
   } else {
@@ -666,6 +690,41 @@ void Rewriter_user::append_plugin_name(const LEX_USER *user,
     append_query_string(m_thd, system_charset_info, &default_plugin, str);
   }
 }
+
+/**
+  Append the authentication plugin name from LEX_MFA for the user
+
+  @param [in]       user    User to retrieve the plugin string
+  @param [in, out]  str     The string in which plugin info is suffixed
+*/
+void Rewriter_user::append_mfa_plugin_name(const LEX_MFA *user,
+                                           String *str) const {
+  if (user->plugin.length > 0) {
+    str->append(STRING_WITH_LEN(" WITH "));
+    String from_plugin(user->plugin.str, user->plugin.length,
+                       system_charset_info);
+    append_query_string(m_thd, system_charset_info, &from_plugin, str);
+  }
+}
+
+/**
+  Append the authentication string from LEX_MFA for the user
+
+  @param [in]       user    User to retrieve the plugin string
+  @param [in, out]  str     The string in which plugin info is suffixed
+*/
+void Rewriter_user::append_mfa_auth_str(const LEX_MFA *user,
+                                        String *str) const {
+  if (user->uses_identified_by_clause &&
+      m_consumer_type == Consumer_type::TEXTLOG) {
+    str->append(STRING_WITH_LEN(" BY "));
+    append_literal_secret(str);
+  } else if (user->auth.length > 0) {
+    str->append(STRING_WITH_LEN(" AS "));
+    String auth_str(user->auth.str, user->auth.length, system_charset_info);
+    append_query_string(m_thd, system_charset_info, &auth_str, str);
+  }
+}
 /**
    The default implementation is to append the PASSWORD HISTORY clause iff it
    is specified. Though concrete classes may add their own implementation.
@@ -772,23 +831,55 @@ bool Rewriter_create_user::rewrite(String &rlb) const {
 void Rewriter_create_user::append_user_auth_info(LEX_USER *user, bool comma,
                                                  String *str) const {
   append_auth_id(m_thd, user, comma, str);
-  if (user->uses_identified_by_clause || user->uses_identified_with_clause ||
-      user->uses_authentication_string_clause ||
+  List_iterator<LEX_MFA> mfa_list(user->mfa_list);
+  LEX_MFA *tmp_mfa;
+  if (user->with_initial_auth) {
+    /*
+      In case of passwordless user, server fiddles with user specified syntax,
+      thus we are expected to write the original syntax.
+      ex:
+      CREATE USER foo IDENTIFIED WITH authentication_fido INITIAL AUTHENTICATION
+         IDENTIFIED BY 'abc';
+      above sql is converted by server as:
+      CREATE USER foo IDENTIFIED BY 'abc' AND IDENTIFIED WITH
+      authentication_fido;
+
+      This block ensures that query is rewritten in logs as:
+      CREATE USER foo IDENTIFIED WITH authentication_fido INITIAL AUTHENTICATION
+         IDENTIFIED WITH <default auth plugin> AS <auth_hash_string>;
+    */
+    assert(user->mfa_list.size());
+    /* point to 2nd factor which is authentication_fido */
+    tmp_mfa = mfa_list++;
+    str->append(STRING_WITH_LEN(" IDENTIFIED"));
+    append_mfa_plugin_name(tmp_mfa, str);
+    str->append(STRING_WITH_LEN(" INITIAL AUTHENTICATION"));
+  }
+  if (user->first_factor_auth_info.uses_identified_by_clause ||
+      user->first_factor_auth_info.uses_identified_with_clause ||
+      user->first_factor_auth_info.uses_authentication_string_clause ||
       m_consumer_type == Consumer_type::BINLOG) {
     str->append(STRING_WITH_LEN(" IDENTIFIED"));
-    if (user->uses_identified_with_clause ||
+    if (user->first_factor_auth_info.uses_identified_with_clause ||
         m_consumer_type == Consumer_type::BINLOG)
       append_plugin_name(user, str);
 
-    if (user->uses_identified_by_clause &&
+    if (user->first_factor_auth_info.uses_identified_by_clause &&
         m_consumer_type == Consumer_type::TEXTLOG) {
       str->append(STRING_WITH_LEN(" BY "));
       append_literal_secret(str);
-    } else if (user->auth.length > 0) {
+    } else if (user->first_factor_auth_info.auth.length > 0) {
       str->append(STRING_WITH_LEN(" AS "));
       append_auth_str(user, str);
     }
   }
+  if (user->mfa_list.size() && !user->with_initial_auth) {
+    while ((tmp_mfa = mfa_list++)) {
+      str->append(STRING_WITH_LEN(" AND IDENTIFIED"));
+      append_mfa_plugin_name(tmp_mfa, str);
+      append_mfa_auth_str(tmp_mfa, str);
+    }
+  }
 }
 /**
   Append the PASSWORD HISTORY clause for users iff it is specified
@@ -847,14 +938,15 @@ bool Rewriter_alter_user::rewrite(String &rlb) const {
 void Rewriter_alter_user::append_user_auth_info(LEX_USER *user, bool comma,
                                                 String *str) const {
   append_auth_id(m_thd, user, comma, str);
-  if (user->uses_identified_by_clause || user->uses_identified_with_clause ||
-      user->uses_authentication_string_clause) {
+  if (user->first_factor_auth_info.uses_identified_by_clause ||
+      user->first_factor_auth_info.uses_identified_with_clause ||
+      user->first_factor_auth_info.uses_authentication_string_clause) {
     str->append(STRING_WITH_LEN(" IDENTIFIED"));
-    if (user->uses_identified_with_clause ||
+    if (user->first_factor_auth_info.uses_identified_with_clause ||
         m_consumer_type == Consumer_type::BINLOG)
       append_plugin_name(user, str);
 
-    if (user->uses_identified_by_clause &&
+    if (user->first_factor_auth_info.uses_identified_by_clause &&
         m_consumer_type == Consumer_type::TEXTLOG) {
       str->append(STRING_WITH_LEN(" BY "));
       append_literal_secret(str);
@@ -862,7 +954,7 @@ void Rewriter_alter_user::append_user_auth_info(LEX_USER *user, bool comma,
       if (user->uses_replace_clause) {
         str->append(STRING_WITH_LEN(" REPLACE <secret>"));
       }
-    } else if (user->auth.length > 0) {
+    } else if (user->first_factor_auth_info.auth.length > 0) {
       str->append(STRING_WITH_LEN(" AS "));
       append_auth_str(user, str);
     }
@@ -871,6 +963,44 @@ void Rewriter_alter_user::append_user_auth_info(LEX_USER *user, bool comma,
       str->append(STRING_WITH_LEN(" RETAIN CURRENT PASSWORD"));
     }
   }
+  List_iterator<LEX_MFA> mfa_list(user->mfa_list);
+  if (user->mfa_list.size()) {
+    LEX_MFA *tmp_mfa;
+    while ((tmp_mfa = mfa_list++)) {
+      if (tmp_mfa->unregister) {
+        str->append(" ");
+        str->append(std::to_string(tmp_mfa->nth_factor).c_str());
+        str->append(STRING_WITH_LEN(" FACTOR UNREGISTER"));
+      } else if (tmp_mfa->finish_registration || tmp_mfa->modify_factor) {
+        /*
+          ALTER USER foo Nth FACTOR FINISH REGISTRATION;
+          is binlogged as
+          ALTER USER foo MODIFY Nth FACTOR IDENTIFIED WITH <plugin>
+             AS <auth-str>;
+        */
+        str->append(STRING_WITH_LEN(" MODIFY "));
+        str->append(std::to_string(tmp_mfa->nth_factor).c_str());
+        str->append(STRING_WITH_LEN(" FACTOR"));
+        str->append(STRING_WITH_LEN(" IDENTIFIED"));
+        append_mfa_plugin_name(tmp_mfa, str);
+        append_mfa_auth_str(tmp_mfa, str);
+      } else if (tmp_mfa->drop_factor) {
+        str->append(STRING_WITH_LEN(" DROP "));
+        str->append(std::to_string(tmp_mfa->nth_factor).c_str());
+        str->append(STRING_WITH_LEN(" FACTOR"));
+      } else if (tmp_mfa->add_factor) {
+        str->append(STRING_WITH_LEN(" ADD "));
+        str->append(std::to_string(tmp_mfa->nth_factor).c_str());
+        str->append(STRING_WITH_LEN(" FACTOR"));
+        if (tmp_mfa->uses_identified_by_clause ||
+            tmp_mfa->uses_identified_with_clause) {
+          str->append(STRING_WITH_LEN(" IDENTIFIED"));
+          append_mfa_plugin_name(tmp_mfa, str);
+          append_mfa_auth_str(tmp_mfa, str);
+        }
+      }
+    }
+  }
 
   if (user->discard_old_password) {
     str->append(STRING_WITH_LEN(" DISCARD OLD PASSWORD"));
@@ -900,11 +1030,10 @@ void Rewriter_alter_user::rewrite_password_reuse(const LEX *lex,
     parent::rewrite_password_reuse(lex, str);
   }
 }
-Rewriter_show_create_user::Rewriter_show_create_user(THD *thd,
-                                                     Consumer_type type,
-                                                     Rewrite_params *params)
+Rewriter_show_create_user::Rewriter_show_create_user(
+    THD *thd, Consumer_type type, const Rewrite_params *params)
     : Rewriter_user(thd, type),
-      show_params_(dynamic_cast<Show_user_params *>(params)) {}
+      show_params_(dynamic_cast<const Show_user_params *>(params)) {}
 
 /**
   Rewrite the query for the SHOW CREATE USER statement.
@@ -926,14 +1055,14 @@ bool Rewriter_show_create_user::rewrite(String &rlb) const {
 }
 
 /**
-  Overrides implementaiton of the the rewriter for user application
+  Overrides implementation of the the rewriter for user application
   user metadata. This is needed because we have to read the
   ATTRIBUTE data from disk.
   @param [in]       lex    LEX struct to know if the clause was specified
   @param [in, out]  str    The string in which the clause is suffixed
 */
 void Rewriter_show_create_user::rewrite_user_application_user_metadata(
-    const LEX *lex MY_ATTRIBUTE((unused)), String *str) const {
+    const LEX *lex [[maybe_unused]], String *str) const {
   /* Only show the ATTRIBUTE operator if there's any attribute to show. */
   if (show_params_->metadata_str->length() > 0) {
     str->append(" ATTRIBUTE '");
@@ -953,16 +1082,22 @@ void Rewriter_show_create_user::rewrite_user_application_user_metadata(
 */
 void Rewriter_show_create_user::append_auth_str(LEX_USER *user,
                                                 String *str) const {
-  String from_auth(user->auth.str, user->auth.length, system_charset_info);
+  String from_auth(user->first_factor_auth_info.auth.str,
+                   user->first_factor_auth_info.auth.length,
+                   system_charset_info);
 
   if (show_params_ && show_params_->print_identified_with_as_hex_ &&
-      user->auth.length) {
-    for (const char *c = user->auth.str;
-         static_cast<size_t>(c - user->auth.str) < user->auth.length; c++) {
+      user->first_factor_auth_info.auth.length) {
+    for (const char *c = user->first_factor_auth_info.auth.str;
+         static_cast<size_t>(c - user->first_factor_auth_info.auth.str) <
+         user->first_factor_auth_info.auth.length;
+         c++) {
       if (!my_isgraph(system_charset_info, *c)) {
-        from_auth.alloc(user->auth.length * 2 + 3);
-        str_to_hex(from_auth.c_ptr_quick(), user->auth.str, user->auth.length);
-        from_auth.length(user->auth.length * 2 + 2);
+        from_auth.alloc(user->first_factor_auth_info.auth.length * 2 + 3);
+        str_to_hex(from_auth.c_ptr_quick(),
+                   user->first_factor_auth_info.auth.str,
+                   user->first_factor_auth_info.auth.length);
+        from_auth.length(user->first_factor_auth_info.auth.length * 2 + 2);
         str->append(from_auth);
 
         return;
@@ -1001,11 +1136,23 @@ void Rewriter_show_create_user::rewrite_password_reuse(const LEX *lex,
 void Rewriter_show_create_user::append_user_auth_info(LEX_USER *user,
                                                       bool comma,
                                                       String *str) const {
-  append_auth_id(m_thd, user, comma, str);
+  append_auth_id_identifier(m_thd, user, comma, str);
+  List_iterator<LEX_MFA> mfa_list(user->mfa_list);
+  LEX_MFA *tmp_mfa;
+  bool is_passwordless = false;
+  if (user->mfa_list.size()) {
+    tmp_mfa = mfa_list++;
+    is_passwordless = tmp_mfa->passwordless;
+    if (is_passwordless) {
+      str->append(STRING_WITH_LEN(" IDENTIFIED"));
+      append_mfa_plugin_name(tmp_mfa, str);
+      str->append(STRING_WITH_LEN(" INITIAL AUTHENTICATION"));
+    }
+  }
   assert(m_thd->lex->contains_plaintext_password == false);
   str->append(STRING_WITH_LEN(" IDENTIFIED"));
   append_plugin_name(user, str);
-  if (user->auth.length > 0) {
+  if (user->first_factor_auth_info.auth.length > 0) {
     str->append(STRING_WITH_LEN(" AS "));
     if (show_params_ && show_params_->hide_password_hash) {
       append_literal_secret(str);
@@ -1013,6 +1160,21 @@ void Rewriter_show_create_user::append_user_auth_info(LEX_USER *user,
       append_auth_str(user, str);
     }
   }
+  mfa_list = user->mfa_list;
+  if (user->mfa_list.size() && !is_passwordless) {
+    while ((tmp_mfa = mfa_list++)) {
+      str->append(STRING_WITH_LEN(" AND IDENTIFIED WITH "));
+      String plugin_str(tmp_mfa->plugin.str, tmp_mfa->plugin.length,
+                        system_charset_info);
+      append_query_string(m_thd, system_charset_info, &plugin_str, str);
+      if (tmp_mfa->auth.length > 0) {
+        str->append(STRING_WITH_LEN(" AS "));
+        String auth_str(tmp_mfa->auth.str, tmp_mfa->auth.length,
+                        system_charset_info);
+        append_query_string(m_thd, system_charset_info, &auth_str, str);
+      }
+    }
+  }
 }
 
 Rewriter_set::Rewriter_set(THD *thd, Consumer_type type)
@@ -1027,24 +1189,28 @@ Rewriter_set::Rewriter_set(THD *thd, Consumer_type type)
   @retval        false   otherwise
 */
 bool Rewriter_set::rewrite(String &rlb) const {
+  String local_rlb;
   LEX *lex = m_thd->lex;
   List_iterator_fast<set_var_base> it(lex->var_list);
   set_var_base *var;
   bool comma = false;
 
-  rlb.append(STRING_WITH_LEN("SET "));
+  local_rlb.append(STRING_WITH_LEN("SET "));
 
   while ((var = it++)) {
-    comma_maybe(&rlb, &comma);
-    var->print(m_thd, &rlb);
+    comma_maybe(&local_rlb, &comma);
+    if (var->print(m_thd, &local_rlb) == false) {
+      return false;
+    }
   }
+  rlb.takeover(local_rlb);
   return true;
 }
 
 Rewriter_set_password::Rewriter_set_password(THD *thd, Consumer_type type,
-                                             Rewrite_params *params)
+                                             const Rewrite_params *params)
     : Rewriter_set(thd, type) {
-  User_params *param = dynamic_cast<User_params *>(params);
+  const User_params *param = dynamic_cast<const User_params *>(params);
   if (param) m_users = param->users;
 }
 
@@ -1079,7 +1245,9 @@ bool Rewriter_set_password::rewrite(String &rlb) const {
                         system_charset_info);
 #endif
     } else {
-      auth_str = String(user->auth.str, user->auth.length, system_charset_info);
+      auth_str =
+          String(user->first_factor_auth_info.auth.str,
+                 user->first_factor_auth_info.auth.length, system_charset_info);
     }
     /*
       Construct :
@@ -1090,7 +1258,7 @@ bool Rewriter_set_password::rewrite(String &rlb) const {
     rlb.append(STRING_WITH_LEN("@"));
     append_query_string(m_thd, system_charset_info, &current_host, &rlb);
     rlb.append(STRING_WITH_LEN(" IDENTIFIED WITH '"));
-    rlb.append(user->plugin.str);
+    rlb.append(user->first_factor_auth_info.plugin.str);
     rlb.append(STRING_WITH_LEN("' AS "));
     append_query_string(m_thd, system_charset_info, &auth_str, &rlb);
     if (user->retain_current_password)
@@ -1104,9 +1272,9 @@ bool Rewriter_set_password::rewrite(String &rlb) const {
 }
 
 Rewriter_grant::Rewriter_grant(THD *thd, Consumer_type type,
-                               Rewrite_params *params)
+                               const Rewrite_params *params)
     : I_rewriter(thd, type) {
-  grant_params = dynamic_cast<Grant_params *>(params);
+  grant_params = dynamic_cast<const Grant_params *>(params);
 }
 
 /**
@@ -1119,7 +1287,7 @@ Rewriter_grant::Rewriter_grant(THD *thd, Consumer_type type,
 bool Rewriter_grant::rewrite(String &rlb) const {
   LEX *lex = m_thd->lex;
 
-  TABLE_LIST *first_table = lex->query_block->table_list.first;
+  Table_ref *first_table = lex->query_block->get_table_list();
   bool proxy_grant = lex->type == TYPE_ENUM_PROXY;
   String cols(1024);
   int c;
@@ -1244,7 +1412,7 @@ bool Rewriter_grant::rewrite(String &rlb) const {
     AS ... clause is added in following cases
     1. User has explicitly executed GRANT ... AS ...
        In this case we write it as it.
-    2. --partial_revokes is ON and we are rewritting
+    2. --partial_revokes is ON and we are rewriting
        GRANT for binary log.
   */
   if (grant_params != nullptr) {
