@@ -71,8 +71,6 @@ Place, Suite 330, Boston, MA 02111-1307 USA
 #include "fsp0fsp.h"
 #include "xb_regex.h"
 
-extern bool opt_no_server_version_check;
-
 /** Possible values for system variable "innodb_checksum_algorithm". */
 extern const char *innodb_checksum_algorithm_names[];
 
@@ -370,7 +368,7 @@ static bool check_server_version(unsigned long version_number,
   }
 
   version_supported =
-      version_supported || (version_number > 80000 && version_number < 90000);
+      version_supported || (version_number > 80000 && version_number <= 80099);
 
   mysql51 = version_number > 50100 && version_number < 50500;
   pxb24 = pxb24 || (mysql51 && innodb_version != NULL);
@@ -392,24 +390,10 @@ static bool check_server_version(unsigned long version_number,
     if (pxb24) {
       xb::info() << "Please use Percona XtraBackup 2.4 for this database.";
     }
+    return false;
   }
 
-  auto pxb_base_ver =
-      xtrabackup::utils::get_version_number(MYSQL_SERVER_VERSION);
-
-  DBUG_EXECUTE_IF("simulate_lower_version", pxb_base_ver = 80014;);
-
-  if (!opt_no_server_version_check && pxb_base_ver < version_number) {
-    xb::error() << "Unsupported server version "
-                << mysql_connection->server_version;
-    xb::error()
-        << "Please upgrade PXB, if a new "
-           "version is available. To continue with risk, use the option "
-           "--no-server-version-check.";
-    version_supported = false;
-  }
-
-  return (version_supported);
+  return true;
 }
 
 /*********************************************************************/ /**
@@ -2153,13 +2137,21 @@ void mdl_lock_tables() {
       if (fsp_is_ibd_tablespace(atoi(row[1]))) {
         char full_table_name[MAX_FULL_NAME_LEN + 1];
         innobase_format_name(full_table_name, sizeof(full_table_name), row[0]);
+        if (check_if_skip_table(row[0]) &&
+            !fsp_is_dd_tablespace(atoi(row[1]))) {
+          xb::info() << full_table_name << " match an exclude rule. Skipping";
+          continue;
+        }
         if (is_fts_index(full_table_name)) {
           // We will eventually get to the row to lock the main table
           xb::info() << full_table_name << " is a Full-Text Index. Skipping";
           continue;
-        } else if (is_tmp_table(full_table_name)) {
+        } else if (is_tmp_table(full_table_name) ||
+                   (is_access_blocked(full_table_name) &&
+                    fsp_is_dd_tablespace(atoi(row[1])))) {
           // We cannot run SELECT ... #sql-; Skipped to avoid invalid query.
-          xb::info() << full_table_name << " is a temporary table. Skipping";
+          xb::info() << full_table_name
+                     << " is a temporary table or internal. Skipping";
           continue;
         }
 
@@ -2184,41 +2176,47 @@ void mdl_unlock_all() {
     mysql_close(mdl_con);
   }
 }
-bool is_fts_index(const std::string &tablespace) {
+bool is_fts_index(const std::string &table_name) {
   const char *pattern =
       "^(FTS|fts)_[0-9a-fA-f]{16}_(([0-9a-fA-]{16}_(INDEX|index)_[1-6])|"
       "DELETED_CACHE|deleted_cache|DELETED|deleted|CONFIG|config|BEING_DELETED|"
-      "being_deleted|BEING_DELETED_CACHE|beign_deleted_cache)$";
+      "being_deleted|BEING_DELETED_CACHE|being_deleted_cache)$";
   const char *error_context = "is_fts_index";
-  return check_regexp_table_name(tablespace, error_context, pattern);
+  return check_regexp_table_name(table_name, error_context, pattern);
 }
 
-bool is_tmp_table(const std::string &tablespace) {
+bool is_tmp_table(const std::string &table_name) {
   const char *pattern = "^#sql";
   const char *error_context = "is_tmp_table";
-  return check_regexp_table_name(tablespace, error_context, pattern);
+  return check_regexp_table_name(table_name, error_context, pattern);
 }
 
-bool check_regexp_table_name(std::string tablespace, const char *error_context,
+bool is_access_blocked(const std::string &table_name) {
+  const char *pattern = "compression_dictionary";
+  const char *error_context = "is_blocked_table";
+  return check_regexp_table_name(table_name, error_context, pattern);
+}
+
+bool check_regexp_table_name(std::string table_name, const char *error_context,
                              const char *pattern) {
   bool result = false;
-  get_table_name_from_tablespace(tablespace);
+  get_table_name_from_fq(table_name);
   xb_regex_t preg;
   size_t nmatch = 1;
   xb_regmatch_t pmatch[1];
   compile_regex(pattern, error_context, &preg);
 
-  if (xb_regexec(&preg, tablespace.c_str(), nmatch, pmatch, 0) != REG_NOMATCH) {
+  if (xb_regexec(&preg, table_name.c_str(), nmatch, pmatch, 0) != REG_NOMATCH) {
     result = true;
   }
   xb_regfree(&preg);
   return result;
 }
-void get_table_name_from_tablespace(std::string &tablespace) {
-  std::size_t pos = tablespace.find("`.`");  //`db`.`table` separator
+void get_table_name_from_fq(std::string &fq_table_name) {
+  std::size_t pos = fq_table_name.find("`.`");  //`db`.`table` separator
   if (pos != std::string::npos) {
-    tablespace = tablespace.substr(pos + 3);
-    tablespace.erase(tablespace.size() - 1);  // remove leading `
+    fq_table_name = fq_table_name.substr(pos + 3);
+    fq_table_name.erase(fq_table_name.size() - 1);  // remove leading `
   }
 }
 
