@@ -130,14 +130,14 @@ xb_fil_cur_result_t xb_fil_cur_open(
   cursor->is_system = fsp_is_system_or_temp_tablespace(node->space->id);
   cursor->is_ibd = fsp_is_ibd_tablespace(node->space->id);
 
-  strncpy(cursor->abs_path, node->name, sizeof(cursor->abs_path));
+  strncpy(cursor->abs_path, node->name, sizeof(cursor->abs_path) - 1);
 
   /* Get the relative path for the destination tablespace name, i.e. the
   one that can be appended to the backup root directory. Non-system
   tablespaces may have absolute paths for remote tablespaces in MySQL
   5.6+. We want to make "local" copies for the backup. */
   strncpy(cursor->rel_path, xb_get_relative_path(node->space, cursor->abs_path),
-          sizeof(cursor->rel_path));
+          sizeof(cursor->rel_path) - 1);
 
   /* In the backup mode we should already have a tablespace handle created
   by fil_load_single_table_tablespace() unless it is a system
