@@ -1,6 +1,33 @@
+/******************************************************
+Copyright (c) 2021-2023 Percona LLC and/or its affiliates.
+
+Streaming implementation for XtraBackup.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; version 2 of the License.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
+
+*******************************************************/
+
 #include "file_utils.h"
 #include <mysql/service_mysql_alloc.h>
+#ifdef __APPLE__
+#include <sys/event.h>
+#else
+#include <sys/epoll.h>
+#endif
+#include <thread>
 #include "common.h"
+#include "ds_fifo.h"
 #include "msg.h"
 #include "my_dir.h"
 #include "my_io.h"
@@ -43,6 +70,11 @@ inline page_type_t fil_page_get_type(const byte *page) {
   return (static_cast<page_type_t>(mach_read_from_2(page + XB_FIL_PAGE_TYPE)));
 }
 
+inline bool is_page_io_compressed(const byte *page) {
+  return (fil_page_get_type(page) == XB_FIL_PAGE_COMPRESSED ||
+          fil_page_get_type(page) == XB_FIL_PAGE_COMPRESSED_AND_ENCRYPTED);
+}
+
 /** Calculates the smallest multiple of m that is not smaller than n
  when m is a power of two.  In other words, rounds n up to m * k.
  @param n in: number to round up
@@ -192,14 +224,14 @@ bool datafile_open(const char *file, datafile_cur_t *cursor, bool read_only,
                    uint buffer_size) {
   memset(cursor, 0, sizeof(datafile_cur_t));
 
-  strncpy(cursor->abs_path, file, sizeof(cursor->abs_path));
+  strncpy(cursor->abs_path, file, sizeof(cursor->abs_path) - 1);
 
   /* Get the relative path for the destination tablespace name, i.e. the
   one that can be appended to the backup root directory. Non-system
   tablespaces may have absolute paths for remote tablespaces in MySQL
   5.6+. We want to make "local" copies for the backup. */
   strncpy(cursor->rel_path, get_relative_path(cursor->abs_path),
-          sizeof(cursor->rel_path));
+          sizeof(cursor->rel_path) - 1);
 
   cursor->fd =
       my_open(cursor->abs_path, (read_only) ? O_RDONLY : O_RDWR, MYF(MY_WME));
@@ -249,8 +281,42 @@ xb_fil_cur_result_t datafile_read(datafile_cur_t *cursor) {
   return (XB_FIL_CUR_SUCCESS);
 }
 
+/** Check if an IBD file is IO compressed or not. We just check page numbers
+1 & 2. This is because the first 7 pages are created when an IBD file is
+created. This is the minimum size of a tablespace. So it is sufficient to check
+only two pages.
+Note that page 0 is not compressed. So we check page number 1 & 2 only. */
+class PageCompressionTracker {
+ public:
+  explicit PageCompressionTracker(ulint page_size)
+      : page_size_{page_size},
+        is_compressed_{false, false},
+        checked_exit_{false} {}
+
+  // Call this for every page
+  bool record(ulint buf_offset, const byte *page) {
+    const ulint page_num = buf_offset / page_size_;
+
+    if (page_num == 1 || page_num == 2) {
+      is_compressed_[page_num - 1] = is_page_io_compressed(page);
+    }
+
+    if (page_num == 3 && !checked_exit_) {
+      checked_exit_ = true;  // Ensure we only check once
+      return !is_compressed_[0] && !is_compressed_[1];  // true = can exit early
+    }
+
+    return false;
+  }
+
+ private:
+  const ulint page_size_;
+  bool is_compressed_[2];  // 0 => page 1, 1 => page 2
+  bool checked_exit_;      // ensures we only check once at page 3
+};
+
 bool restore_sparseness(const char *src_file_path, uint buffer_size,
-                        char error[512]) {
+                        char error[512], bool opt_verbose) {
   datafile_cur_t cursor;
   size_t page_size = 0;
   size_t seek = 0;
@@ -261,7 +327,7 @@ bool restore_sparseness(const char *src_file_path, uint buffer_size,
     return false;
   }
   auto punch_hole_func = [&](const auto page) {
-    if (fil_page_get_type(page) == XB_FIL_PAGE_COMPRESSED) {
+    if (is_page_io_compressed(page)) {
 #ifdef UNIV_DEBUG
       assert(page_size % (size_t)cursor.statinfo.st_blksize == 0);
 #endif
@@ -304,8 +370,20 @@ bool restore_sparseness(const char *src_file_path, uint buffer_size,
 #endif
     }
 
+    PageCompressionTracker compression_tracker(page_size);
+
     for (ulint i = 0; i < cursor.buf_read / page_size; ++i) {
       const auto page = cursor.buf + buf_offset;
+
+      if (compression_tracker.record(buf_offset, page)) {
+        if (opt_verbose) {
+          msg("File %s is not sparse, skipping restore_sparseness\n",
+              src_file_path);
+        }
+        datafile_close(&cursor);
+        return true;
+      }
+
       if (!punch_hole_func(page)) return false;
 
       buf_offset += page_size;
@@ -315,3 +393,99 @@ bool restore_sparseness(const char *src_file_path, uint buffer_size,
   datafile_close(&cursor);
   return true;
 }
+
+File open_fifo_for_write_with_timeout(const char *path, uint timeout) {
+  File fd;
+  uint attempt = 0;
+  do {
+    fd = my_open(path, O_WRONLY | O_NONBLOCK, MYF(0));
+    if (fd < 0) {
+      attempt++;
+      std::this_thread::sleep_for(std::chrono::seconds(1));
+    }
+
+  } while (fd < 0 && attempt < timeout);
+  if (fd < 0) {
+    return -1;
+  }
+  /* Adjust file flags to blocking - we are in sync mode now. */
+  if (fcntl(fd, F_SETFL, O_WRONLY) == -1) {
+    my_close(fd, MYF(0));
+    return -1;
+  }
+
+  /*
+   * Signals the read side by writing DS_FIFO_CONTROL_CHAR
+   * This triggers EVFILT_READ/EPOLLIN. This is necessary because
+   * the read side is also opened in non-blocking mode. We change
+   * back to blocking node once kqueue/epoll is notified. The way
+   * to notify that data is available to read is to write some data
+   * to it.
+   */
+  my_write(fd, (uchar *)DS_FIFO_CONTROL_CHAR, 1, MYF(0));
+  return fd;
+}
+
+File open_fifo_for_read_with_timeout(const char *path, uint timeout) {
+  int fd;
+  uint attempt = 0;
+  do {
+    fd = my_open(path, O_RDONLY | O_NONBLOCK, MYF(0));
+    if (fd < 0) {
+      attempt++;
+      std::this_thread::sleep_for(std::chrono::seconds(1));
+    }
+  } while (fd < 0 && attempt < timeout);
+
+  if (fd < 0) {
+    return -1;
+  }
+
+  /* File was open, lets check its open on the other side */
+#ifdef __APPLE__
+  struct timespec tm = {timeout, 0};
+  int kqueue_fd = kqueue();
+  if (kqueue_fd < 0) {
+    return -1;
+  }
+  struct kevent ev;
+  EV_SET(&ev, fd, EVFILT_READ, EV_ADD, 0, 0, NULL);
+  if (kevent(kqueue_fd, &ev, 1, NULL, 0, NULL) < 0) {
+    return -1;
+  }
+  if (kevent(kqueue_fd, NULL, 0, &ev, 1, &tm) <= 0) {
+    return -1;
+  }
+  close(kqueue_fd);
+#else
+  int epoll_fd = epoll_create1(0);
+  if (epoll_fd < 0) {
+    return -1;
+  }
+  struct epoll_event ev;
+  ev.events = EPOLLIN;
+  ev.data.fd = fd;
+  if (epoll_ctl(epoll_fd, EPOLL_CTL_ADD, fd, &ev) < 0) {
+    return -1;
+  }
+  if (epoll_wait(epoll_fd, &ev, 1, timeout * 1000) <= 0) {
+    return -1;
+  }
+  close(epoll_fd);
+#endif  // __APPLE__
+
+  if (fcntl(fd, F_SETFL, O_RDONLY) == -1) {
+    return -1;
+  }
+
+  /* Read the control char to make sure the other side is in sync. */
+  char buf[1];
+  if (my_read(fd, (uchar *)buf, 1, MYF(0)) <= 0) {
+    return -1;
+  }
+  if (memcmp(buf, DS_FIFO_CONTROL_CHAR, 1)) {
+    return -1;
+  }
+
+  return fd;
+}
