@@ -493,7 +493,7 @@ static void extract_worker_thread_func(extract_ctxt_t &ctxt) {
   xb_rstream_t *stream = NULL;
   xb_rstream_chunk_t chunk;
   file_entry_t *entry;
-  xb_rstream_result_t res;
+  xb_rstream_result_t res = XB_STREAM_READ_CHUNK;
 
   ctxt.mutex->lock();
   stream = ctxt.streams->front();
@@ -592,7 +592,8 @@ static void extract_worker_thread_func(extract_ctxt_t &ctxt) {
                qpress_offset] = 0;
 
         char error[512];
-        if (!restore_sparseness(path, XBSTREAM_BUFFER_SIZE, error)) {
+        if (!restore_sparseness(path, XBSTREAM_BUFFER_SIZE, error,
+                                opt_verbose)) {
           msg("%s: restore_sparseness failed for file %s: %s\n", my_progname,
               chunk.path, error);
         }
