@@ -261,9 +261,6 @@ bool check_if_skip_database_by_path(
     const char *path /*!< in: path to the db directory. */
 );
 
-/* pause xtrabackup and wait for resume */
-void debug_sync_point(const char *name);
-
 /************************************************************************
 Check if parameter is set in defaults file or via command line argument
 @return true if parameter is set. */
@@ -336,4 +333,10 @@ bool xb_process_datadir(const char *path,   /*!<in: datadir path */
 @param[in,out]	buf		log header buffer
 @param[in]	lsn		lsn to update */
 void update_log_temp_checkpoint(byte *buf, lsn_t lsn);
+
+/** Apply the data from .delta to the tablespace
+@param[in] entry  the .delta file entry
+@return true on success, false on error */
+bool xtrabackup_apply_delta(const datadir_entry_t &entry, void *);
+
 #endif /* XB_XTRABACKUP_H */
