@@ -1,5 +1,10 @@
 set -eu
 
+function is_sparse_file() {
+    let n=$(stat --printf "%b*%B/%s" $1)
+    [ "$n" = "0" ]
+}
+
 ### save a copy of backup directory
 function save_backup()
 {
@@ -376,6 +381,7 @@ pid-file=${MYSQLD_PIDFILE}
 replicate-ignore-db=mysql
 replicate-ignore-db=performance_schema
 replicate-ignore-db=sys
+secure-file-priv=""
 innodb_log_file_size=48M
 ${MYSQLD_EXTRA_MY_CNF_OPTS:-}
 #core-file
@@ -833,7 +839,7 @@ function verify_db_state()
 {
     $MYSQLDUMP $MYSQL_ARGS $MYSQLDUMP_ARGS -t --compact --skip-extended-insert \
         $1 >"$topdir/tmp/$1_new.sql"
-    diff -u "$topdir/tmp/$1_old.sql" "$topdir/tmp/$1_new.sql"
+    run_cmd diff -u "$topdir/tmp/$1_old.sql" "$topdir/tmp/$1_new.sql"
 }
 
 ########################################################################
@@ -1438,11 +1444,12 @@ function grep_general_log()
 function kill_query_pattern()
 {
   local condition=$1
-  run_cmd $MYSQL $MYSQL_ARGS --force --batch  test <<EOF
+  $MYSQL $MYSQL_ARGS --force --batch <<EOF
   select concat('KILL ',id,';') from information_schema.processlist
   where $condition into outfile '$MYSQLD_TMPDIR/killcondition.sql';
   source $MYSQLD_TMPDIR/killcondition.sql;
 EOF
+
   rm -f $MYSQLD_TMPDIR/killcondition.sql
 }
 
