@@ -1,7 +1,8 @@
 #
 # PXB-1784: ALTER UNDO TABLESPACE fails after restore when undo tablespace
 #           state is changed during backup
-#
+# As part of making lock-ddl default it is to check ALTER UNDO TABLESPACE set INACTIVE/ACTIVE
+#should work with both ps and ms
 
 start_server
 
@@ -12,7 +13,7 @@ while true ; do
     mysql -e "ALTER UNDO TABLESPACE undo1 SET ACTIVE"
 done &
 
-xtrabackup --no-backup-locks --lock-ddl --backup --target-dir=$topdir/backup
+xtrabackup --backup --target-dir=$topdir/backup
 
 mysql -e "CREATE UNDO TABLESPACE undo2 ADD DATAFILE 'undo2.ibu'"
 
@@ -21,7 +22,7 @@ while true ; do
     mysql -e "ALTER UNDO TABLESPACE undo2 SET ACTIVE"
 done &
 
-xtrabackup --no-backup-locks --lock-ddl --backup --incremental-basedir=$topdir/backup --target-dir=$topdir/inc
+xtrabackup --backup --incremental-basedir=$topdir/backup --target-dir=$topdir/inc
 
 xtrabackup --prepare --apply-log-only --target-dir=$topdir/backup
 xtrabackup --prepare --target-dir=$topdir/backup --incremental-dir=$topdir/inc
