@@ -178,6 +178,7 @@ get_sources(){
     sed -i "s:@@XB_VERSION_EXTRA@@:${EXTRAVER}:g" storage/innobase/xtrabackup/utils/percona-xtrabackup.spec
     sed -i "s:@@XB_RPM_VERSION_EXTRA@@:${RPM_EXTRAVER}:g" storage/innobase/xtrabackup/utils/percona-xtrabackup.spec
     sed -i "s:@@XB_REVISION@@:${REVISION}:g" storage/innobase/xtrabackup/utils/percona-xtrabackup.spec
+    sed -i "s:@@RPM_RELEASE@@:${RPM_RELEASE}:g" storage/innobase/xtrabackup/utils/percona-xtrabackup.spec
     #
     # create a PXB tar
     cd ${WORKDIR}/percona-xtrabackup
@@ -245,25 +246,30 @@ install_deps() {
         yum -y install git wget yum-utils curl
         yum install -y https://repo.percona.com/yum/percona-release-latest.noarch.rpm
         if [ x"$ARCH" = "xx86_64" ]; then
-            if [ $RHEL = 9 ]; then
-                yum-config-manager --enable ol9_distro_builder
-                yum-config-manager --enable ol9_codeready_builder
-                yum -y install https://dl.fedoraproject.org/pub/epel/epel-release-latest-9.noarch.rpm
+            if [ ${RHEL} = 9 -o ${RHEL} = 10 ]; then
+                yum-config-manager --enable ol${RHEL}_distro_builder
+                yum-config-manager --enable ol${RHEL}_codeready_builder
+                yum -y install https://dl.fedoraproject.org/pub/epel/epel-release-latest-${RHEL}.noarch.rpm
             else
                 add_percona_yum_repo
                 percona-release enable tools testing
             fi
         else
             yum-config-manager --enable ol"${RHEL}"_codeready_builder
-            yum -y install epel-release
+            if [ ${RHEL} = 10 ]; then
+                yum -y install https://dl.fedoraproject.org/pub/epel/epel-release-latest-10.noarch.rpm
+                yum -y install epel-release
+            else
+                yum -y install epel-release
+            fi
         fi
 
-        if [ ${RHEL} = 8 -o ${RHEL} = 9 ]; then
+        if [ ${RHEL} = 8 -o ${RHEL} = 9 -o ${RHEL} = 10 ]; then
             PKGLIST+=" binutils-devel python3-pip python3-setuptools"
             PKGLIST+=" libcurl-devel cmake libaio-devel zlib-devel libev-devel bison make gcc"
             PKGLIST+=" rpm-build libgcrypt-devel ncurses-devel readline-devel openssl-devel gcc-c++"
             PKGLIST+=" vim-common rpmlint patchelf python3-wheel libudev-devel"
-            if [ $RHEL = 9 ]; then
+            if [ ${RHEL} = 9 -o ${RHEL} = 10 ]; then
                 PKGLIST+=" rsync procps-ng-devel python3-sphinx"
             else
                 if [ x"$ARCH" = "xx86_64" ]; then
@@ -445,6 +451,20 @@ build_srpm(){
     sed -i "/^%changelog/a - Release ${VERSION}-${RELEASE}" percona-xtrabackup.spec
     sed -i "/^%changelog/a * $(date "+%a") $(date "+%b") $(date "+%d") $(date "+%Y") Percona Development Team <info@percona.com> - ${VERSION}-${RELEASE}" percona-xtrabackup.spec
     #
+    cd ${WORKDIR}/rpmbuild/SOURCES
+    wget https://raw.githubusercontent.com/Percona-Lab/telemetry-agent/phase-0/call-home.sh
+    cd ${WORKDIR}/rpmbuild/SPECS
+    line_number=$(grep -n SOURCE999 percona-xtrabackup.spec | awk -F ':' '{print $1}')
+    cp ../SOURCES/call-home.sh ./
+    awk -v n=$line_number 'NR <= n {print > "part1.txt"} NR > n {print > "part2.txt"}' percona-xtrabackup.spec
+    head -n -1 part1.txt > temp && mv temp part1.txt
+    echo "cat <<'CALLHOME' > /tmp/call-home.sh" >> part1.txt
+    cat call-home.sh >> part1.txt
+    echo "CALLHOME" >> part1.txt
+    cat part2.txt >> part1.txt
+    rm -f call-home.sh part2.txt
+    mv part1.txt percona-xtrabackup.spec
+
     cd $WORKDIR
     #
     mv -fv $TARFILE $WORKDIR/rpmbuild/SOURCES
@@ -589,6 +609,18 @@ build_deb(){
     dpkg-source -x $DSC
     cd $DIRNAME
     dch -m -D "$OS_NAME" --force-distribution -v "$VERSION-$DEB_RELEASE.$OS_NAME" 'Update distribution'
+    cd debian/
+    wget https://raw.githubusercontent.com/Percona-Lab/telemetry-agent/phase-0/call-home.sh
+    sed -i 's:exit 0::' percona-xtrabackup-80.postinst
+    echo "cat <<'CALLHOME' > /tmp/call-home.sh" >> percona-xtrabackup-80.postinst
+    cat call-home.sh >> percona-xtrabackup-80.postinst
+    echo "CALLHOME" >> percona-xtrabackup-80.postinst
+    echo "bash +x /tmp/call-home.sh -f \"PRODUCT_FAMILY_PXB\" -v \"${VERSION}-${DEB_RELEASE}\" -d \"PACKAGE\" &>/dev/null || :" >> percona-xtrabackup-80.postinst
+    echo "rm -rf /tmp/call-home.sh" >> percona-xtrabackup-80.postinst
+    echo "exit 0" >> percona-xtrabackup-80.postinst
+    rm -f call-home.sh
+    cd ../
+
     dpkg-buildpackage -rfakeroot -uc -us -b
 
     cd ${WORKDIR}
