@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -227,8 +227,8 @@ static struct my_option my_long_options[] = {
      "Correct checksum information for table.", nullptr, nullptr, nullptr,
      GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
 #ifdef NDEBUG
-    {"debug", '#', "This is a non-debug version. Catch this and exit.", 0, 0, 0,
-     GET_DISABLED, OPT_ARG, 0, 0, 0, 0, 0, 0},
+    {"debug", '#', "This is a non-debug version. Catch this and exit.", nullptr,
+     nullptr, nullptr, GET_DISABLED, OPT_ARG, 0, 0, 0, nullptr, 0, nullptr},
 #else
     {"debug", '#', "Output debug log. Often this is 'd:t:o,filename'.", nullptr,
      nullptr, nullptr, GET_STR, OPT_ARG, 0, 0, 0, nullptr, 0, nullptr},
@@ -286,9 +286,6 @@ static struct my_option my_long_options[] = {
     {"recover", 'r',
      "Can fix almost anything except unique keys that aren't unique.", nullptr,
      nullptr, nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
-    {"parallel-recover", 'p',
-     "Same as '-r' but creates all the keys in parallel.", nullptr, nullptr,
-     nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
     {"safe-recover", 'o',
      "Uses old recovery method; Slower than '-r' but can handle a couple of "
      "cases where '-r' reports that it can't fix the data file.",
@@ -437,7 +434,7 @@ static void usage(void) {
   puts(
       "Check options (check is the default action for myisamchk):\n\
   -c, --check	      Check table for errors.\n\
-  -e, --extend-check  Check the table VERY throughly.  Only use this in\n\
+  -e, --extend-check  Check the table VERY thoroughly. Only use this in\n\
                       extreme cases as myisamchk should normally be able to\n\
                       find out if the table is ok even without this switch.\n\
   -F, --fast	      Check only tables that haven't been closed properly.\n\
@@ -471,9 +468,6 @@ static void usage(void) {
                       unique.\n\
   -n, --sort-recover  Forces recovering with sorting even if the temporary\n\
 		      file would be very big.\n\
-  -p, --parallel-recover\n\
-                      Uses the same technique as '-r' and '-n', but creates\n\
-                      all the keys in parallel, in different threads.\n\
   -o, --safe-recover  Uses old recovery method; Slower than '-r' but can\n\
 		      handle a couple of cases where '-r' reports that it\n\
 		      can't fix the data file.\n\
@@ -484,7 +478,7 @@ static void usage(void) {
   -q, --quick         Faster repair by not modifying the data file.\n\
                       One can give a second '-q' to force myisamchk to\n\
 		      modify the original datafile in case of duplicate keys.\n\
-		      NOTE: Tables where the data file is currupted can't be\n\
+		      NOTE: Tables where the data file is corrupted can't be\n\
 		      fixed with this option.\n\
   -u, --unpack        Unpack file packed with myisampack.\n\
 ");
@@ -525,7 +519,7 @@ TYPELIB myisam_stats_method_typelib = {
 /* Read options */
 
 static bool get_one_option(int optid,
-                           const struct my_option *opt MY_ATTRIBUTE((unused)),
+                           const struct my_option *opt [[maybe_unused]],
                            char *argument) {
   switch (optid) {
     case 'a':
@@ -627,11 +621,6 @@ static bool get_one_option(int optid,
       check_param.testflag &= ~T_REP_ANY;
       if (argument != disabled_my_option) check_param.testflag |= T_REP_BY_SORT;
       break;
-    case 'p':
-      check_param.testflag &= ~T_REP_ANY;
-      if (argument != disabled_my_option)
-        check_param.testflag |= T_REP_PARALLEL;
-      break;
     case 'o':
       check_param.testflag &= ~T_REP_ANY;
       check_param.force_sort = false;
@@ -879,7 +868,7 @@ static int myisamchk(MI_CHECK *param, char *filename) {
     return 1;
   }
   share = info->s;
-  share->options &= ~HA_OPTION_READ_ONLY_DATA; /* We are modifing it */
+  share->options &= ~HA_OPTION_READ_ONLY_DATA; /* We are modifying it */
   share->tot_locks -= share->r_locks;
   share->r_locks = 0;
 
@@ -1003,7 +992,7 @@ static int myisamchk(MI_CHECK *param, char *filename) {
         }
       }
       if (!error) {
-        if ((param->testflag & (T_REP_BY_SORT | T_REP_PARALLEL)) &&
+        if ((param->testflag & T_REP_BY_SORT) &&
             (mi_is_any_key_active(share->state.key_map) ||
              (rep_quick && !param->keys_in_use && !recreate)) &&
             mi_test_if_sort_rep(info, info->state->records,
@@ -1012,10 +1001,7 @@ static int myisamchk(MI_CHECK *param, char *filename) {
             The new file might not be created with the right stats depending
             on how myisamchk is run, so we must copy file stats from old to new.
           */
-          if (param->testflag & T_REP_BY_SORT)
-            error = mi_repair_by_sort(param, info, filename, rep_quick, false);
-          else
-            error = mi_repair_parallel(param, info, filename, rep_quick, false);
+          error = mi_repair_by_sort(param, info, filename, rep_quick, false);
           state_updated = true;
         } else if (param->testflag & T_REP_ANY)
           error = mi_repair(param, info, filename, rep_quick, false);
@@ -1199,8 +1185,8 @@ static void descript(MI_CHECK *param, MI_INFO *info, char *name) {
     puts("Packed");
   else
     puts("Fixed length");
-  printf("Character set:       %s (%d)\n",
-         get_charset_name(share->state.header.language),
+  printf("Collation:           %s (%d)\n",
+         get_collation_name(share->state.header.language),
          share->state.header.language);
 
   if (param->testflag & T_VERBOSE) {
@@ -1478,7 +1464,7 @@ static int mi_sort_records(MI_CHECK *param, MI_INFO *info, char *name,
     if (filecopy(param, new_file, info->dfile, 0L, share->pack.header_length,
                  "datafile-header"))
       goto err;
-  info->rec_cache.file = new_file; /* Use this file for cacheing*/
+  info->rec_cache.file = new_file; /* Use this file for caching */
 
   for (key = 0; key < share->base.keys; key++)
     share->keyinfo[key].flag |= HA_SORT_ALLOWS_SAME;
@@ -1628,15 +1614,15 @@ err:
 
 static int not_killed = 0;
 
-volatile int *killed_ptr(MI_CHECK *param MY_ATTRIBUTE((unused))) {
+volatile int *killed_ptr(MI_CHECK *param [[maybe_unused]]) {
   return &not_killed; /* always NULL */
 }
 
 /* print warnings and errors */
 /* VARARGS */
 
-void mi_check_print_info(MI_CHECK *param MY_ATTRIBUTE((unused)),
-                         const char *fmt, ...) {
+void mi_check_print_info(MI_CHECK *param [[maybe_unused]], const char *fmt,
+                         ...) {
   va_list args;
 
   va_start(args, fmt);
