@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -40,12 +40,12 @@ class MasterGCPConf {
   friend class Dbdih;
     
 public:
-  STATIC_CONST( SignalLength = 10 + 2 );
+  static constexpr Uint32 SignalLength = 10 + 2;
 
   enum State {
     GCP_READY            = 0,
     /**
-     * GCP_PREPARE recevied (and replied)
+     * GCP_PREPARE received (and replied)
      */
     GCP_PREPARE_RECEIVED = 1,
 
@@ -118,7 +118,7 @@ class MasterGCPReq {
   friend class Dbdih;
     
 public:
-  STATIC_CONST( SignalLength = 2 );
+  static constexpr Uint32 SignalLength = 2;
 private:
   Uint32 masterRef;
   Uint32 failedNodeId;
@@ -134,7 +134,7 @@ class MasterGCPRef {
   friend class Dbdih;
     
 public:
-  STATIC_CONST( SignalLength = 2 );
+  static constexpr Uint32 SignalLength = 2;
 private:
   Uint32 senderNodeId;
   Uint32 failedNodeId;
