@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2005, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2005, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -234,7 +234,7 @@ extern "C" {
      * NDB_LE_EventBufferStatus with new fields added,
      * as well as the report text is improved.
      * Though it would work to add more fields to NDB_LE_EventBufferStatus
-     * since the Data[] struct can accomodate 29 fields, it is cleaner
+     * since the Data[] struct can accommodate 29 fields, it is cleaner
      * to introduce a new struct.
      * During an upgrade with incorrect order-
      * old mgmd with newer other components :
@@ -245,6 +245,11 @@ extern "C" {
      *   report will not be produced.
      */
     ,NDB_LE_EventBufferStatus2 = 85
+    /** NDB_LE_EventBufferStatus3 is an extension of
+      * NDB_LE_EventBufferStatus with new fields added to extend the sizes
+      * of total, max and alloc'ed bytes to 64-bit values
+      */
+    ,NDB_LE_EventBufferStatus3 = 87
   };
 
   /**
@@ -257,7 +262,7 @@ extern "C" {
     /*  Must be a nonnegative integer (used for array indexing) */
     /** Cluster log on */
     NDB_MGM_EVENT_SEVERITY_ON    = 0,
-    /** Used in NDB Cluster developement */
+    /** Used in NDB Cluster development */
     NDB_MGM_EVENT_SEVERITY_DEBUG = 1,
     /** Informational messages*/
     NDB_MGM_EVENT_SEVERITY_INFO = 2,
@@ -548,6 +553,21 @@ extern "C" {
     unsigned report_reason;
   };
 
+  struct ndb_logevent_EventBufferStatus3 {
+    unsigned usage_l;
+    unsigned alloc_l;
+    unsigned max_l;
+    unsigned latest_consumed_epoch_l;
+    unsigned latest_consumed_epoch_h;
+    unsigned latest_buffered_epoch_l;
+    unsigned latest_buffered_epoch_h;
+    unsigned ndb_reference;
+    unsigned report_reason;
+    unsigned usage_h;
+    unsigned alloc_h;
+    unsigned max_h;
+  };
+
   /* STATISTIC */
   struct ndb_logevent_TransReportCounters {
     unsigned trans_count;
@@ -850,7 +870,7 @@ extern "C" {
     /** Which event */
     enum Ndb_logevent_type type;
 
-    /** Time when log event was registred at the management server */
+    /** Time when log event was registered at the management server */
     unsigned time;
 
     /** Category of log event */
@@ -942,6 +962,7 @@ extern "C" {
       struct ndb_logevent_EventBufferStatus EventBufferStatus;
       struct ndb_logevent_SavedEvent SavedEvent;
       struct ndb_logevent_EventBufferStatus2 EventBufferStatus2;
+      struct ndb_logevent_EventBufferStatus3 EventBufferStatus3;
 
       /** Log event data for @ref NDB_LE_BackupStarted */
       struct ndb_logevent_BackupStarted BackupStarted;
@@ -1010,7 +1031,8 @@ enum ndb_logevent_event_buffer_status_report_reason{
   PARTIALLY_BUFFERING,
   BUFFERED_EPOCHS_OVER_THRESHOLD,
   ENOUGH_FREE_EVENTBUFFER,
-  LOW_FREE_EVENTBUFFER
+  LOW_FREE_EVENTBUFFER,
+  EVENTBUFFER_USAGE_HIGH
 };
 
 #ifdef __cplusplus
