@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2004, 2020, Oracle and/or its affiliates.
+   Copyright (c) 2004, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -329,8 +329,7 @@ private:
                              const char *row,
                              Uint32 bound_type,
                              Uint32*& firstWordOfBound);
-  int insert_open_bound(const NdbRecord* key_record,
-                        Uint32*& firstWordOfBound);
+  int insert_open_bound(Uint32*& firstWordOfBound);
 
   int equal_impl(const NdbColumnImpl*, const char*) override;
   NdbRecAttr* getValue_impl(const NdbColumnImpl*, char*) override;
@@ -391,20 +390,4 @@ NdbIndexScanOperation::setBound(Uint32 anAttrId, int type, const void* value,
   return setBound(anAttrId, type, value);
 }
 
-/**
- *   Compare keys of  the current records of two NdbReceiver objects.
- * @param r1 holds the first record to compare.
- * @param r2 holds the second record to compare.
- * @param key_record specifies the keys to compare.
- * @param result_record specifies the format of full records.
- * @param descending if true, descending sort order is to be used.
- * @param read_range_no if true, range numbers will first be compared, and then keys if range numbers are the same for both records.
- * @return -1 if r1<r2, 0 if r1=r2, 1 if r1> r2 (reversed when using 'descending').
- **/
-int compare_ndbrecord(const NdbReceiver *r1,
-                      const NdbReceiver *r2,
-                      const NdbRecord *key_record,
-                      const NdbRecord *result_record,
-                      bool descending,
-                      bool read_range_no);
 #endif
