@@ -32,7 +32,7 @@
 
 /*
  * These macros are robuster versions of the ones in MySQL's DBUG package.
- *
+ * 
  * As the DBUG macros/functions don't check arguments, the caller (or JVM!)
  * crashes in case, for instance, of NULL args.  Also, macros returning a
  * value (like DBUG_EXPLAIN) ought to do so even if NDEBUG was defined.
@@ -40,59 +40,62 @@
 
 #ifndef NDEBUG
 
-#define MY_DBUG_PUSH(a1)     \
-  do {                       \
-    if ((a1)) DBUG_PUSH(a1); \
-  } while (0)
-#define MY_DBUG_POP() DBUG_POP()
-#define MY_DBUG_SET(a1)     \
-  do {                      \
-    if ((a1)) DBUG_SET(a1); \
-  } while (0)
-#define MY_DBUG_EXPLAIN(buf, len) \
-  ((!(buf) || (len) <= 0) ? 1 : DBUG_EXPLAIN(buf, len))
-#define MY_DBUG_PRINT(keyword, arglist)          \
-  do {                                           \
-    if ((keyword)) DBUG_PRINT(keyword, arglist); \
-  } while (0)
+#define MY_DBUG_PUSH(a1)                                                \
+    do { if ((a1)) DBUG_PUSH(a1); } while (0)
+#define MY_DBUG_POP()                                                   \
+    DBUG_POP()
+#define MY_DBUG_SET(a1)                                                 \
+    do { if ((a1)) DBUG_SET(a1); } while (0)
+#define MY_DBUG_EXPLAIN(buf, len)                                       \
+    ((!(buf) || (len) <= 0) ? 1 : DBUG_EXPLAIN(buf, len))
+#define MY_DBUG_PRINT(keyword, arglist)                                 \
+    do { if ((keyword)) DBUG_PRINT(keyword, arglist); } while (0)
 
-#else  // NDEBUG
+#else // NDEBUG
 
 #define MY_DBUG_PUSH(a1)
 #define MY_DBUG_POP()
 #define MY_DBUG_SET(a1)
-#define MY_DBUG_EXPLAIN(buf, len) 1
+#define MY_DBUG_EXPLAIN(buf,len) 1
 #define MY_DBUG_PRINT(keyword, arglist)
 
-#endif  // NDEBUG
+#endif // NDEBUG
 
 /*
  * These DBUG functions provide suitable mapping targets for use from Java.
  */
 
 /** Push the state of the DBUG package */
-inline void dbugPush(const char *state [[maybe_unused]]) {
-  MY_DBUG_PUSH(state);
+inline void dbugPush(const char* state [[maybe_unused]])
+{
+    MY_DBUG_PUSH(state);
 }
 
 /** Pop the state of the DBUG package */
-inline void dbugPop() { MY_DBUG_POP(); }
+inline
+void
+dbugPop()
+{
+    MY_DBUG_POP();
+}
 
 /** Set the state of the DBUG package */
-inline void dbugSet(const char *state [[maybe_unused]]) { MY_DBUG_SET(state); }
+inline void dbugSet(const char* state [[maybe_unused]]) { MY_DBUG_SET(state); }
 
 /** Return the state of the DBUG package */
-inline const char *dbugExplain(char *buffer, int length [[maybe_unused]]) {
-  if (!MY_DBUG_EXPLAIN(buffer, length)) {
-    return buffer;
-  }
-  return nullptr;
+inline const char* dbugExplain(char* buffer, int length [[maybe_unused]])
+{
+    if (!MY_DBUG_EXPLAIN(buffer, length)) {
+        return buffer;
+    }
+    return nullptr;
 }
 
 /** Print a message */
-inline void dbugPrint(const char *keyword [[maybe_unused]],
-                      const char *message [[maybe_unused]]) {
-  MY_DBUG_PRINT(keyword, ("%s", message));
+inline void dbugPrint(const char* keyword [[maybe_unused]],
+                      const char* message [[maybe_unused]])
+{
+    MY_DBUG_PRINT(keyword, ("%s", message));
 }
 
-#endif  // dbug_utils_hpp
+#endif // dbug_utils_hpp
