@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2017, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -31,6 +31,8 @@
 #include "my_inttypes.h"
 #include "storage/ndb/include/ndbapi/NdbDictionary.hpp"
 
+class Ndb_event_data;
+class NdbEventOperation;
 struct NDB_SHARE;
 struct st_conflict_fn_def;
 struct st_conflict_fn_arg;
@@ -51,6 +53,11 @@ class Ndb_binlog_client {
   void log_warning(uint code, const char *fmt, ...) const
       MY_ATTRIBUTE((format(printf, 3, 4)));
 
+  /**
+   * @brief Log the NDB error using log_warning()
+   */
+  void log_ndb_error(const NdbError &ndberr) const;
+
   /**
    * @brief event_name_for_table, generate name for the event for this table
    *
@@ -98,12 +105,12 @@ class Ndb_binlog_client {
 
   /**
    * @brief table_should_have_event_op, decide if a NdbEventOperation
-   * should be crated for the current table. Only table which need to
-   * be binlogged would create such a event operation. The exeception
+   * should be created for the current table. Only table which need to
+   * be binlogged would create such a event operation. The exception
    * is the ndb_schema table who subscribes to events for schema distribution.
    * @return  true if table should have a NdbEventOperation
    */
-  bool table_should_have_event_op(const NDB_SHARE *share);
+  bool table_should_have_event_op(const NDB_SHARE *share) const;
 
   /**
    * @brief event_exists_for_table, check if event already exists for this
@@ -118,10 +125,17 @@ class Ndb_binlog_client {
 
   int create_event(Ndb *ndb, const NdbDictionary::Table *ndbtab,
                    const NDB_SHARE *share);
-  bool create_event_data(NDB_SHARE *share, const dd::Table *table_def,
-                         class Ndb_event_data **event_data) const;
-  int create_event_op(NDB_SHARE *share, const NdbDictionary::Table *ndbtab,
-                      const Ndb_event_data *event_data);
+
+ private:
+  NdbEventOperation *create_event_op_in_NDB(Ndb *ndb,
+                                            const NdbDictionary::Table *ndbtab,
+                                            const std::string &event_name,
+                                            const Ndb_event_data *event_data);
+
+ public:
+  int create_event_op(NDB_SHARE *share, const dd::Table *table_def,
+                      const NdbDictionary::Table *ndbtab,
+                      bool replace_op = false);
 
   /**
    * @brief drop_events_for_table, drop all binlog events for the table
@@ -134,10 +148,10 @@ class Ndb_binlog_client {
    * @param thd            thread context
    * @param ndb            Ndb pointer
    * @param dbname         database of table
-   * @param tabname        name of table
+   * @param table_name     name of table
    */
   static void drop_events_for_table(THD *thd, Ndb *ndb, const char *dbname,
-                                    const char *tabname);
+                                    const char *table_name);
 };
 
 #endif
