@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2020, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -27,6 +27,7 @@
 
 #include <functional>
 
+#include "m_string.h"                                 // is_prefix
 #include "storage/ndb/plugin/ha_ndbcluster_binlog.h"  // ndbcluster_binlog_setup_table
 #include "storage/ndb/plugin/ndb_dd.h"                // ndb_dd_fs_name_case
 #include "storage/ndb/plugin/ndb_dd_client.h"         // Ndb_dd_client
@@ -141,6 +142,17 @@ bool Ndb_dd_sync::remove_all_metadata() const {
     ndb_log_verbose(50, "Found %zu NDB tables in schema '%s'",
                     ndb_tables.size(), schema_name);
     for (const std::string &table_name : ndb_tables) {
+      // Check if the table has a trigger. Such tables are handled differently
+      // and not deleted as that would result in the trigger being deleted as
+      // well
+      const dd::Table *table_def;
+      if (!dd_client.get_table(schema_name, table_name.c_str(), &table_def)) {
+        ndb_log_error("Failed to open table '%s.%s' from DD", schema_name,
+                      table_name.c_str());
+        return false;
+      }
+      if (ndb_dd_table_has_trigger(table_def)) continue;
+
       ndb_log_info("Removing table '%s.%s'", schema_name, table_name.c_str());
       if (!remove_table(schema_name, table_name.c_str())) {
         ndb_log_error("Failed to remove table '%s.%s' from DD", schema_name,
@@ -200,12 +212,13 @@ bool Ndb_dd_sync::remove_deleted_tables() const {
     }
 
     // Fetch list of tables in NDB. The util tables are skipped since the
-    // ndb_schema, ndb_schema_result, and ndb_sql_metadata tables are handled
-    // separately during binlog setup. The index stat tables are not installed
-    // in the DD.
+    // ndb_schema, ndb_schema_result, ndb_apply_status, ndb_sql_metadata
+    // and index stat tables are handled separately during binlog setup.
     std::unordered_set<std::string> tables_in_NDB;
+    std::unordered_set<std::string> temp_tables_in_NDB;
     if (!ndb_get_table_names_in_schema(m_thd_ndb->ndb->getDictionary(),
-                                       schema_name, &tables_in_NDB)) {
+                                       schema_name, &tables_in_NDB,
+                                       &temp_tables_in_NDB)) {
       log_NDB_error(m_thd_ndb->ndb->getDictionary()->getNdbError());
       ndb_log_error(
           "Failed to get list of NDB tables in schema '%s' from "
@@ -218,6 +231,8 @@ bool Ndb_dd_sync::remove_deleted_tables() const {
                     "Found %zu NDB tables in schema '%s' in the NDB Dictionary",
                     tables_in_NDB.size(), schema_name);
 
+    remove_copying_alter_temp_tables(schema_name, temp_tables_in_NDB);
+
     // Iterate over all NDB tables found in DD. If they don't exist in NDB
     // anymore, then remove the table from DD
     for (const std::string &ndb_table_name : ndb_tables_in_DD) {
@@ -331,7 +346,7 @@ bool Ndb_dd_sync::synchronize_logfile_group(
   if (existing == nullptr) {
     ndb_log_error("Logfile group '%s' does not exist in DD",
                   logfile_group_name);
-    DBUG_ASSERT(false);
+    assert(false);
     return false;
   }
 
@@ -343,7 +358,7 @@ bool Ndb_dd_sync::synchronize_logfile_group(
         "Could not extract id and version from the definition "
         "of logfile group '%s'",
         logfile_group_name);
-    DBUG_ASSERT(false);
+    assert(false);
     return false;
   }
 
@@ -528,7 +543,7 @@ bool Ndb_dd_sync::synchronize_tablespace(
 
   if (existing == nullptr) {
     ndb_log_error("Tablespace '%s' does not exist in DD", tablespace_name);
-    DBUG_ASSERT(false);
+    assert(false);
     return false;
   }
 
@@ -540,7 +555,7 @@ bool Ndb_dd_sync::synchronize_tablespace(
         "Could not extract id and version from the definition "
         "of tablespace '%s'",
         tablespace_name);
-    DBUG_ASSERT(false);
+    assert(false);
     return false;
   }
 
@@ -647,7 +662,7 @@ bool Ndb_dd_sync::synchronize_tablespaces() const {
 const NdbError *Ndb_dd_sync::fetch_database_ddls(
     NdbTransaction *ndb_transaction, const NdbDictionary::Table *ndb_schema_tab,
     std::vector<Ndb_schema_tuple> *database_ddls) {
-  DBUG_ASSERT(ndb_transaction != nullptr);
+  assert(ndb_transaction != nullptr);
   DBUG_TRACE;
 
   // Create scan operation and define the read
@@ -796,7 +811,7 @@ bool Ndb_dd_sync::synchronize_databases() const {
     unsigned int ddl_counter, ddl_node_id;
     std::tie(db_name, query, schema_ddl_type, ddl_counter, ddl_node_id) =
         ddl_tuple;
-    DBUG_ASSERT(ddl_counter != 0 && ddl_node_id != 0);
+    assert(ddl_counter != 0 && ddl_node_id != 0);
     ndb_log_verbose(5,
                     "ndb_schema query : '%s', db : '%s', "
                     "counter : %u, node_id : %u",
@@ -958,7 +973,7 @@ bool Ndb_dd_sync::synchronize_databases() const {
         }
       } break;
       default:
-        DBUG_ASSERT(!"Unknown database DDL type");
+        assert(!"Unknown database DDL type");
     }
   }
 
@@ -1170,15 +1185,13 @@ bool Ndb_dd_sync::install_table(const char *schema_name, const char *table_name,
 
 bool Ndb_dd_sync::synchronize_table(const char *schema_name,
                                     const char *table_name) const {
-  Ndb *ndb = m_thd_ndb->ndb;
-
   ndb_log_verbose(1, "Synchronizing table '%s.%s'", schema_name, table_name);
 
-  Ndb_table_guard ndbtab_g(ndb, schema_name, table_name);
+  Ndb_table_guard ndbtab_g(m_thd_ndb->ndb, schema_name, table_name);
   const NdbDictionary::Table *ndbtab = ndbtab_g.get_table();
   if (!ndbtab) {
     // Failed to open the table from NDB
-    log_NDB_error(ndb->getDictionary()->getNdbError());
+    log_NDB_error(ndbtab_g.getNdbError());
     ndb_log_error("Failed to setup table '%s.%s'", schema_name, table_name);
 
     // Failed, table was listed but could not be opened, retry
@@ -1251,29 +1264,32 @@ bool Ndb_dd_sync::synchronize_table(const char *schema_name,
     return true;  // Skipped
   }
 
-  int table_id, table_version;
-  if (!ndb_dd_table_get_object_id_and_version(existing, table_id,
-                                              table_version)) {
+  Ndb_dd_handle dd_handle = ndb_dd_table_get_spi_and_version(existing);
+  if (!dd_handle.valid()) {
     ndb_log_error(
         "Failed to extract id and version from table definition "
         "for table '%s.%s'",
         schema_name, table_name);
-    DBUG_ASSERT(false);
+    assert(false);
     return false;
   }
 
-  // Check that latest version of table definition for this NDB table
-  // is installed in DD
-  if (ndbtab->getObjectId() != table_id ||
-      ndbtab->getObjectVersion() != table_version) {
-    ndb_log_info("Table '%s.%s' have different version in DD, reinstalling...",
-                 schema_name, table_name);
-    if (!install_table(schema_name, table_name, ndbtab,
-                       true /* need overwrite */)) {
-      // Failed to create table from NDB
-      ndb_log_error("Failed to install table '%s.%s' from NDB", schema_name,
-                    table_name);
-      return false;
+  {
+    // Check that latest version of table definition for this NDB table
+    // is installed in DD
+    Ndb_dd_handle curr_handle{ndbtab->getObjectId(),
+                              ndbtab->getObjectVersion()};
+    if (curr_handle != dd_handle) {
+      ndb_log_info(
+          "Table '%s.%s' have different version in DD, reinstalling...",
+          schema_name, table_name);
+      if (!install_table(schema_name, table_name, ndbtab,
+                         true /* need overwrite */)) {
+        // Failed to create table from NDB
+        ndb_log_error("Failed to install table '%s.%s' from NDB", schema_name,
+                      table_name);
+        return false;
+      }
     }
   }
 
@@ -1285,8 +1301,8 @@ bool Ndb_dd_sync::synchronize_table(const char *schema_name,
     return false;
   }
 
-  if (ndbcluster_binlog_setup_table(m_thd, ndb, schema_name, table_name,
-                                    table_def) != 0) {
+  if (ndbcluster_binlog_setup_table(m_thd, m_thd_ndb->ndb, schema_name,
+                                    table_name, table_def) != 0) {
     ndb_log_error("Failed to setup binlog for table '%s.%s'", schema_name,
                   table_name);
     return false;
@@ -1309,22 +1325,10 @@ bool Ndb_dd_sync::synchronize_schema(const char *schema_name) const {
   std::unordered_set<std::string> ndb_tables_in_NDB;
   NdbDictionary::Dictionary *dict = m_thd_ndb->ndb->getDictionary();
   /*
-    Fetch list of tables in NDB. The util tables are skipped since the
-    ndb_schema, ndb_schema_result, and ndb_sql_metadata tables are handled
-    separately during binlog setup. The index stat tables are not installed in
-    the DD. This is due to an issue after an initial system restart. The binlog
-    thread of the first MySQL Server connecting to the Cluster post an initial
-    restart pokes the index stat thread to create the index stat tables in NDB.
-    This happens only after the synchronization phase during binlog setup which
-    means that the tables aren't synced to the DD of the first MySQL Server
-    connecting to the Cluster. If there are multiple MySQL Servers connecting to
-    the Cluster, there's a race condition where the index stat tables could be
-    synchronized during subsequent MySQL Server connections depending on when
-    the index stat thread creates them in NDB. If the creation occurs in the
-    middle of the sync during binlog setup of a Server, it opens the door to
-    errors in the sync. The contents of these tables are incomprehensible
-    without some kind of parsing and are thus not exposed to the MySQL Server.
-    They remain visible and accessible via the ndb_select_all tool.
+    Fetch list of tables in NDB.
+    The util tables are skipped since the ndb_schema, ndb_schema_result,
+    ndb_apply_status, ndb_sql_metadata and index stat tables are handled
+    separately during binlog setup.
   */
   if (!ndb_get_table_names_in_schema(dict, schema_name, &ndb_tables_in_NDB)) {
     log_NDB_error(dict->getNdbError());
@@ -1392,3 +1396,38 @@ bool Ndb_dd_sync::synchronize() const {
   ndb_log_info("Completed metadata synchronization");
   return true;
 }
+
+void Ndb_dd_sync::remove_copying_alter_temp_tables(
+    const char *schema_name,
+    const std::unordered_set<std::string> &temp_tables_in_ndb) const {
+  for (const std::string &ndb_table_name : temp_tables_in_ndb) {
+    // if the table starts with #sql2, it's the table left behind after
+    // renaming original table to temporary one, cannot be deleted to prevent
+    // data loss
+    if (is_prefix(ndb_table_name.c_str(), "#sql2")) {
+      ndb_log_error(
+          "Found temporary table %s.%s, which is most likely left behind"
+          " by failed copying alter table",
+          schema_name, ndb_table_name.c_str());
+      continue;
+    }
+
+    // the table is temporary and does not start with prefix #sql2,
+    // so it must have been left behind before renaming orignal table,
+    // if so, it can be deleted to cleanup unfinished copying alter table
+    ndb_log_warning(
+        "Found temporary table %s.%s, wich is most likely left behind by failed"
+        " copying alter table, this table will be removed, the operation"
+        " does not affect original data",
+        schema_name, ndb_table_name.c_str());
+    Ndb_table_guard ndbtab_g(m_thd_ndb->ndb, schema_name,
+                             ndb_table_name.c_str());
+    auto ndbtab = *ndbtab_g.get_table();
+    constexpr int flag = NdbDictionary::Dictionary::DropTableCascadeConstraints;
+
+    if (m_thd_ndb->ndb->getDictionary()->dropTableGlobal(ndbtab, flag)) {
+      log_NDB_error(m_thd_ndb->ndb->getDictionary()->getNdbError());
+      ndb_log_error("Cannot drop %s.%s", schema_name, ndb_table_name.c_str());
+    }
+  }
+}
