@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2019, 2021, Oracle and/or its affiliates.
+  Copyright (c) 2019, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -110,7 +110,7 @@ class ThreadContext : public Ndb_local_connection {
 
   bool get_local_user(const std::string &) const;
   int get_grants_for_user(std::string);
-  bool show_create_user(std::string, std::string &);
+  bool show_create_user(std::string, std::string &, bool use_hex = true);
   void get_create_user(std::string, int);
   void create_user(std::string &, std::string &);
 
@@ -295,9 +295,19 @@ bool ThreadContext::exec_sql(const std::string &statement) {
 /* Run SHOW CREATE USER, and place the result SQL in result.
    Return true on success.
 */
-bool ThreadContext::show_create_user(std::string user, std::string &result) {
+bool ThreadContext::show_create_user(std::string user, std::string &result,
+                                     bool use_hex) {
   std::string statement("SHOW CREATE USER " + user);
-  if (exec_sql(statement)) {
+  int exec_return_val;
+
+  {
+    bool saved_val = m_thd->variables.print_identified_with_as_hex;
+    m_thd->variables.print_identified_with_as_hex = use_hex;
+    exec_return_val = exec_sql(statement);
+    m_thd->variables.print_identified_with_as_hex = saved_val;
+  }
+
+  if (exec_return_val) {
     ndb_log_error("Failed SHOW CREATE USER for %s", user.c_str());
     return false;
   }
@@ -470,7 +480,7 @@ inline bool blacklisted(std::string user) {
    This query selects only those users that have been directly granted
    NDB_STORED_USER, not those granted it transitively via a role. This
    entails that the direct grant is required -- a limitation which must
-   be documented. If there were a table in information_schema analagous to
+   be documented. If there were a table in information_schema analogous to
    mysql.role_edges, we could solve this problem by issuing a JOIN query
    of user_privileges and role_edges. For now, though, living with the
    documented limitation is preferable to relying on the mysql table.
@@ -627,15 +637,29 @@ int ThreadContext::drop_users(ChangeNotice *notice,
 }
 
 /* Stored in the snapshot is a CREATE USER statement. This statement has come
-   from SHOW CREATE USER, so its exact format is known. For idempotence, it
-   must be rewritten as several statements. The final result is:
-      CREATE USER IF NOT EXISTS user@host;
+   from SHOW CREATE USER, so its exact format is known.
+
+   If the user already exists locally and the local SHOW CREATE USER exactly
+   matches the snapshot, then return without doing anything, so that the
+   last_mod timestamp on the user's password does not get unnecessarily reset.
+
+   Otherwise, try to apply the statement as-is. This can fail for several
+   reasons (some of which are tested in the apply_stored_grants test case),
+   but if might succeed.
+
+   If running the CREATE USER statement failed, it must be rewritten as
+   several statements. The final result is:
+      CREATE USER IF NOT EXISTS user@host  IDENTIFIED BY RANDOM PASSWORD;
       REVOKE ALL ON *.* FROM user@host;
       ALTER USER user@host ...;   [CLEAR RESOURCE LIMITS]
       ALTER USER user@host ...;   [SET VALUES FROM SHOW CREATE USER]
+      ALTER USER user@host DEFAULT ROLE ... ;
+
+  The DEFAULT ROLE statement is pushed to the end of a list and run later,
+  after some other statement in the snapshot possibly creates the named role.
 */
 void ThreadContext::create_user(std::string &name, std::string &statement) {
-  const std::string create_user("CREATE USER IF NOT EXISTS ");
+  const std::string create_user_if_ne("CREATE USER IF NOT EXISTS ");
   const std::string random_pass(" IDENTIFIED BY RANDOM PASSWORD");
   const std::string alter_user("ALTER USER ");
   const std::string revoke_all("REVOKE ALL ON *.* FROM ");
@@ -643,20 +667,32 @@ void ThreadContext::create_user(std::string &name, std::string &statement) {
       " WITH MAX_QUERIES_PER_HOUR 0 MAX_UPDATES_PER_HOUR 0 "
       " MAX_CONNECTIONS_PER_HOUR 0 MAX_USER_CONNECTIONS 0");
 
-  const bool exists_local = get_local_user(name);
+  const bool exists_in_cache = get_local_user(name);
 
-  if (exists_local) {
+  if (exists_in_cache) {
+    /* Compare the snapshot user to the local one. Before comparing, we must
+       detect whether the password hash stored in the snapshot used hex encoding
+       or a plain string.  In statement.find(), 36 characters skips past
+       "CREATE USER `a`@`%` IDENTIFIED WITH".
+    */
     std::string show_create;
-    if (show_create_user(name, show_create) && show_create == statement)
+    bool is_hex = (statement.find(" AS 0x", 36) != std::string::npos);
+    if (show_create_user(name, show_create, is_hex) && show_create == statement)
       return;  // Current SHOW CREATE USER already matches snapshot
+  } else {
+    local_granted_users.insert(name);
   }
 
-  if (!exists_local) {
-    /* Create the user with a random password */
-    ndb_log_info("From stored snapshot, adding NDB stored user: %s",
-                 name.c_str());
-    run_acl_statement(create_user + name + random_pass);
-  }
+  ndb_log_info("From stored snapshot, adding NDB stored user: %s",
+               name.c_str());
+
+  /* Try to run the CREATE USER statement stored in the snapshot.
+     If this succeeds, we are done; but there are several reasons it might fail.
+  */
+  if (try_create_user(statement) == false) return;
+
+  /* Create the user with a random password (unless the user already exists) */
+  run_acl_statement(create_user_if_ne + name + random_pass);
 
   /* Revoke any privileges the user may have had prior to this snapshot. */
   run_acl_statement(revoke_all + name);
@@ -675,7 +711,7 @@ void ThreadContext::create_user(std::string &name, std::string &statement) {
   }
 
   /* Locate the part between DEFAULT ROLE and REQUIRE */
-  size_t require_pos = statement.find("REQUIRE ", default_role_pos + 14);
+  size_t require_pos = statement.find(" REQUIRE ", default_role_pos + 14);
   assert(require_pos != std::string::npos);
   size_t role_clause_len = require_pos - default_role_pos;
 
@@ -695,8 +731,8 @@ void ThreadContext::create_user(std::string &name, std::string &statement) {
 void ThreadContext::apply_current_snapshot() {
   for (const char *row : m_current_rows) {
     unsigned int note = 0;
-    size_t str_length;
-    const char *str_start;
+    size_t str_length = 0;
+    const char *str_start = nullptr;
     bool is_null;
 
     int type = Buffer::getType(row);
