@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -32,6 +32,8 @@
 #include <GlobalSignalNumbers.h>
 #include <signaldata/SignalDataPrint.hpp>
 
+#include <EventLogger.hpp>
+
 static const char *            localSignalNames[MAX_GSN+1];
 static SignalDataPrintFunction localPrintFunctions[MAX_GSN+1];
 static const char *            localBlockNames[NO_OF_BLOCKS]; 
@@ -41,18 +43,18 @@ int
 initSignalNames(const char * dst[], const GsnName src[], unsigned short len){
   unsigned i;
   for(i = 0; i<=MAX_GSN; i++)
-    dst[i] = 0;
+    dst[i] = nullptr;
   
   for(i = 0; i<len; i++){
     unsigned short gsn = src[i].gsn;
     const char * name  = src[i].name;
     
-    if(dst[gsn] != 0 && name != 0){
+    if(dst[gsn] != nullptr && name != nullptr){
       if(strcmp(dst[gsn], name) != 0){
-	fprintf(stderr, 
-		"Multiple definition of signal name for gsn: %d (%s, %s)\n", 
-		gsn, dst[gsn], name);
-	exit(0);
+        g_eventLogger->info(
+            "Multiple definition of signal name for gsn: %d (%s, %s)", gsn,
+            dst[gsn], name);
+        exit(0);
       }
     }
     dst[gsn] = name;
@@ -66,18 +68,17 @@ initSignalPrinters(SignalDataPrintFunction dst[],
 		   const NameFunctionPair src[]){
   unsigned i;
   for(i = 0; i<=MAX_GSN; i++)
-    dst[i] = 0;
+    dst[i] = nullptr;
   
   unsigned short gsn;
   for(i = 0; (gsn = src[i].gsn) > 0; i++){
     SignalDataPrintFunction fun = src[i].function;
     
-    if(dst[gsn] != 0 && fun != 0){
+    if(dst[gsn] != nullptr && fun != nullptr){
       if(dst[gsn] != fun){
-	fprintf(stderr, 
-		"Multiple definition of signal print function for gsn: %d\n", 
-		gsn);
-	exit(0);
+        g_eventLogger->info(
+            "Multiple definition of signal print function for gsn: %d", gsn);
+        exit(0);
       }
     }
     dst[gsn] = fun;
@@ -92,17 +93,16 @@ initBlockNames(const char * dst[],
 	       unsigned len){
   unsigned i;
   for(i = 0; i<NO_OF_BLOCKS; i++)
-    dst[i] = 0;
+    dst[i] = nullptr;
 
   for(i = 0; i<len; i++){
     const int index = src[i].number - MIN_BLOCK_NO;
     if(index < 0 ||             // Too small
        index >= NO_OF_BLOCKS || // Too large
-       dst[index] != 0)         // Already occupied
+       dst[index] != nullptr)         // Already occupied
     {
-      fprintf(stderr, 
-	      "Invalid block name definition: %d %s\n",
-	      src[i].number, src[i].name);
+      g_eventLogger->info("Invalid block name definition: %d %s", src[i].number,
+                          src[i].name);
       exit(0);
     }
     dst[index] = src[i].name;
@@ -133,16 +133,11 @@ getSignalName(unsigned short gsn, const char * defVal){
   return defVal;
 }
 
-unsigned short
-getGsn(const char * signalName){
-  return 0;
-}
-
 const char * 
 getBlockName(unsigned short blockNo, const char * ret){
   if(blockNo >= MIN_BLOCK_NO && blockNo <= MAX_BLOCK_NO)
     return localBlockNames[blockNo-MIN_BLOCK_NO];
-  if (ret == 0) {
+  if (ret == nullptr) {
     static char buf[20];
     BaseString::snprintf(buf, sizeof(buf), "BLOCK#%d", (int)blockNo);
     return buf;
@@ -153,7 +148,7 @@ getBlockName(unsigned short blockNo, const char * ret){
 unsigned short
 getBlockNo(const char * blockName){
   for(int i = 0; i<NO_OF_BLOCKS; i++)
-    if(localBlockNames[i] != 0 && strcmp(localBlockNames[i], blockName) == 0)
+    if(localBlockNames[i] != nullptr && strcmp(localBlockNames[i], blockName) == 0)
       return i + MIN_BLOCK_NO;
   return 0;
 }
@@ -162,5 +157,5 @@ SignalDataPrintFunction
 findPrintFunction(unsigned short gsn){
   if(gsn > 0 && gsn <= MAX_GSN)
     return localPrintFunctions[gsn];
-  return 0;
+  return nullptr;
 }
