@@ -1,17 +1,16 @@
-/* Copyright (c) 2007, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2007, 2023, Oracle and/or its affiliates.
    Use is subject to license terms
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,14 +23,18 @@
 
 #include <signaldata/BuildIndxImpl.hpp>
 
-bool printBUILD_INDX_IMPL_REQ(FILE *output, const Uint32 *theData, Uint32 len,
-                              Uint16 /*rbn*/) {
-  if (len < BuildIndxImplReq::SignalLength) {
+bool printBUILD_INDX_IMPL_REQ(FILE* output,
+                              const Uint32* theData,
+                              Uint32 len,
+                              Uint16 /*rbn*/)
+{
+  if (len < BuildIndxImplReq::SignalLength)
+  {
     assert(false);
     return false;
   }
 
-  const BuildIndxImplReq *sig = (const BuildIndxImplReq *)theData;
+  const BuildIndxImplReq* sig = (const BuildIndxImplReq*)theData;
   fprintf(output, " senderRef: 0x%x", sig->senderRef);
   fprintf(output, " senderData: %u", sig->senderData);
   fprintf(output, " requestType: %u", sig->requestType);
@@ -48,28 +51,36 @@ bool printBUILD_INDX_IMPL_REQ(FILE *output, const Uint32 *theData, Uint32 len,
   return true;
 }
 
-bool printBUILD_INDX_IMPL_CONF(FILE *output, const Uint32 *theData, Uint32 len,
-                               Uint16 /*rbn*/) {
-  if (len < BuildIndxImplConf::SignalLength) {
+bool printBUILD_INDX_IMPL_CONF(FILE* output,
+                               const Uint32* theData,
+                               Uint32 len,
+                               Uint16 /*rbn*/)
+{
+  if (len < BuildIndxImplConf::SignalLength)
+  {
     assert(false);
     return false;
   }
 
-  const BuildIndxImplConf *sig = (const BuildIndxImplConf *)theData;
+  const BuildIndxImplConf* sig = (const BuildIndxImplConf*)theData;
   fprintf(output, " senderRef: 0x%x", sig->senderRef);
   fprintf(output, " senderData: %u", sig->senderData);
   fprintf(output, "\n");
   return true;
 }
 
-bool printBUILD_INDX_IMPL_REF(FILE *output, const Uint32 *theData, Uint32 len,
-                              Uint16 /*rbn*/) {
-  if (len < BuildIndxImplRef::SignalLength) {
+bool printBUILD_INDX_IMPL_REF(FILE* output,
+                              const Uint32* theData,
+                              Uint32 len,
+                              Uint16 /*rbn*/)
+{
+  if (len < BuildIndxImplRef::SignalLength)
+  {
     assert(false);
     return false;
   }
 
-  const BuildIndxImplRef *sig = (const BuildIndxImplRef *)theData;
+  const BuildIndxImplRef* sig = (const BuildIndxImplRef*)theData;
   fprintf(output, " senderRef: 0x%x", sig->senderRef);
   fprintf(output, " senderData: %u", sig->senderData);
   fprintf(output, "\n");
