@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2020, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -23,11 +23,13 @@
 */
 
 
+#include "util/require.h"
 #include <pc.hpp>
 #define DBLQH_C
 #include "Dblqh.hpp"
 #include <ndb_limits.h>
 #include "DblqhCommon.hpp"
+#include "debugger/EventLogger.hpp"
 
 #define JAM_FILE_ID 452
 
@@ -164,9 +166,9 @@ void Dblqh::initData()
   m_startup_report_frequency = 0;
 
   c_active_add_frag_ptr_i = RNIL;
-  for (Uint32 i = 0; i < 4096; i++) {
-    ctransidHash[i] = RNIL;
-  }//for
+
+  ctransidHash = NULL;
+  ctransidHashSize = 0;
 
   c_last_force_lcp_time = 0;
   c_free_mb_force_lcp_limit = 16;
@@ -247,7 +249,8 @@ void Dblqh::initData()
   m_restart_local_latest_lcp_id = 0;
 }//Dblqh::initData()
 
-void Dblqh::initRecords(const ndb_mgm_configuration_iterator *mgm_cfg) 
+void Dblqh::initRecords(const ndb_mgm_configuration_iterator* mgm_cfg,
+                        Uint64 logPageFileSize)
 {
 #if defined(USE_INIT_GLOBAL_VARIABLES)
   {
@@ -301,19 +304,51 @@ void Dblqh::initRecords(const ndb_mgm_configuration_iterator *mgm_cfg)
 		  sizeof(LogFileOperationRecord),
 		  clfoFileSize);
 
+    if (clogPartFileSize == 0)
+    {
+      /*
+       * If the number of fragment log parts are fewer than number of LDMs,
+       * some LDM will not own any log part.
+       */
+      ndbrequire(logPageFileSize == 0);
+      ndbrequire(clogFileFileSize == 0);
+    }
+    else
+    {
+      ndbrequire(logPageFileSize % clogPartFileSize == 0);
+    }
+    const Uint32 target_pages_per_logpart =
+        clogPartFileSize > 0 ? logPageFileSize / clogPartFileSize : 0;
+    Uint32 total_logpart_pages = 0;
     LogPartRecordPtr logPartPtr;
     for (logPartPtr.i = 0; logPartPtr.i < clogPartFileSize; logPartPtr.i++)
     {
       ptrAss(logPartPtr, logPartRecord);
       new (logPartPtr.p) LogPartRecord();
       AllocChunk chunks[16];
-      const Uint32 chunkcnt = allocChunks(chunks, 16, RG_FILE_BUFFERS,
-                                          clogPageFileSize / clogPartFileSize,
+      const Uint32 chunkcnt = allocChunks(chunks,
+                                          16,
+                                          RG_FILE_BUFFERS,
+                                          target_pages_per_logpart,
                                           CFG_DB_REDO_BUFFER);
+      require(chunkcnt > 0);
+      const Uint32 endPageI =
+          chunks[chunkcnt - 1].ptrI + chunks[chunkcnt - 1].cnt;
+      if (chunkcnt > 1)
+      {
+        g_eventLogger->info(
+            "Redo log part buffer memory %u was split over %u chunks.",
+            logPartPtr.i,
+            chunkcnt);
+      }
       Ptr<GlobalPage> pagePtr;
-      m_shared_page_pool.getPtr(pagePtr, chunks[0].ptrI);
+      ndbrequire(m_shared_page_pool.getPtr(pagePtr, chunks[0].ptrI));
       logPartPtr.p->logPageRecord = (LogPageRecord*)pagePtr.p;
-      logPartPtr.p->logPageFileSize = clogPageFileSize / clogPartFileSize;
+      /*
+       * Since there can be gaps in page number range, logPageFileSize can be
+       * bigger than the number of pages.
+       */
+      logPartPtr.p->logPageFileSize = endPageI - chunks[0].ptrI;
       logPartPtr.p->firstFreeLogPage = RNIL;
       logPartPtr.p->logPageCount = 0;
       for (Int32 i = chunkcnt - 1; i >= 0; i--)
@@ -322,7 +357,7 @@ void Dblqh::initRecords(const ndb_mgm_configuration_iterator *mgm_cfg)
         ndbrequire(cnt != 0);
 
         Ptr<GlobalPage> pagePtr;
-        m_shared_page_pool.getPtr(pagePtr, chunks[i].ptrI);
+        ndbrequire(m_shared_page_pool.getPtr(pagePtr, chunks[i].ptrI));
         LogPageRecord * base = (LogPageRecord*)pagePtr.p;
         ndbrequire(base >= logPartPtr.p->logPageRecord);
         const Uint32 ptrI = Uint32(base - logPartPtr.p->logPageRecord);
@@ -347,9 +382,13 @@ void Dblqh::initRecords(const ndb_mgm_configuration_iterator *mgm_cfg)
        * they can be accessed from multiple threads in parallel.
        */
       logPartPtr.p->noOfFreeLogPages = logPartPtr.p->logPageCount;
+      /*
+       * Wrap log part pages in ArrayPool for getPtr. May not use seize since
+       * there may be holes in array.
+       */
       logPartPtr.p->m_redo_page_cache.m_pool.set(
-        (RedoCacheLogPageRecord*)&logPartPtr.p->logPageRecord[0],
-        clogPageFileSize/clogPartFileSize);
+          (RedoCacheLogPageRecord*)&logPartPtr.p->logPageRecord[0],
+          logPartPtr.p->logPageFileSize);
       logPartPtr.p->m_redo_page_cache.m_hash.setSize(1023);
       logPartPtr.p->m_redo_page_cache.m_first_page = 0;
 
@@ -358,7 +397,16 @@ void Dblqh::initRecords(const ndb_mgm_configuration_iterator *mgm_cfg)
         (RedoCacheLogPageRecord*)logPartPtr.p->logPageRecord;
       ndbrequire(&base[ZPOS_PAGE_NO] == &tmp1->m_page_no);
       ndbrequire(&base[ZPOS_PAGE_FILE_NO] == &tmp1->m_file_no);
+      total_logpart_pages += logPartPtr.p->logPageCount;
     }
+    if (total_logpart_pages < logPageFileSize)
+    {
+      g_eventLogger->warning(
+          "Not all redo log buffer memory was allocated, got %u pages of %ju.",
+          total_logpart_pages,
+          uintmax_t{logPageFileSize});
+    }
+
     m_redo_open_file_cache.m_pool.set(logFileRecord, clogFileFileSize);
 
     pageRefRecord = (PageRefRecord*)allocRecord("PageRefRecord",
@@ -388,7 +436,6 @@ void Dblqh::initRecords(const ndb_mgm_configuration_iterator *mgm_cfg)
     logFileRecord = 0;
     logFileOperationRecord = 0;
     clogFileFileSize = 0;
-    clogPageFileSize = 0;
     clfoFileSize = 0;
   }
   Pool_context pc;
@@ -485,6 +532,18 @@ void Dblqh::initRecords(const ndb_mgm_configuration_iterator *mgm_cfg)
       bat[i].bits.v = 5;
     }
   }
+
+  ctransidHashSize = tcConnect_pool.getSize();
+  if (ctransidHashSize < 4096) {
+    ctransidHashSize = 4096;
+  }
+  ctransidHash = (Uint32*)allocRecord("TransIdHash",
+                                       sizeof(Uint32),
+                                       ctransidHashSize);
+
+  for (Uint32 i = 0; i < ctransidHashSize; i++) {
+    ctransidHash[i] = RNIL;
+  }
 }//Dblqh::initRecords()
 
 bool
@@ -660,8 +719,6 @@ Dblqh::Dblqh(Block_context& ctx,
     addRecSignal(GSN_TUX_ADD_ATTRCONF, &Dblqh::execTUX_ADD_ATTRCONF);
     addRecSignal(GSN_TUX_ADD_ATTRREF, &Dblqh::execTUX_ADD_ATTRREF);
 
-    addRecSignal(GSN_READ_PSEUDO_REQ, &Dblqh::execREAD_PSEUDO_REQ);
-
     addRecSignal(GSN_DEFINE_BACKUP_REF, &Dblqh::execDEFINE_BACKUP_REF);
     addRecSignal(GSN_DEFINE_BACKUP_CONF, &Dblqh::execDEFINE_BACKUP_CONF);
 
@@ -682,8 +739,6 @@ Dblqh::Dblqh(Block_context& ctx,
     addRecSignal(GSN_DROP_FRAG_CONF, &Dblqh::execDROP_FRAG_CONF);
 
     addRecSignal(GSN_SUB_GCP_COMPLETE_REP, &Dblqh::execSUB_GCP_COMPLETE_REP);
-    addRecSignal(GSN_FSWRITEREQ,
-                 &Dblqh::execFSWRITEREQ);
     addRecSignal(GSN_DBINFO_SCANREQ, &Dblqh::execDBINFO_SCANREQ);
 
     addRecSignal(GSN_FIRE_TRIG_REQ, &Dblqh::execFIRE_TRIG_REQ);
@@ -748,7 +803,6 @@ Dblqh::Dblqh(Block_context& ctx,
     addRecSignal(GSN_LQHKEYREF, &Dblqh::execLQHKEYREF);
     addRecSignal(GSN_LQHKEYCONF, &Dblqh::execLQHKEYCONF);
     addRecSignal(GSN_PACKED_SIGNAL, &Dblqh::execPACKED_SIGNAL);
-    addRecSignal(GSN_READ_PSEUDO_REQ, &Dblqh::execREAD_PSEUDO_REQ);
     addRecSignal(GSN_CONTINUEB, &Dblqh::execCONTINUEB);
     addRecSignal(GSN_SIGNAL_DROPPED_REP, &Dblqh::execSIGNAL_DROPPED_REP, true);
     addRecSignal(GSN_DUMP_STATE_ORD, &Dblqh::execDUMP_STATE_ORD);
@@ -798,7 +852,7 @@ Dblqh::Dblqh(Block_context& ctx,
     &c_scanRecordPool;
   c_transient_pools[DBLQH_COMMIT_ACK_MARKER_TRANSIENT_POOL_INDEX] =
     &m_commitAckMarkerPool;
-  NDB_STATIC_ASSERT(c_transient_pool_count == 3);
+  static_assert(c_transient_pool_count == 3);
   c_transient_pools_shrinking.clear();
 }//Dblqh::Dblqh()
 
@@ -884,6 +938,11 @@ Dblqh::~Dblqh()
                 "TcNodeFailRecord",
                 sizeof(TcNodeFailRecord),
                 ctcNodeFailrecFileSize);
+
+  deallocRecord((void**)&ctransidHash,
+                "TransIdHash",
+                sizeof(Uint32),
+                ctransidHashSize);
 }//Dblqh::~Dblqh()
 
 BLOCK_FUNCTIONS(Dblqh)
