@@ -606,8 +606,7 @@ Dbtux::readKeyAttrs(TuxCtx& ctx,
                             tupVersion,
                             keyAttrs32,
                             count,
-                            outputBuffer,
-                            false);
+                            outputBuffer);
   thrjamDebug(ctx.jamBuffer);
   ndbrequire(ret > 0);
   keyData.reset();
@@ -655,14 +654,13 @@ Dbtux::readKeyAttrs(TuxCtx& ctx,
                                tupVersion,
                                keyAttrs32,
                                count,
-                               outputBuffer,
-                               false);
+                               outputBuffer);
   thrjamDebug(ctx.jamBuffer);
   ndbrequire(ret > 0);
 }
 
 void
-Dbtux::readTablePk(TreeEnt ent, Uint32* pkData, unsigned& pkSize)
+Dbtux::readTableHashKey(TreeEnt ent, Uint32* pkData, unsigned& pkSize)
 {
   const TupLoc tupLoc = ent.m_tupLoc;
   int ret = c_tup->tuxReadPk(c_ctx.tupRealFragPtr,
@@ -670,7 +668,7 @@ Dbtux::readTablePk(TreeEnt ent, Uint32* pkData, unsigned& pkSize)
                              tupLoc.getPageId(),
                              tupLoc.getPageOffset(),
                              pkData,
-                             true);
+                             /*hash=*/true);
   jamEntry();
   if (unlikely(ret <= 0))
   {
