@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -49,6 +49,7 @@
 #include <signaldata/AllocMem.hpp>
 #include <signaldata/BuildIndxImpl.hpp>
 
+#include "debugger/DebuggerNames.hpp"
 #include <RefConvert.hpp>
 #include <portlib/NdbDir.hpp>
 #include <NdbOut.hpp>
@@ -58,6 +59,18 @@
 
 #define JAM_FILE_ID 393
 
+#if defined(VM_TRACE) || defined(ERROR_INSERT)
+/*
+ * To be able to test different combinations of compression, encryption, and,
+ * use of ODirect enable the define below.
+ * This works ok for LCP data files there are typically several files with
+ * different table and fragment number as part of name.
+ * For other file types which only are present in a few copies with predictable
+ * names the actual combinations will not vary much, even between runs.
+ */
+//#define NAME_BASED_DISABLING_COMPRESS_ENCRYPT_ODIRECT
+#endif
+
 /**
  * NDBFS has two types of async IO file threads : Bound and non-bound.
  * These threads are kept in two distinct idle pools.
@@ -173,7 +186,7 @@ Ndbfs::~Ndbfs()
    */
 
   /**
-   * Post enought Request::end to saturate all unbound threads
+   * Post enough Request::end to saturate all unbound threads
    */
   Request request;
   request.action = Request::end;
@@ -384,7 +397,7 @@ Ndbfs::execREAD_CONFIG_REQ(Signal* signal)
     /**
      * each logpart keeps up to 3 logfiles open at any given time...
      *   (bound)
-     * make sure noIdleFiles is atleast 4 times #logparts
+     * make sure noIdleFiles is at least 4 times #logparts
      * In addition the LCP execution can have up to 4 files open in each
      * LDM thread. In the LCP prepare phase we can have up to 2 files
      * (2 CTL files first, then 1 CTL file and finally 1 CTL file and
@@ -562,12 +575,17 @@ void
 Ndbfs::execFSOPENREQ(Signal* signal)
 {
   jamEntry();
+  require(signal->getLength() >= FsOpenReq::SignalLength);
+#if defined(NAME_BASED_DISABLING_COMPRESS_ENCRYPT_ODIRECT)
+  FsOpenReq * const fsOpenReq = (FsOpenReq *)&signal->theData[0];
+#else
   const FsOpenReq * const fsOpenReq = (FsOpenReq *)&signal->theData[0];
+#endif
   const BlockReference userRef = fsOpenReq->userReference;
-
   bool bound = (fsOpenReq->fileFlags & FsOpenReq::OM_THREAD_POOL) == 0;
   AsyncFile* file = getIdleFile(bound);
   ndbrequire(file != NULL);
+  ndbrequire(local_ref(userRef));
 
   Uint32 userPointer = fsOpenReq->userPointer;
   
@@ -576,40 +594,56 @@ Ndbfs::execFSOPENREQ(Signal* signal)
   if (handle.m_cnt)
   {
     jam();
-    handle.getSection(ptr, FsOpenReq::FILENAME);
+    ndbrequire(handle.getSection(ptr, FsOpenReq::FILENAME));
   }
   file->theFileName.set(this, userRef, fsOpenReq->fileNumber, false, ptr);
-  if (handle.m_cnt > FsOpenReq::PASSWORD)
+  if (handle.m_cnt > FsOpenReq::ENCRYPT_KEY_MATERIAL)
   {
     jam();
     SegmentedSectionPtr ptr;
-    handle.getSection(ptr, FsOpenReq::PASSWORD);
-    ndbrequire(ptr.sz * sizeof(Uint32) <= sizeof(file->m_password));
-    copy((Uint32*)&file->m_password, ptr);
-    ndbrequire(4 + file->m_password.password_length <= ptr.sz * sizeof(Uint32));
-    file->m_password.encryption_password[file->m_password.password_length] = 0;
+    ndbrequire(handle.getSection(ptr, FsOpenReq::ENCRYPT_KEY_MATERIAL));
+    ndbrequire(ptr.sz * sizeof(Uint32) <= sizeof(file->m_key_material));
+    copy((Uint32*)&file->m_key_material, ptr);
+    ndbrequire(file->m_key_material.get_needed_words() <= ptr.sz);
   }
   else
   {
-    file->m_password.password_length = 0;
+    file->m_key_material.length = 0;
   }
   releaseSections(handle);
   
+  const Uint32 page_size = fsOpenReq->page_size;
+  const Uint64 file_size = (Uint64{fsOpenReq->file_size_hi} << 32) |
+                           fsOpenReq->file_size_lo;
+  const Uint32 auto_sync_size = fsOpenReq->auto_sync_size;
+#if defined(NAME_BASED_DISABLING_COMPRESS_ENCRYPT_ODIRECT)
+  const int name_hash = crc32(0,
+                              ((const unsigned char*)file->theFileName.c_str()),
+                              strlen(file->theFileName.c_str()));
+  const bool backup = (file->theFileName.get_base_path_spec() == FsOpenReq::BP_BACKUP);
+  const bool allow_gz = backup || (name_hash & 1);
+  const bool allow_enc = backup || (name_hash & 2);
+  const bool allow_odirect = (name_hash & 4);
+#endif
+
   if (fsOpenReq->fileFlags & FsOpenReq::OM_INIT)
   {
     jam();
     Uint32 cnt = 16; // 512k
+    // Need at least two pages when initializing encrypted REDO/TS/UNDO files
+    const Uint32 min_cnt =
+        (fsOpenReq->fileFlags & FsOpenReq::OM_ENCRYPT_CIPHER_MASK) ? 2 : 1;
     Ptr<GlobalPage> page_ptr;
-    m_ctx.m_mm.alloc_pages(RT_NDBFS_INIT_FILE_PAGE, &page_ptr.i, &cnt, 1);
+    m_ctx.m_mm.alloc_pages(RT_NDBFS_INIT_FILE_PAGE, &page_ptr.i, &cnt, min_cnt);
     if(cnt == 0)
     {
-      file->m_page_ptr.setNull();
-      file->m_page_cnt = 0;
+      ndbrequire(!file->has_buffer());
       
       FsRef * const fsRef = (FsRef *)&signal->theData[0];
       fsRef->userPointer  = userPointer; 
       fsRef->setErrorCode(fsRef->errorCode, FsRef::fsErrOutOfMemory);
       fsRef->osErrorCode  = ~0; // Indicate local error
+      log_file_error(GSN_FSOPENREF, file, nullptr, fsRef);
       sendSignal(userRef, GSN_FSOPENREF, signal, 3, JBB);
       return;
     }
@@ -625,14 +659,13 @@ Ndbfs::execFSOPENREQ(Signal* signal)
     if (cnt == 0)
     {
       jam();
-      file->m_page_ptr.setNull();
-      file->m_page_cnt = 0;
+      ndbrequire(!file->has_buffer());
 
       FsRef * const fsRef = (FsRef *)&signal->theData[0];
       fsRef->userPointer  = userPointer;
       fsRef->setErrorCode(fsRef->errorCode, FsRef::fsErrOutOfMemory);
       fsRef->osErrorCode  = ~0; // Indicate local error
-ndbabort();
+      log_file_error(GSN_FSOPENREF, file, nullptr, fsRef);
       sendSignal(userRef, GSN_FSOPENREF, signal, 3, JBB);
       return;
     }
@@ -641,9 +674,7 @@ ndbabort();
   }
   else
   {
-    ndbassert(file->m_page_ptr.isNull());
-    file->m_page_ptr.setNull();
-    file->m_page_cnt = 0;
+    ndbrequire(!file->has_buffer());
   }
   
   if (getenv("NDB_TRACE_OPEN"))
@@ -651,18 +682,32 @@ ndbabort();
 
   Request* request = theRequestPool->get();
   request->action = Request::open;
-  request->error = 0;
+  NDBFS_SET_REQUEST_ERROR(request, 0);
   request->set(userRef, userPointer, newId() );
   request->file = file;
   request->theTrace = signal->getTrace();
   request->par.open.flags = fsOpenReq->fileFlags;
-  request->par.open.page_size = fsOpenReq->page_size;
-  request->par.open.file_size = fsOpenReq->file_size_hi;
-  request->par.open.file_size <<= 32;
-  request->par.open.file_size |= fsOpenReq->file_size_lo;
-  request->par.open.auto_sync_size = fsOpenReq->auto_sync_size;
+#if defined(NAME_BASED_DISABLING_COMPRESS_ENCRYPT_ODIRECT)
+  if (!allow_gz)
+  {
+    request->par.open.flags &= ~(FsOpenReq::OM_GZ);
+  }
+  if (!allow_enc)
+  {
+    request->par.open.flags &= ~(FsOpenReq::OM_ENCRYPT_CIPHER_MASK |
+                                 FsOpenReq::OM_ENCRYPT_KEY_MATERIAL_MASK);
+    file->m_key_material.length = 0;
+  }
+  if (!allow_odirect)
+  {
+    request->par.open.flags &=
+        ~(FsOpenReq::OM_DIRECT|FsOpenReq::OM_DIRECT_SYNC);
+  }
+#endif
+  request->par.open.page_size = page_size;
+  request->par.open.file_size = file_size;
+  request->par.open.auto_sync_size = auto_sync_size;
   request->m_do_bind = bound;
-
   ndbrequire(forward(file, request));
 }
 
@@ -675,13 +720,14 @@ Ndbfs::execFSREMOVEREQ(Signal* signal)
   bool bound = true;
   AsyncFile* file = getIdleFile(bound);
   ndbrequire(file != NULL);
+  ndbrequire(local_ref(userRef));
 
   SectionHandle handle(this, signal);
   SegmentedSectionPtr ptr; ptr.setNull();
   if(handle.m_cnt)
   {
     jam();
-    handle.getSection(ptr, FsOpenReq::FILENAME);
+    ndbrequire(handle.getSection(ptr, FsOpenReq::FILENAME));
   }
 
   file->theFileName.set(this, userRef, req->fileNumber, req->directory, ptr);
@@ -694,7 +740,7 @@ Ndbfs::execFSREMOVEREQ(Signal* signal)
   request->action = Request::rmrf;
   request->par.rmrf.directory = req->directory;
   request->par.rmrf.own_directory = req->ownDirectory;
-  request->error = 0;
+  NDBFS_SET_REQUEST_ERROR(request, 0);
   request->set(userRef, req->userPointer, newId() );
   request->file = file;
   request->theTrace = signal->getTrace();
@@ -727,6 +773,7 @@ Ndbfs::execFSCLOSEREQ(Signal * signal)
   const BlockReference userRef = fsCloseReq->userReference;
   const Uint16 filePointer = (Uint16)fsCloseReq->filePointer;
   const UintR userPointer = fsCloseReq->userPointer; 
+  ndbrequire(local_ref(userRef));
 
   AsyncFile* openFile = theOpenFiles.find(filePointer);
   if (openFile == NULL) {
@@ -737,6 +784,7 @@ Ndbfs::execFSCLOSEREQ(Signal * signal)
     fsRef->userPointer  = userPointer; 
     fsRef->setErrorCode(fsRef->errorCode, FsRef::fsErrFileDoesNotExist);
     fsRef->osErrorCode  = ~0; // Indicate local error
+    log_file_error(GSN_FSOPENREF, openFile, nullptr, fsRef);
     sendSignal(userRef, GSN_FSCLOSEREF, signal, 3, JBB);
 
     g_eventLogger->warning("Trying to close unknown file!! %u", userPointer);
@@ -759,7 +807,7 @@ Ndbfs::execFSCLOSEREQ(Signal * signal)
   }
   request->set(userRef, fsCloseReq->userPointer, filePointer);
   request->file = openFile;
-  request->error = 0;
+  NDBFS_SET_REQUEST_ERROR(request, 0);
   request->theTrace = signal->getTrace();
   request->m_do_bind = false;
 
@@ -770,6 +818,7 @@ void
 Ndbfs::readWriteRequest(int action, Signal * signal)
 {
   Uint32 theData[25 + 1 + NDB_FS_RW_PAGES];
+  ndbrequire(signal->getLength() <= NDB_ARRAY_SIZE(theData));
   memcpy(theData, signal->theData, 4 * signal->getLength());
   SectionHandle handle(this, signal);
   if (handle.m_cnt > 0)
@@ -788,11 +837,12 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
   const BlockReference userRef = fsRWReq->userReference;
   const BlockNumber blockNumber = refToMain(userRef);
   const Uint32 instanceNumber = refToInstance(userRef);
+  ndbrequire(local_ref(userRef));
 
   AsyncFile* openFile = theOpenFiles.find(filePointer);
 
   const NewVARIABLE *myBaseAddrRef =
-    &getBat(blockNumber, instanceNumber)[fsRWReq->varIndex];
+    getBatVar(blockNumber, instanceNumber, fsRWReq->varIndex);
   UintPtr tPageSize;
   UintPtr tClusterSize;
   UintPtr tNRR;
@@ -801,7 +851,7 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
   FsRef::NdbfsErrorCodeType errorCode;
 
   Request *request = theRequestPool->get();
-  request->error = 0;
+  NDBFS_SET_REQUEST_ERROR(request, 0);
   request->set(userRef, userPointer, filePointer);
   request->file = openFile;
   request->action = (Request::Action) action;
@@ -818,12 +868,7 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
 
   if(format != FsReadWriteReq::fsFormatGlobalPage &&
      format != FsReadWriteReq::fsFormatSharedPage)
-  {     
-    if (fsRWReq->varIndex >= getBatSize(blockNumber, instanceNumber)) {
-      jam();// Ensure that a valid variable is used    
-      errorCode = FsRef::fsErrInvalidParameters;
-      goto error;
-    }
+  {
     if (myBaseAddrRef == NULL) {
       jam(); // Ensure that a valid variable is used
       errorCode = FsRef::fsErrInvalidParameters;
@@ -847,7 +892,7 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
       for (unsigned int i = 0; i < fsRWReq->numberOfPages; i++) {
 	jam();
 	const UintPtr varIndex = fsRWReq->data.listOfPair[i].varIndex;
-	const UintPtr fileOffset = fsRWReq->data.listOfPair[i].fileOffset;
+	const ndb_off_t fileOffset = fsRWReq->data.listOfPair[i].fileOffset;
 	if (varIndex >= tNRR) {
 	  jam();
 	  errorCode = FsRef::fsErrInvalidParameters;
@@ -855,7 +900,7 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
 	}//if
 	request->par.readWrite.pages[i].buf = &tWA[varIndex * tClusterSize];
 	request->par.readWrite.pages[i].size = tPageSize;
-	request->par.readWrite.pages[i].offset = (off_t)(fileOffset*tPageSize);
+	request->par.readWrite.pages[i].offset = (fileOffset*tPageSize);
       }//for
       request->par.readWrite.numberOfPages = fsRWReq->numberOfPages;
       break;
@@ -869,9 +914,9 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
         goto error;
       }//if
       const UintPtr varIndex = fsRWReq->data.arrayOfPages.varIndex;
-      const UintPtr fileOffset = fsRWReq->data.arrayOfPages.fileOffset;
+      const ndb_off_t fileOffset = fsRWReq->data.arrayOfPages.fileOffset;
       
-      request->par.readWrite.pages[0].offset = (off_t)(fileOffset * tPageSize);
+      request->par.readWrite.pages[0].offset = (fileOffset * tPageSize);
       request->par.readWrite.pages[0].size = tPageSize * fsRWReq->numberOfPages;
       request->par.readWrite.numberOfPages = 1;
       request->par.readWrite.pages[0].buf = &tWA[varIndex * tPageSize];
@@ -894,10 +939,10 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
 	  goto error;
 	}//if
         // NDB_FS_RW_PAGES overkill, at most 15 ! Or more via execute direct?
+        const ndb_off_t offset = (tPageOffset + (i * tPageSize));
 	request->par.readWrite.pages[i].buf = &tWA[varIndex * tClusterSize];
 	request->par.readWrite.pages[i].size = tPageSize;
-	request->par.readWrite.pages[i].offset = (off_t)
-          (tPageOffset + (i*tPageSize));
+	request->par.readWrite.pages[i].offset = offset;
       }//for
       request->par.readWrite.numberOfPages = fsRWReq->numberOfPages;
       break;
@@ -915,12 +960,12 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
       }
 
       const Uint32 memoryOffset = fsRWReq->data.memoryAddress.memoryOffset;
-      const Uint32 fileOffset = fsRWReq->data.memoryAddress.fileOffset;
+      const ndb_off_t fileOffset = fsRWReq->data.memoryAddress.fileOffset;
       const Uint32 sz = fsRWReq->data.memoryAddress.size;
 
       request->par.readWrite.pages[0].buf = &tWA[memoryOffset];
       request->par.readWrite.pages[0].size = sz;
-      request->par.readWrite.pages[0].offset = (off_t)(fileOffset);
+      request->par.readWrite.pages[0].offset = fileOffset;
       request->par.readWrite.numberOfPages = 1;
       break;
     }
@@ -934,7 +979,7 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
   else if (format == FsReadWriteReq::fsFormatGlobalPage)
   {
     Ptr<GlobalPage> ptr;
-    m_global_page_pool.getPtr(ptr, fsRWReq->data.globalPage.pageNumber);
+    ndbrequire(m_global_page_pool.getPtr(ptr, fsRWReq->data.globalPage.pageNumber));
     request->par.readWrite.pages[0].buf = (char*)ptr.p;
     request->par.readWrite.pages[0].size = ((UintPtr)GLOBAL_PAGE_SIZE)*fsRWReq->numberOfPages;
     request->par.readWrite.pages[0].offset= ((UintPtr)GLOBAL_PAGE_SIZE)*fsRWReq->varIndex;
@@ -944,7 +989,7 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
   {
     ndbrequire(format == FsReadWriteReq::fsFormatSharedPage);
     Ptr<GlobalPage> ptr;
-    m_shared_page_pool.getPtr(ptr, fsRWReq->data.sharedPage.pageNumber);
+    ndbrequire(m_shared_page_pool.getPtr(ptr, fsRWReq->data.sharedPage.pageNumber));
     request->par.readWrite.pages[0].buf = (char*)ptr.p;
     request->par.readWrite.pages[0].size = ((UintPtr)GLOBAL_PAGE_SIZE)*fsRWReq->numberOfPages;
     request->par.readWrite.pages[0].offset= ((UintPtr)GLOBAL_PAGE_SIZE)*fsRWReq->varIndex;
@@ -964,12 +1009,14 @@ error:
   case Request:: write:
   case Request:: writeSync: {
     jam();
+    log_file_error(GSN_FSWRITEREF, openFile, request, fsRef);
     sendSignal(userRef, GSN_FSWRITEREF, signal, 3, JBB);
     break;
   }//case
   case Request:: readPartial: 
   case Request:: read: {
     jam();
+    log_file_error(GSN_FSREADREF, openFile, request, fsRef);
     sendSignal(userRef, GSN_FSREADREF, signal, 3, JBB);
   }//case
   }//switch
@@ -1037,6 +1084,7 @@ Ndbfs::execFSSYNCREQ(Signal * signal)
   BlockReference userRef = signal->theData[1];
   const UintR userPointer = signal->theData[2]; 
   AsyncFile* openFile = theOpenFiles.find(filePointer);
+  ndbrequire(local_ref(userRef));
 
   if (openFile == NULL) {
      jam(); //file not open
@@ -1044,12 +1092,13 @@ Ndbfs::execFSSYNCREQ(Signal * signal)
      fsRef->userPointer = userPointer;
      fsRef->setErrorCode(fsRef->errorCode, FsRef::fsErrFileDoesNotExist);
      fsRef->osErrorCode = ~0; // Indicate local error
+     log_file_error(GSN_FSSYNCREF, openFile, nullptr, fsRef);
      sendSignal(userRef, GSN_FSSYNCREF, signal, 3, JBB);
      return;
   }
   
   Request *request = theRequestPool->get();
-  request->error = 0;
+  NDBFS_SET_REQUEST_ERROR(request, 0);
   request->action = Request::sync;
   request->set(userRef, userPointer, filePointer);
   request->file = openFile;
@@ -1077,7 +1126,7 @@ Ndbfs::execFSSUSPENDORD(Signal * signal)
   }
 
   Request *request = theRequestPool->get();
-  request->error = 0;
+  NDBFS_SET_REQUEST_ERROR(request, 0);
   request->action = Request::suspend;
   request->set(0, 0, filePointer);
   request->file = openFile;
@@ -1091,65 +1140,63 @@ Ndbfs::execFSSUSPENDORD(Signal * signal)
 void 
 Ndbfs::execFSAPPENDREQ(Signal * signal)
 {
+  jamEntry();
   const FsAppendReq * const fsReq = (FsAppendReq *)&signal->theData[0];
   const Uint16 filePointer =  (Uint16)fsReq->filePointer;
   const UintR userPointer = fsReq->userPointer; 
   const BlockReference userRef = fsReq->userReference;
   const BlockNumber blockNumber = refToMain(userRef);
   const Uint32 instanceNumber = refToInstance(userRef);
+  ndbrequire(local_ref(userRef));
 
   FsRef::NdbfsErrorCodeType errorCode;
 
+  Request *request = theRequestPool->get();
   AsyncFile* openFile = theOpenFiles.find(filePointer);
   const NewVARIABLE *myBaseAddrRef =
-    &getBat(blockNumber, instanceNumber)[fsReq->varIndex];
+    getBatVar(blockNumber, instanceNumber, fsReq->varIndex);
 
-  const Uint32* tWA   = (const Uint32*)myBaseAddrRef->WA;
-  const Uint32  tSz   = myBaseAddrRef->nrr;
-  const Uint32 offset = fsReq->offset;
-  const Uint32 size   = fsReq->size;
-  const Uint32 synch_flag = fsReq->synch_flag;
-  Request *request = theRequestPool->get();
-
-  if (openFile == NULL) {
-    jam();
-    errorCode = FsRef::fsErrFileDoesNotExist;
-    goto error;
-  }
-
-  if (myBaseAddrRef == NULL) {
-    jam(); // Ensure that a valid variable is used
-    errorCode = FsRef::fsErrInvalidParameters;
-    goto error;
-  }
-  
-  if (fsReq->varIndex >= getBatSize(blockNumber, instanceNumber)) {
-    jam();// Ensure that a valid variable is used    
-    errorCode = FsRef::fsErrInvalidParameters;
-    goto error;
-  }
-  
-  if(offset + size > tSz){
+  if (unlikely(myBaseAddrRef == NULL))
+  {
     jam(); // Ensure that a valid variable is used
     errorCode = FsRef::fsErrInvalidParameters;
     goto error;
   }
+  {
+    const Uint32* tWA   = (const Uint32*)myBaseAddrRef->WA;
+    const Uint32  tSz   = myBaseAddrRef->nrr;
+    const Uint32 offset = fsReq->offset;
+    const Uint32 size   = fsReq->size;
+    const Uint32 synch_flag = fsReq->synch_flag;
 
-  request->error = 0;
-  request->set(userRef, userPointer, filePointer);
-  request->file = openFile;
-  request->theTrace = signal->getTrace();
-  
-  request->par.append.buf = (const char *)(tWA + offset);
-  request->par.append.size = size << 2;
+    if (openFile == NULL) {
+      jam();
+      errorCode = FsRef::fsErrFileDoesNotExist;
+      goto error;
+    }
 
-  if (!synch_flag)
-    request->action = Request::append;
-  else
-    request->action = Request::append_synch;
-  request->m_do_bind = false;
-  ndbrequire(forward(openFile, request));
-  return;
+    if(offset + size > tSz){
+      jam(); // Ensure that a valid variable is used
+      errorCode = FsRef::fsErrInvalidParameters;
+      goto error;
+    }
+
+    NDBFS_SET_REQUEST_ERROR(request, 0);
+    request->set(userRef, userPointer, filePointer);
+    request->file = openFile;
+    request->theTrace = signal->getTrace();
+
+    request->par.append.buf = (const char *)(tWA + offset);
+    request->par.append.size = size << 2;
+
+    if (!synch_flag)
+      request->action = Request::append;
+    else
+      request->action = Request::append_synch;
+    request->m_do_bind = false;
+    ndbrequire(forward(openFile, request));
+    return;
+  }
   
 error:
   jam();
@@ -1160,6 +1207,7 @@ error:
   fsRef->osErrorCode = ~0; // Indicate local error
 
   jam();
+  log_file_error(GSN_FSAPPENDREF, openFile, request, fsRef);
   sendSignal(userRef, GSN_FSAPPENDREF, signal, 3, JBB);
   return;
 }
@@ -1168,16 +1216,16 @@ void
 Ndbfs::execALLOC_MEM_REQ(Signal* signal)
 {
   jamEntry();
-
   AllocMemReq* req = (AllocMemReq*)signal->getDataPtr();
 
   bool bound = true;
   AsyncFile* file = getIdleFile(bound);
   ndbrequire(file != NULL);
+  ndbrequire(local_ref(req->senderRef));
 
   Request *request = theRequestPool->get();
 
-  request->error = 0;
+  NDBFS_SET_REQUEST_ERROR(request, 0);
   request->set(req->senderRef, req->senderData, 0);
   request->file = file;
   request->theTrace = signal->getTrace();
@@ -1199,9 +1247,10 @@ Ndbfs::execBUILD_INDX_IMPL_REQ(Signal* signal)
   bool bound = true;
   AsyncFile* file = getIdleFile(bound);
   ndbrequire(file != NULL);
+  ndbrequire(local_ref(req->senderRef));
 
   Request *request = theRequestPool->get();
-  request->error = 0;
+  NDBFS_SET_REQUEST_ERROR(request, 0);
   request->set(req->senderRef, req->senderData, 0);
   request->file = file;
   request->theTrace = signal->getTrace();
@@ -1212,9 +1261,7 @@ Ndbfs::execBUILD_INDX_IMPL_REQ(Signal* signal)
   m_ctx.m_mm.alloc_pages(RT_NDBFS_BUILD_INDEX_PAGE, &page_ptr.i, &cnt, cnt);
   if(cnt == 0)
   {
-    file->m_page_ptr.setNull();
-    file->m_page_cnt = 0;
-
+    ndbrequire(!file->has_buffer());
     ndbabort(); // TODO
     return;
   }
@@ -1372,7 +1419,10 @@ Ndbfs::report(Request * request, Signal* signal)
 
   if (request->file->has_buffer())
   {
-    if ((request->action == Request::open && request->error) ||
+    if ((request->action == Request::open && request->error.code != 0) ||
+        // Buffer only used for initializing (OM_INIT) file during open
+        (request->action == Request::open &&
+         (request->par.open.flags & FsOpenReq::OM_WRITE_BUFFER) == 0) ||
         request->action == Request::close ||
         request->action == Request::closeRemove ||
         request->action == Request::buildindx)
@@ -1385,33 +1435,35 @@ Ndbfs::report(Request * request, Signal* signal)
     }
   }
   
-  if (request->error) {
+  if (request->error.code != 0)
+  {
     jam();
     // Initialise FsRef signal
     FsRef * const fsRef = (FsRef *)&signal->theData[0];
     fsRef->userPointer = request->theUserPointer;
-    if(request->error & FsRef::FS_ERR_BIT)
+    if(request->error.code & FsRef::FS_ERR_BIT)
     {
-      fsRef->errorCode = request->error;
+      fsRef->errorCode = request->error.code;
       fsRef->osErrorCode = 0;
     }
     else 
     {
-      fsRef->setErrorCode(fsRef->errorCode, translateErrno(request->error));
-      fsRef->osErrorCode = request->error; 
+      fsRef->setErrorCode(fsRef->errorCode, translateErrno(request->error.code));
+      fsRef->osErrorCode = request->error.code; 
     }
     switch (request->action) {
     case Request:: open: {
       jam();
+      log_file_error(GSN_FSOPENREF, nullptr, request, fsRef);
       // Put the file back in idle files list
       pushIdleFile(request->file);
-//ndbabort();
       sendSignal(ref, GSN_FSOPENREF, signal, FsRef::SignalLength, JBB);
       break;
     }
     case Request:: closeRemove:
     case Request:: close: {
       jam();
+      log_file_error(GSN_FSCLOSEREF, nullptr, request, fsRef);
       sendSignal(ref, GSN_FSCLOSEREF, signal, FsRef::SignalLength, JBB);
 
       g_eventLogger->warning("Error closing file: %s %u/%u",
@@ -1427,6 +1479,7 @@ Ndbfs::report(Request * request, Signal* signal)
     case Request:: write:
     {
       jam();
+      log_file_error(GSN_FSWRITEREF, nullptr, request, fsRef);
       sendSignal(ref, GSN_FSWRITEREF, signal, FsRef::SignalLength, JBB);
       break;
     }
@@ -1434,11 +1487,13 @@ Ndbfs::report(Request * request, Signal* signal)
     case Request:: readPartial:
     {
       jam();
+      log_file_error(GSN_FSREADREF, nullptr, request, fsRef);
       sendSignal(ref, GSN_FSREADREF, signal, FsRef::SignalLength, JBB);
       break;
     }
     case Request:: sync: {
       jam();
+      log_file_error(GSN_FSSYNCREF, nullptr, request, fsRef);
       sendSignal(ref, GSN_FSSYNCREF, signal, FsRef::SignalLength, JBB);
       break;
     }
@@ -1446,11 +1501,13 @@ Ndbfs::report(Request * request, Signal* signal)
     case Request::append_synch:
     {
       jam();
+      log_file_error(GSN_FSAPPENDREF, nullptr, request, fsRef);
       sendSignal(ref, GSN_FSAPPENDREF, signal, FsRef::SignalLength, JBB);
       break;
     }
     case Request::rmrf: {
       jam();
+      log_file_error(GSN_FSREMOVEREF, nullptr, request, fsRef);
       // Put the file back in idle files list
       pushIdleFile(request->file);
       sendSignal(ref, GSN_FSREMOVEREF, signal, FsRef::SignalLength, JBB);
@@ -1467,7 +1524,8 @@ Ndbfs::report(Request * request, Signal* signal)
       AllocMemRef* rep = (AllocMemRef*)signal->getDataPtrSend();
       rep->senderRef = reference();
       rep->senderData = request->theUserPointer;
-      rep->errorCode = request->error;
+      rep->errorCode = request->error.code;
+      log_file_error(GSN_ALLOC_MEM_REF, nullptr, request, fsRef);
       sendSignal(ref, GSN_ALLOC_MEM_REF, signal,
                  AllocMemRef::SignalLength, JBB);
       pushIdleFile(request->file);
@@ -1478,7 +1536,8 @@ Ndbfs::report(Request * request, Signal* signal)
       BuildIndxImplRef* rep = (BuildIndxImplRef*)signal->getDataPtrSend();
       rep->senderRef = reference();
       rep->senderData = request->theUserPointer;
-      rep->errorCode = (BuildIndxImplRef::ErrorCode)request->error;
+      rep->errorCode = (BuildIndxImplRef::ErrorCode)request->error.code;
+      log_file_error(GSN_BUILD_INDX_IMPL_REF, nullptr, request, fsRef);
       sendSignal(ref, GSN_BUILD_INDX_IMPL_REF, signal,
                  BuildIndxImplRef::SignalLength, JBB);
       pushIdleFile(request->file);
@@ -1496,7 +1555,7 @@ Ndbfs::report(Request * request, Signal* signal)
 
       // Keep track on max number of opened files
       if (theOpenFiles.size() > m_maxOpenedFiles)
-	m_maxOpenedFiles = theOpenFiles.size();
+        m_maxOpenedFiles = theOpenFiles.size();
 
       fsConf->filePointer = request->theFilePointer;
       fsConf->fileInfo = 0;
@@ -1530,7 +1589,10 @@ Ndbfs::report(Request * request, Signal* signal)
     }
     case Request:: readPartial: {
       jam();
-      fsConf->bytes_read = Uint32(request->par.readWrite.pages[0].size);
+      size_t bytes_read = 0;
+      for (int i = 0; i < request->par.readWrite.numberOfPages; i++)
+        bytes_read += request->par.readWrite.pages[i].size;
+      fsConf->bytes_read = Uint32(bytes_read);
       sendSignal(ref, GSN_FSREADCONF, signal, 2, JBA);
       break;
     }
@@ -1770,7 +1832,15 @@ Ndbfs::execCONTINUEB(Signal* signal)
 void
 Ndbfs::execSEND_PACKED(Signal* signal)
 {
+  /**
+   * This function is called, but not in response to any incoming signal
+   * Skip locality checking.
+   * In future : Remove possibility for external invocation and/or
+   * initialise the passed Signal object in some way.
+   */
+  //LOCAL_SIGNAL(signal);
   jamEntryDebug();
+
   if (scanIPC(signal))
   {
     if (scanningInProgress == false)
@@ -1792,6 +1862,8 @@ Ndbfs::execSEND_PACKED(Signal* signal)
 void
 Ndbfs::execDUMP_STATE_ORD(Signal* signal)
 {
+  LOCAL_SIGNAL(signal); // Not local for all blocks!
+  jamEntry();
   if(signal->theData[0] == 19){
     return;
   }
@@ -1849,9 +1921,9 @@ Ndbfs::execDUMP_STATE_ORD(Signal* signal)
   {
     g_eventLogger->info("NDBFS: Dump requests: %u",
                         theRequestPool->inuse());
-    for (unsigned ridx=0; ridx < theRequestPool->inuse(); ridx++)
+    const Request* req = nullptr;
+    while((req = theRequestPool->getNextInUseItem(req)))
     {
-      const Request* req = theRequestPool->peekInuseItem(ridx);
       Uint64 duration = 0;
 
       if (NdbTick_IsValid(req->m_startTime))
@@ -1860,10 +1932,9 @@ Ndbfs::execDUMP_STATE_ORD(Signal* signal)
                                    getHighResTimer()).microSec();
       }
 
-      g_eventLogger->info("Request %u action %u %s userRef 0x%x "
+      g_eventLogger->info("Request action %u %s userRef 0x%x "
                           "userPtr %u filePtr %u bind %u "
                           "duration(us) %llu filename %s",
-                          ridx,
                           req->action,
                           Request::actionName(req->action),
                           req->theUserReference,
@@ -1935,6 +2006,8 @@ void Ndbfs::callFSWRITEREQ(BlockReference ref, FsReadWriteReq* req) const
   Uint32 block = refToMain(ref);
   Uint32 instance = refToInstance(ref);
 
+  ndbrequire(block <= MAX_BLOCK_NO);
+
   SimulatedBlock* main_block = globalData.getBlock(block);
   ndbrequire(main_block != nullptr);
   ndbrequire(instance < NDBMT_MAX_BLOCK_INSTANCES);
@@ -1956,6 +2029,147 @@ void Ndbfs::callFSWRITEREQ(BlockReference ref, FsReadWriteReq* req) const
   }
 }
 
+#if defined(VM_TRACE) || defined(ERROR_INSERT) || !defined(NDEBUG)
+extern const char * ndb_basename(const char *path);
+
+static bool check_for_expected_errors(GlobalSignalNumber gsn, AsyncFile* file,
+                                      int error_code)
+{
+  if (gsn == GSN_FSOPENREF && error_code == FsRef::fsErrFileDoesNotExist)
+  {
+    const char* name = file->theFileName.get_base_name();
+    const char* endp = name + strlen(name);
+    size_t len = endp - name;
+    if (file->theFileName.get_base_path_spec() == FsOpenReq::BP_FS)
+    {
+      // LCP/0/T1F0.ctl
+      if (len >= 14 && strncmp(name, "LCP", 3) == 0 &&
+          strncmp(endp - 3, "ctl", 3) == 0)
+      {
+        return true;
+      }
+      // D1/DBDIH/S0.sysfile, D1/NDBCNTR/S0.sysfile
+      if (len >= 19 && strncmp(endp - 10, "S0.sysfile", 10) == 0)
+      {
+        return true;
+      }
+
+      // D1/DBDIH/P0.sysfile, D1/NDBCNTR/P0.sysfile
+      if (len >= 19 && strncmp(endp - 10, "P0.sysfile", 10) == 0)
+      {
+        return true;
+      }
+      // D1/DBDIH/S1.FragList
+      if (len >= 20 && strncmp(endp - 8, "FragList", 8) == 0)
+      {
+        return true;
+      }
+    }
+  }
+  return false;
+}
+
+void Ndbfs::log_file_error(GlobalSignalNumber gsn, AsyncFile* file,
+                           Request* request, FsRef* fsRef)
+{
+  const char* req_file = nullptr;
+  const char* req_func = nullptr;
+  int req_line = 0;
+  int req_code = 0;
+  if (request != nullptr)
+  {
+    req_file = ndb_basename(request->error.file);
+    req_func = request->error.func;
+    req_line = request->error.line;
+    req_code = request->error.code;
+    if (file == nullptr) file = request->file;
+  }
+  const char* file_name = nullptr;
+  unsigned file_bp = FsOpenReq::BP_MAX;
+  if (file != nullptr)
+  {
+    file_bp = file->theFileName.get_base_path_spec();
+    file_name = file->theFileName.get_base_name();
+  }
+  const char* signal_name = getSignalName(gsn);
+  /*
+   * Suppress common expected errors.
+   *
+   * TODO:
+   * Add information in request about what failures requester expects
+   * and use that information to only log if unexpected errors occur.
+   * Make message an error message and enable function also in release build.
+   */
+  bool expected_error = check_for_expected_errors(gsn, file, fsRef->errorCode);
+  if (!expected_error)
+  {
+    g_eventLogger->info("(debug) NDBFS: signal %s %d %d: file %u %s: "
+                        "request error %s %u %s %d",
+                        signal_name, fsRef->errorCode, fsRef->osErrorCode,
+                        file_bp, file_name, req_file, req_line, req_func,
+                        req_code);
+#if defined(VM_TRACE) || defined(ERROR_INSERT)
+    if (gsn == GSN_FSOPENREF &&
+        file != nullptr &&
+        file->theFileName.get_base_path_spec() == FsOpenReq::BP_BACKUP &&
+        fsRef->errorCode == FsRef::fsErrFileExists)
+    {
+      // propagate error to end user
+    }
+    else if (gsn == GSN_FSOPENREF &&
+             file_name != nullptr &&
+             (strstr(file_name, "tmp/t1.dat") ||
+              strstr(file_name, "tmp\\t1.dat")) &&
+             fsRef->errorCode == FsRef::fsErrFileExists)
+    {
+      // test ndb.ndb_dd_ddl create undofile, datafile, with already existing file
+    }
+    else if (gsn == GSN_FSOPENREF &&
+             file_name != nullptr &&
+             (strstr(file_name, "tmp/t1.dat") ||
+              strstr(file_name, "tmp\\t1.dat")) &&
+             fsRef->errorCode == FsRef::fsErrFileDoesNotExist)
+    {
+      // test ndb.ndb_dd_ddl create undofile, datafile - fail in windows
+    }
+    else if (gsn == GSN_FSOPENREF &&
+             file != nullptr &&
+             strstr(file_name, "FragLog"))
+    {
+      // D11/DBLQH/S2.FragLog
+    }
+    else if (gsn == GSN_FSOPENREF &&
+             file != nullptr &&
+             strstr(file_name, ".Data"))
+    {
+      // LCP/0/T10F1.Data does not exist FsRef::fsErrFileDoesNotExist(2815)
+    }
+    else if (gsn == GSN_FSREADREF &&
+             fsRef->errorCode == FsRef::fsErrReadUnderflow &&
+             file != nullptr &&
+             strstr(file_name, ".FragList"))
+    {
+      // OM_READWRITE existing: D1/DBDIH/S17.FragList - disk full?
+    }
+    else if (gsn == GSN_FSREADREF &&
+             file != nullptr &&
+             strstr(file_name, "S0.sysfile"))
+    {
+      // Invalid/corrupt secretsfile D1/NDBCNTR/S0.sysfile
+    }
+    else
+    {
+      ndbabort(); // Unexpected error?
+    }
+#endif
+  }
+}
+#else
+void Ndbfs::log_file_error(GlobalSignalNumber gsn, AsyncFile* file,
+                           Request* request, FsRef* fsRef)
+{}
+#endif
+
 BLOCK_FUNCTIONS(Ndbfs)
 
 template class Vector<AsyncFile*>;
