@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -72,6 +72,7 @@
 #define ZCHECK_MULTI_TRP_CONNECT 8
 #define ZRESEND_GET_NUM_MULTI_TRP_REQ 9
 #define ZSWITCH_MULTI_TRP 10
+#define ZSEND_TRP_KEEP_ALIVE 11
 
 /* Error Codes ------------------------------*/
 #define ZERRTOOMANY 1101
@@ -389,6 +390,7 @@ private:
 
   void execAPI_VERSION_REQ(Signal* signal);
   void execAPI_BROADCAST_REP(Signal* signal);
+  void execTRP_KEEP_ALIVE(Signal* signal);
 
   void execNODE_FAILREP(Signal *);
   void execALLOC_NODEID_REQ(Signal *);
@@ -494,6 +496,9 @@ private:
   void setHbApiDelay(UintR aHbApiDelay);
   void setArbitTimeout(UintR aArbitTimeout);
   void setCCDelay(UintR aCCDelay);
+  void send_trp_keep_alive_start(Signal* signal);
+  void send_trp_keep_alive(Signal* signal);
+  void setTrpKeepAliveSendDelay(Uint32 delay);
 
   // Interface to arbitration module
   void handleArbitStart(Signal* signal);
@@ -603,14 +608,17 @@ private:
 
   QmgrState ctoStatus;
   bool cHbSent;
+  bool c_keep_alive_send_in_progress;
+  Uint32 c_keepalive_seqnum;
 
   Timer interface_check_timer;
   Timer hb_check_timer;
   Timer hb_send_timer;
   Timer hb_api_timer;
+  Timer ka_send_timer;
 
   Int16 processInfoNodeIndex[MAX_NODES];
-  ProcessInfo * receivedProcessInfo;
+  ProcessInfo * receivedProcessInfo = nullptr;
   Uint16 max_api_node_id;
 
   NdbNodeBitmask cfailedNodes;
