@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -112,6 +112,17 @@ void Qmgr::initData()
   interface_check_timer.setDelay(1000);
   interface_check_timer.reset(now);
 
+  Uint32 trp_keep_alive_send_interval = 60*1000; // 1 minute
+  ndb_mgm_get_int_parameter(p, CFG_DB_TRP_KEEP_ALIVE_SEND_INTERVAL,
+                            &trp_keep_alive_send_interval);
+  if (trp_keep_alive_send_interval > 0 &&
+      trp_keep_alive_send_interval < 10)
+  {
+    trp_keep_alive_send_interval = 10;
+  }
+  setTrpKeepAliveSendDelay(trp_keep_alive_send_interval);
+  c_keepalive_seqnum = 0;
+
 #ifdef ERROR_INSERT
   nodeFailCount = 0;
 #endif
@@ -132,6 +143,7 @@ void Qmgr::initData()
   cdelayRegreq = ZDELAY_REGREQ;
   c_allow_api_connect = 0;
   ctoStatus = Q_NOT_ACTIVE;
+  c_keep_alive_send_in_progress = false;
 
   for (nodePtr.i = 1; nodePtr.i < MAX_NODES; nodePtr.i++)
   {
@@ -189,7 +201,15 @@ void Qmgr::initData()
       break;
     }
   }
-  receivedProcessInfo = new ProcessInfo[numOfApiAndMgmNodes];
+  /*
+   * In reality there must be at least one MGM, API, and, DB node.
+   * And numOfApiAndMgmNodes will be betweeen 2 and 254 (MAX_NODES - 2).
+   *
+   * If in future one can have DB only cluster it is ok to leave
+   * receivedProcessInfo as nullptr since it is only used for MGM and API.
+   */
+  if (likely(numOfApiAndMgmNodes > 0))
+    receivedProcessInfo = new ProcessInfo[numOfApiAndMgmNodes];
 }//Qmgr::initData()
 
 void Qmgr::initRecords()
@@ -239,6 +259,7 @@ Qmgr::Qmgr(Block_context& ctx)
   addRecSignal(GSN_API_FAILCONF, &Qmgr::execAPI_FAILCONF);
   addRecSignal(GSN_READ_NODESREQ, &Qmgr::execREAD_NODESREQ);
   addRecSignal(GSN_API_BROADCAST_REP,  &Qmgr::execAPI_BROADCAST_REP);
+  addRecSignal(GSN_TRP_KEEP_ALIVE,  &Qmgr::execTRP_KEEP_ALIVE);
 
   addRecSignal(GSN_NODE_FAILREP, &Qmgr::execNODE_FAILREP);
   addRecSignal(GSN_ALLOC_NODEID_REQ,  &Qmgr::execALLOC_NODEID_REQ);
