@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2005, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2005, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -228,7 +228,8 @@ private:
   friend class Tablespace_client;
   Datafile_pool m_file_pool;
   Tablespace_pool m_tablespace_pool;
-  
+
+  bool c_encrypted_filesystem;
   bool m_lcp_ongoing;
   BlockReference m_end_lcp_ref;
   Datafile_hash m_file_hash;
@@ -378,7 +379,7 @@ public:
   /**
    * Allocated a page from an extent
    *   performs linear search in extent free bits until it find 
-   *   page that has atleast <em>bits</em> bits free
+   *   page that has at least <em>bits</em> bits free
    * 
    * Start search from key->m_page_no 
    *   and return found page in key->m_page_no
@@ -438,13 +439,13 @@ public:
    *
    * Store result in <em>rep</em>
    *
-   * Return  0 - on sucess
+   * Return  0 - on success
    *        <0 - on error
    */
   int get_tablespace_info(CreateFilegroupImplReq* rep);
 
   /**
-   * Update lsn of page corresponing to key
+   * Update lsn of page corresponding to key
    */
   int update_lsn(Local_key* key, Uint64 lsn);
 
