@@ -1,18 +1,17 @@
 /*
- Copyright (c) 2010, 2024, Oracle and/or its affiliates.
+ Copyright (c) 2010, 2023, Oracle and/or its affiliates.
  Use is subject to license terms.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
  as published by the Free Software Foundation.
 
- This program is designed to work with certain software (including
+ This program is also distributed with certain software (including
  but not limited to OpenSSL) that is licensed under separate terms,
  as designated in a particular file or component or in included license
  documentation.  The authors of MySQL hereby grant you an additional
  permission to link the program and your derivative works with the
- separately licensed software that they have either included with
- the program or referenced in the documentation.
+ separately licensed software that they have included with MySQL.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -49,14 +48,14 @@
  * detectable to the compiler as purely type-changing copy operations.
  */
 struct _jtie_jint_Enum {
-  // the enum value (cannot be const to enable assignment)
-  jint value;
+    // the enum value (cannot be const to enable assignment)
+    jint value;
 
-  // conversion constructor
-  _jtie_jint_Enum(jint v) : value(v) {}
+    // conversion constructor
+    _jtie_jint_Enum(jint v) : value(v) {}
 
-  // conversion operator
-  operator jint() { return value; }
+    // conversion operator
+    operator jint() { return value; }
 };
 
 /**
@@ -69,12 +68,14 @@ struct _jtie_jint_Enum {
  *
  * Naming convention:
  *   type alias:                specifies a mapping:
- *   ttrait_<T>_i               jint <->       C
+ *   ttrait_<T>_i               jint <->       C 
  *   ttrait_<T>_c_iv            jint <-> const C
  */
-#define JTIE_DEFINE_JINT_ENUM_TYPE_MAPPING(C, T)            \
-  typedef ttrait<jint, C, _jtie_jint_Enum> ttrait_##T##_iv; \
-  typedef ttrait<jint, C const, _jtie_jint_Enum> ttrait_##T##_c_iv;
+#define JTIE_DEFINE_JINT_ENUM_TYPE_MAPPING( C, T )              \
+    typedef ttrait< jint, C, _jtie_jint_Enum                    \
+                    > ttrait_##T##_iv;                          \
+    typedef ttrait< jint, C const, _jtie_jint_Enum              \
+                    > ttrait_##T##_c_iv;
 
 /**
  * A helper class template that predicates the supported type conversions
@@ -86,9 +87,9 @@ struct is_valid_enum_type_mapping;
 */
 
 // XXX to document
-#define JTIE_INSTANTIATE_JINT_ENUM_TYPE_MAPPING(C)
+#define JTIE_INSTANTIATE_JINT_ENUM_TYPE_MAPPING( C )
 //    template struct is_valid_enum_type_mapping< jint, C >;
 
 // ---------------------------------------------------------------------------
 
-#endif  // jtie_tconv_enum_hpp
+#endif // jtie_tconv_enum_hpp
