@@ -1,4 +1,4 @@
-/* Copyright (c) 2003, 2022, Oracle and/or its affiliates.
+/* Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -572,72 +572,71 @@ doall()
 static struct my_option
 my_long_options[] =
 {
-  NDB_STD_OPTS("ndb_index_stat"),
+  NdbStdOpt::usage,
+  NdbStdOpt::help,
+  NdbStdOpt::version,
+  NdbStdOpt::ndb_connectstring,
+  NdbStdOpt::mgmd_host,
+  NdbStdOpt::connectstring,
+  NdbStdOpt::ndb_nodeid,
+  NdbStdOpt::connect_retry_delay,
+  NdbStdOpt::connect_retries,
+  NDB_STD_OPT_DEBUG
   // stats options
-  { "database", 'd',
-    "Name of database table is in",
-    &_dbname, &_dbname, 0,
-    GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
-  { "delete", NDB_OPT_NOSHORT,
-    "Delete index stats of given table"
+  { "database", 'd', "Name of database table is in",
+    &_dbname, nullptr, nullptr, GET_STR, REQUIRED_ARG,
+    0, 0, 0, nullptr, 0, nullptr },
+  { "delete", NDB_OPT_NOSHORT, "Delete index stats of given table"
      " and stop any configured auto update",
-    &_delete, &_delete, 0,
-    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "update", NDB_OPT_NOSHORT,
-    "Update index stats of given table"
+    &_delete, nullptr, nullptr, GET_BOOL, NO_ARG,
+    0, 0, 0, nullptr, 0, nullptr },
+  { "update", NDB_OPT_NOSHORT, "Update index stats of given table"
      " and restart any configured auto update",
-    &_update, &_update, 0,
-    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "dump", NDB_OPT_NOSHORT,
-    "Dump query cache",
-    &_dump, &_dump, 0,
-    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+    &_update, nullptr, nullptr, GET_BOOL, NO_ARG,
+    0, 0, 0, nullptr, 0, nullptr },
+  { "dump", NDB_OPT_NOSHORT, "Dump query cache",
+    &_dump, nullptr, nullptr, GET_BOOL, NO_ARG,
+    0, 0, 0, nullptr, 0, nullptr },
   { "query", NDB_OPT_NOSHORT,
     "Perform random range queries on first key attr (must be int unsigned)",
-    &_query, &_query, 0,
-    GET_INT, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
+    &_query, nullptr, nullptr, GET_INT, REQUIRED_ARG,
+    0, 0, 0, nullptr, 0, nullptr },
   // sys options
   { "sys-drop", NDB_OPT_NOSHORT,
     "Drop any stats tables and events in NDB kernel (all stats is lost)",
-    &_sys_drop, &_sys_drop, 0,
-    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+    &_sys_drop, nullptr, nullptr, GET_BOOL, NO_ARG,
+    0, 0, 0, nullptr, 0, nullptr },
   { "sys-create", NDB_OPT_NOSHORT,
     "Create stats tables and events in NDB kernel (must not exist)",
-    &_sys_create, &_sys_create, 0,
-    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+    &_sys_create, nullptr, nullptr, GET_BOOL, NO_ARG,
+    0, 0, 0, nullptr, 0, nullptr },
   { "sys-create-if-not-exist", NDB_OPT_NOSHORT,
     "Like --sys-create but do nothing if correct objects exist",
-    &_sys_create_if_not_exist, &_sys_create_if_not_exist, 0,
-    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+    &_sys_create_if_not_exist, nullptr, nullptr, GET_BOOL, NO_ARG,
+    0, 0, 0, nullptr, 0, nullptr },
   { "sys-create-if-not-valid", NDB_OPT_NOSHORT,
     "Like --sys-create-if-not-exist but first drop any invalid objects",
-    &_sys_create_if_not_valid, &_sys_create_if_not_valid, 0,
-    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+    &_sys_create_if_not_valid, nullptr, nullptr, GET_BOOL, NO_ARG,
+    0, 0, 0, nullptr, 0, nullptr },
   { "sys-check", NDB_OPT_NOSHORT,
     "Check that correct stats tables and events exist in NDB kernel",
-    &_sys_check, &_sys_check, 0,
-    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "sys-skip-tables", NDB_OPT_NOSHORT,
-    "Do not apply sys options to tables",
-    &_sys_skip_tables, &_sys_skip_tables, 0,
-    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "sys-skip-events", NDB_OPT_NOSHORT,
-    "Do not apply sys options to events",
-    &_sys_skip_events, &_sys_skip_events, 0,
-    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+    &_sys_check, nullptr, nullptr, GET_BOOL, NO_ARG,
+    0, 0, 0, nullptr, 0, nullptr },
+  { "sys-skip-tables", NDB_OPT_NOSHORT, "Do not apply sys options to tables",
+    &_sys_skip_tables, nullptr, nullptr, GET_BOOL, NO_ARG,
+    0, 0, 0, nullptr, 0, nullptr },
+  { "sys-skip-events", NDB_OPT_NOSHORT, "Do not apply sys options to events",
+    &_sys_skip_events, nullptr, nullptr, GET_BOOL, NO_ARG,
+    0, 0, 0, nullptr, 0, nullptr },
   // other
-  { "verbose", 'v',
-    "Verbose messages",
-    &_verbose, &_verbose, 0,
-    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+  { "verbose", 'v', "Verbose messages",
+    &_verbose, nullptr, nullptr, GET_BOOL, NO_ARG,
+    0, 0, 0, nullptr, 0, nullptr },
   { "loops", NDB_OPT_NOSHORT,
     "Repeat same commands a number of times (for testing)",
-    &_loops, &_loops, 0,
-    GET_INT, REQUIRED_ARG, 1, 0, 0, 0, 0, 0 },
-  { 0, 0,
-    0,
-    0, 0, 0,
-    GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0 }
+    &_loops, nullptr, nullptr, GET_INT, REQUIRED_ARG,
+    1, 0, 0, nullptr, 0, nullptr },
+   NdbStdOpt::end_of_options
 };
 
 const char*
