@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2020, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -28,8 +28,9 @@
 
 #include "storage/perfschema/pfs_user.h"
 
+#include <assert.h>
 #include "my_compiler.h"
-#include "my_dbug.h"
+
 #include "my_sys.h"
 #include "storage/perfschema/pfs.h"
 #include "storage/perfschema/pfs_buffer_container.h"
@@ -61,36 +62,74 @@ int init_user(const PFS_global_param *param) {
 }
 
 /** Cleanup all the user buffers. */
-void cleanup_user(void) { global_user_container.cleanup(); }
+void cleanup_user() { global_user_container.cleanup(); }
 
 static const uchar *user_hash_get_key(const uchar *entry, size_t *length) {
   const PFS_user *const *typed_entry;
   const PFS_user *user;
   const void *result;
   typed_entry = reinterpret_cast<const PFS_user *const *>(entry);
-  DBUG_ASSERT(typed_entry != nullptr);
+  assert(typed_entry != nullptr);
   user = *typed_entry;
-  DBUG_ASSERT(user != nullptr);
-  *length = user->m_key.m_key_length;
-  result = user->m_key.m_hash_key;
+  assert(user != nullptr);
+  *length = sizeof(user->m_key);
+  result = &user->m_key;
   return reinterpret_cast<const uchar *>(result);
 }
 
+static uint user_hash_func(const LF_HASH *, const uchar *key,
+                           size_t key_len [[maybe_unused]]) {
+  const PFS_user_key *user_key;
+  uint64 nr1;
+  uint64 nr2;
+
+  assert(key_len == sizeof(PFS_user_key));
+  user_key = reinterpret_cast<const PFS_user_key *>(key);
+  assert(user_key != nullptr);
+
+  nr1 = 0;
+  nr2 = 0;
+
+  user_key->m_user_name.hash(&nr1, &nr2);
+
+  return nr1;
+}
+
+static int user_hash_cmp_func(const uchar *key1,
+                              size_t key_len1 [[maybe_unused]],
+                              const uchar *key2,
+                              size_t key_len2 [[maybe_unused]]) {
+  const PFS_user_key *user_key1;
+  const PFS_user_key *user_key2;
+  int cmp;
+
+  assert(key_len1 == sizeof(PFS_user_key));
+  assert(key_len2 == sizeof(PFS_user_key));
+  user_key1 = reinterpret_cast<const PFS_user_key *>(key1);
+  user_key2 = reinterpret_cast<const PFS_user_key *>(key2);
+  assert(user_key1 != nullptr);
+  assert(user_key2 != nullptr);
+
+  cmp = user_key1->m_user_name.sort(&user_key2->m_user_name);
+  return cmp;
+}
+
 /**
   Initialize the user hash.
   @return 0 on success
 */
 int init_user_hash(const PFS_global_param *param) {
   if ((!user_hash_inited) && (param->m_user_sizing != 0)) {
-    lf_hash_init(&user_hash, sizeof(PFS_user *), LF_HASH_UNIQUE, 0, 0,
-                 user_hash_get_key, &my_charset_bin);
+    lf_hash_init3(&user_hash, sizeof(PFS_user *), LF_HASH_UNIQUE,
+                  user_hash_get_key, user_hash_func, user_hash_cmp_func,
+                  nullptr /* ctor */, nullptr /* dtor */, nullptr /* init */);
     user_hash_inited = true;
   }
   return 0;
 }
 
 /** Cleanup the user hash. */
-void cleanup_user_hash(void) {
+void cleanup_user_hash() {
   if (user_hash_inited) {
     lf_hash_destroy(&user_hash);
     user_hash_inited = false;
@@ -107,22 +146,11 @@ static LF_PINS *get_user_hash_pins(PFS_thread *thread) {
   return thread->m_user_hash_pins;
 }
 
-static void set_user_key(PFS_user_key *key, const char *user,
-                         uint user_length) {
-  DBUG_ASSERT(user_length <= USERNAME_LENGTH);
-
-  char *ptr = &key->m_hash_key[0];
-  if (user_length > 0) {
-    memcpy(ptr, user, user_length);
-    ptr += user_length;
-  }
-  ptr[0] = 0;
-  ptr++;
-  key->m_key_length = ptr - &key->m_hash_key[0];
+static void set_user_key(PFS_user_key *key, const PFS_user_name *user) {
+  key->m_user_name = *user;
 }
 
-PFS_user *find_or_create_user(PFS_thread *thread, const char *username,
-                              uint username_length) {
+PFS_user *find_or_create_user(PFS_thread *thread, const PFS_user_name *user) {
   LF_PINS *pins = get_user_hash_pins(thread);
   if (unlikely(pins == nullptr)) {
     global_user_container.m_lost++;
@@ -130,7 +158,7 @@ PFS_user *find_or_create_user(PFS_thread *thread, const char *username,
   }
 
   PFS_user_key key;
-  set_user_key(&key, username, username_length);
+  set_user_key(&key, user);
 
   PFS_user **entry;
   PFS_user *pfs;
@@ -140,7 +168,7 @@ PFS_user *find_or_create_user(PFS_thread *thread, const char *username,
 
 search:
   entry = reinterpret_cast<PFS_user **>(
-      lf_hash_search(&user_hash, pins, key.m_hash_key, key.m_key_length));
+      lf_hash_search(&user_hash, pins, &key, sizeof(key)));
   if (entry && (entry != MY_LF_ERRPTR)) {
     pfs = *entry;
     pfs->inc_refcount();
@@ -153,16 +181,10 @@ search:
   pfs = global_user_container.allocate(&dirty_state);
   if (pfs != nullptr) {
     pfs->m_key = key;
-    if (username_length > 0) {
-      pfs->m_username = &pfs->m_key.m_hash_key[0];
-    } else {
-      pfs->m_username = nullptr;
-    }
-    pfs->m_username_length = username_length;
 
     pfs->init_refcount();
     pfs->reset_stats();
-    pfs->m_disconnected_count = 0;
+    pfs->reset_connections_stats();
 
     int res;
     pfs->m_lock.dirty_to_allocated(&dirty_state);
@@ -236,7 +258,32 @@ void PFS_user::aggregate_status() {
 
 void PFS_user::aggregate_stats() {
   /* No parent to aggregate to, clean the stats */
-  m_disconnected_count = 0;
+  reset_connections_stats();
+}
+
+void PFS_user::aggregate_stats_from(PFS_account *pfs) {
+  m_disconnected_count += pfs->m_disconnected_count;
+
+  if (m_max_controlled_memory < pfs->m_max_controlled_memory) {
+    m_max_controlled_memory = pfs->m_max_controlled_memory;
+  }
+
+  if (m_max_total_memory < pfs->m_max_total_memory) {
+    m_max_total_memory = pfs->m_max_total_memory;
+  }
+}
+
+void PFS_user::aggregate_disconnect(ulonglong controlled_memory,
+                                    ulonglong total_memory) {
+  m_disconnected_count++;
+
+  if (m_max_controlled_memory < controlled_memory) {
+    m_max_controlled_memory = controlled_memory;
+  }
+
+  if (m_max_total_memory < total_memory) {
+    m_max_total_memory = total_memory;
+  }
 }
 
 void PFS_user::release() { dec_refcount(); }
@@ -282,13 +329,12 @@ static void purge_user(PFS_thread *thread, PFS_user *user) {
   }
 
   PFS_user **entry;
-  entry = reinterpret_cast<PFS_user **>(lf_hash_search(
-      &user_hash, pins, user->m_key.m_hash_key, user->m_key.m_key_length));
+  entry = reinterpret_cast<PFS_user **>(
+      lf_hash_search(&user_hash, pins, &user->m_key, sizeof(user->m_key)));
   if (entry && (entry != MY_LF_ERRPTR)) {
-    DBUG_ASSERT(*entry == user);
+    assert(*entry == user);
     if (user->get_refcount() == 0) {
-      lf_hash_delete(&user_hash, pins, user->m_key.m_hash_key,
-                     user->m_key.m_key_length);
+      lf_hash_delete(&user_hash, pins, &user->m_key, sizeof(user->m_key));
       user->aggregate(false);
       global_user_container.deallocate(user);
     }
@@ -299,7 +345,7 @@ static void purge_user(PFS_thread *thread, PFS_user *user) {
 
 class Proc_purge_user : public PFS_buffer_processor<PFS_user> {
  public:
-  Proc_purge_user(PFS_thread *thread) : m_thread(thread) {}
+  explicit Proc_purge_user(PFS_thread *thread) : m_thread(thread) {}
 
   void operator()(PFS_user *pfs) override {
     pfs->aggregate(true);
@@ -313,7 +359,7 @@ class Proc_purge_user : public PFS_buffer_processor<PFS_user> {
 };
 
 /** Purge non connected users, reset stats of connected users. */
-void purge_all_user(void) {
+void purge_all_user() {
   PFS_thread *thread = PFS_thread::get_current_thread();
   if (unlikely(thread == nullptr)) {
     return;
