@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -55,7 +55,7 @@ class PFS_index_esgs_by_host_by_event_name : public PFS_engine_index {
         m_key_1("HOST"),
         m_key_2("EVENT_NAME") {}
 
-  ~PFS_index_esgs_by_host_by_event_name() override {}
+  ~PFS_index_esgs_by_host_by_event_name() override = default;
 
   virtual bool match(PFS_host *pfs);
   virtual bool match(PFS_instr_class *instr_class);
@@ -87,12 +87,12 @@ struct row_esgs_by_host_by_event_name {
 struct pos_esgs_by_host_by_event_name : public PFS_double_index {
   pos_esgs_by_host_by_event_name() : PFS_double_index(0, 1) {}
 
-  inline void reset(void) {
+  inline void reset() {
     m_index_1 = 0;
     m_index_2 = 1;
   }
 
-  inline void next_host(void) {
+  inline void next_host() {
     m_index_1++;
     m_index_2 = 1;
   }
@@ -107,7 +107,7 @@ class table_esgs_by_host_by_event_name : public PFS_engine_table {
   static int delete_all_rows();
   static ha_rows get_row_count();
 
-  void reset_position(void) override;
+  void reset_position() override;
 
   int rnd_init(bool scan) override;
   int rnd_next() override;
@@ -123,7 +123,7 @@ class table_esgs_by_host_by_event_name : public PFS_engine_table {
   table_esgs_by_host_by_event_name();
 
  public:
-  ~table_esgs_by_host_by_event_name() override {}
+  ~table_esgs_by_host_by_event_name() override = default;
 
  protected:
   int make_row(PFS_host *host, PFS_stage_class *klass);
