@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -129,12 +129,12 @@ struct row_events_transactions {
 struct pos_events_transactions_history : public PFS_double_index {
   pos_events_transactions_history() : PFS_double_index(0, 0) {}
 
-  inline void reset(void) {
+  inline void reset() {
     m_index_1 = 0;
     m_index_2 = 0;
   }
 
-  inline void next_thread(void) {
+  inline void next_thread() {
     m_index_1++;
     m_index_2 = 0;
   }
@@ -154,7 +154,7 @@ class table_events_transactions_common : public PFS_engine_table {
 
   ~table_events_transactions_common() override = default;
 
-  int make_row(PFS_events_transactions *statement);
+  int make_row(PFS_events_transactions *transaction);
 
   /** Current row. */
   row_events_transactions m_row;
@@ -170,7 +170,7 @@ class table_events_transactions_current
   static int delete_all_rows();
   static ha_rows get_row_count();
 
-  void reset_position(void) override;
+  void reset_position() override;
 
   int rnd_init(bool scan) override;
   int rnd_next() override;
@@ -212,7 +212,7 @@ class table_events_transactions_history
   static int delete_all_rows();
   static ha_rows get_row_count();
 
-  void reset_position(void) override;
+  void reset_position() override;
 
   int rnd_init(bool scan) override;
   int rnd_next() override;
@@ -254,7 +254,7 @@ class table_events_transactions_history_long
   int rnd_init(bool scan) override;
   int rnd_next() override;
   int rnd_pos(const void *pos) override;
-  void reset_position(void) override;
+  void reset_position() override;
 
  protected:
   table_events_transactions_history_long();
