@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -53,6 +53,8 @@ struct row_setup_instruments {
   bool m_update_enabled;
   /** True if column TIMED can be updated. */
   bool m_update_timed;
+  /** True if column FLAGS can be updated. */
+  bool m_update_flags;
 };
 
 /** Position of a cursor on PERFORMANCE_SCHEMA.SETUP_INSTRUMENTS. */
@@ -60,14 +62,14 @@ struct pos_setup_instruments : public PFS_double_index,
                                public PFS_instrument_view_constants {
   pos_setup_instruments() : PFS_double_index(FIRST_INSTRUMENT, 1) {}
 
-  inline void reset(void) {
+  inline void reset() {
     m_index_1 = FIRST_INSTRUMENT;
     m_index_2 = 1;
   }
 
-  inline bool has_more_view(void) { return (m_index_1 <= LAST_INSTRUMENT); }
+  inline bool has_more_view() { return (m_index_1 <= LAST_INSTRUMENT); }
 
-  inline void next_view(void) {
+  inline void next_view() {
     m_index_1++;
     m_index_2 = 1;
   }
@@ -77,7 +79,7 @@ class PFS_index_setup_instruments : public PFS_engine_index {
  public:
   PFS_index_setup_instruments() : PFS_engine_index(&m_key), m_key("NAME") {}
 
-  ~PFS_index_setup_instruments() override {}
+  ~PFS_index_setup_instruments() override = default;
 
   bool match_view(uint view);
   bool match(PFS_instr_class *klass);
@@ -94,7 +96,7 @@ class table_setup_instruments : public PFS_engine_table {
   static PFS_engine_table *create(PFS_engine_table_share *);
   static ha_rows get_row_count();
 
-  void reset_position(void) override;
+  void reset_position() override;
 
   int rnd_next() override;
   int rnd_pos(const void *pos) override;
@@ -112,7 +114,7 @@ class table_setup_instruments : public PFS_engine_table {
   table_setup_instruments();
 
  public:
-  ~table_setup_instruments() override {}
+  ~table_setup_instruments() override = default;
 
  private:
   int make_row(PFS_instr_class *klass, bool update_enabled);
