@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -80,7 +80,7 @@ bool PFS_index_user_defined_functions_by_name::match(
 
 PFS_engine_table *table_user_defined_functions::create(
     PFS_engine_table_share *) {
-  table_user_defined_functions *t = new table_user_defined_functions();
+  auto *t = new table_user_defined_functions();
   if (t != nullptr) {
     THD *thd = current_thd;
     assert(thd != nullptr);
@@ -89,7 +89,7 @@ PFS_engine_table *table_user_defined_functions::create(
   return t;
 }
 
-ha_rows table_user_defined_functions::get_row_count(void) {
+ha_rows table_user_defined_functions::get_row_count() {
   ha_rows count;
   udf_hash_rlock();
   count = udf_hash_size();
@@ -112,7 +112,7 @@ struct udf_materialize_state_s {
 
 void table_user_defined_functions::materialize_udf_funcs(udf_func *udf,
                                                          void *arg) {
-  struct udf_materialize_state_s *s = (struct udf_materialize_state_s *)arg;
+  auto *s = (struct udf_materialize_state_s *)arg;
 
   make_row(udf, s->row);
   s->row++;
@@ -153,18 +153,18 @@ end:
 int table_user_defined_functions::make_row(const udf_func *entry,
                                            row_user_defined_functions *row) {
   /* keep in sync with Item_result */
-  static const char *return_types[] = {
+  static const char *const return_types[] = {
       "char", "double", "integer", "row", /** not valid for UDFs */
       "decimal" /** char *, to be converted to/from a decimal */
   };
-  static uint return_type_lengths[] = {sizeof("char") - 1, sizeof("double") - 1,
-                                       sizeof("integer") - 1, sizeof("row") - 1,
-                                       sizeof("decimal") - 1};
+  static uint const return_type_lengths[] = {
+      sizeof("char") - 1, sizeof("double") - 1, sizeof("integer") - 1,
+      sizeof("row") - 1, sizeof("decimal") - 1};
 
   /* keep in sync with Item_udftype */
-  static const char *udf_types[] = {nullptr,  // invalid value
-                                    "function", "aggregate"};
-  static uint udf_type_lengths[] = {
+  static const char *const udf_types[] = {nullptr,  // invalid value
+                                          "function", "aggregate"};
+  static uint const udf_type_lengths[] = {
       0,  // invalid value
       sizeof("function") - 1,
       sizeof("aggregate") - 1,
@@ -197,12 +197,12 @@ int table_user_defined_functions::make_row(const udf_func *entry,
   return 0;
 }
 
-void table_user_defined_functions::reset_position(void) {
+void table_user_defined_functions::reset_position() {
   m_pos.m_index = 0;
   m_next_pos.m_index = 0;
 }
 
-int table_user_defined_functions::rnd_next(void) {
+int table_user_defined_functions::rnd_next() {
   int result;
 
   m_pos.set_at(&m_next_pos);
@@ -243,7 +243,7 @@ int table_user_defined_functions::index_init(uint idx, bool) {
   return 0;
 }
 
-int table_user_defined_functions::index_next(void) {
+int table_user_defined_functions::index_next() {
   for (m_pos.set_at(&m_next_pos); m_pos.m_index < m_row_count; m_pos.next()) {
     m_row = &m_all_rows[m_pos.m_index];
 
@@ -274,19 +274,19 @@ int table_user_defined_functions::read_row_values(TABLE *table,
     if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
       switch (f->field_index()) {
         case 0: /* UDF_NAME */
-          set_field_varchar_utf8(f, m_row->m_name, m_row->m_name_length);
+          set_field_varchar_utf8mb4(f, m_row->m_name, m_row->m_name_length);
           break;
         case 1: /* UDF_RETURN_TYPE */
-          set_field_varchar_utf8(f, m_row->m_return_type,
-                                 m_row->m_return_type_length);
+          set_field_varchar_utf8mb4(f, m_row->m_return_type,
+                                    m_row->m_return_type_length);
           break;
         case 2: /* UDF_TYPE */
-          set_field_varchar_utf8(f, m_row->m_type, m_row->m_type_length);
+          set_field_varchar_utf8mb4(f, m_row->m_type, m_row->m_type_length);
           break;
         case 3: /* UDF_LIBRARY */
           if (m_row->m_library_length)
-            set_field_varchar_utf8(f, m_row->m_library,
-                                   m_row->m_library_length);
+            set_field_varchar_utf8mb4(f, m_row->m_library,
+                                      m_row->m_library_length);
           else
             f->set_null();
           break;
