@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -27,10 +27,10 @@
 
 #include "storage/perfschema/table_variables_by_thread.h"
 
+#include <assert.h>
 #include <stddef.h>
 #include <new>
 
-#include "my_dbug.h"
 #include "my_thread.h"
 #include "sql/current_thd.h"
 #include "sql/field.h"
@@ -99,10 +99,14 @@ PFS_engine_table *table_variables_by_thread::create(PFS_engine_table_share *) {
   return new table_variables_by_thread();
 }
 
-ha_rows table_variables_by_thread::get_row_count(void) {
+ha_rows table_variables_by_thread::get_row_count() {
   mysql_mutex_lock(&LOCK_plugin_delete);
+#ifndef NDEBUG
+  extern mysql_mutex_t LOCK_plugin;
+  mysql_mutex_assert_not_owner(&LOCK_plugin);
+#endif
   mysql_rwlock_rdlock(&LOCK_system_variables_hash);
-  ulong system_var_count = get_system_variable_hash_records();
+  const ulong system_var_count = get_system_variable_count();
   mysql_rwlock_unlock(&LOCK_system_variables_hash);
   mysql_mutex_unlock(&LOCK_plugin_delete);
   return (global_thread_container.get_row_count() * system_var_count);
@@ -112,33 +116,21 @@ table_variables_by_thread::table_variables_by_thread()
     : PFS_engine_table(&m_share, &m_pos),
       m_sysvar_cache(true),
       m_pos(),
-      m_next_pos(),
-      m_context(nullptr) {}
+      m_next_pos() {}
 
-void table_variables_by_thread::reset_position(void) {
+void table_variables_by_thread::reset_position() {
   m_pos.reset();
   m_next_pos.reset();
 }
 
-int table_variables_by_thread::rnd_init(bool scan) {
+int table_variables_by_thread::rnd_init(bool /* scan */) {
   /* Build array of SHOW_VARs from system variable hash. */
   m_sysvar_cache.initialize_session();
 
-  /* Record the version of the system variable hash, store in TLS. */
-  ulonglong hash_version = m_sysvar_cache.get_sysvar_hash_version();
-
-  m_context = (table_variables_by_thread_context *)current_thd->alloc(
-      sizeof(table_variables_by_thread_context));
-  new (m_context) table_variables_by_thread_context(hash_version, !scan);
   return 0;
 }
 
-int table_variables_by_thread::rnd_next(void) {
-  if (m_context && !m_context->versions_match()) {
-    system_variable_warning();
-    return HA_ERR_END_OF_FILE;
-  }
-
+int table_variables_by_thread::rnd_next() {
   bool has_more_thread = true;
 
   for (m_pos.set_at(&m_next_pos); has_more_thread; m_pos.next_thread()) {
@@ -162,13 +154,8 @@ int table_variables_by_thread::rnd_next(void) {
 }
 
 int table_variables_by_thread::rnd_pos(const void *pos) {
-  if (m_context && !m_context->versions_match()) {
-    system_variable_warning();
-    return HA_ERR_END_OF_FILE;
-  }
-
   set_position(pos);
-  DBUG_ASSERT(m_pos.m_index_1 < global_thread_container.get_row_count());
+  assert(m_pos.m_index_1 < global_thread_container.get_row_count());
 
   PFS_thread *pfs_thread = global_thread_container.get(m_pos.m_index_1);
 
@@ -183,19 +170,12 @@ int table_variables_by_thread::rnd_pos(const void *pos) {
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_variables_by_thread::index_init(uint idx MY_ATTRIBUTE((unused)),
-                                          bool) {
+int table_variables_by_thread::index_init(uint idx [[maybe_unused]], bool) {
   /* Build array of SHOW_VARs from the system variable hash. */
   m_sysvar_cache.initialize_session();
 
-  /* Record the version of the system variable hash, store in TLS. */
-  ulonglong hash_version = m_sysvar_cache.get_sysvar_hash_version();
-  m_context = (table_variables_by_thread_context *)current_thd->alloc(
-      sizeof(table_variables_by_thread_context));
-  new (m_context) table_variables_by_thread_context(hash_version, false);
-
   PFS_index_variables_by_thread *result = nullptr;
-  DBUG_ASSERT(idx == 0);
+  assert(idx == 0);
   result = PFS_NEW(PFS_index_variables_by_thread);
   m_opened_index = result;
   m_index = result;
@@ -203,12 +183,7 @@ int table_variables_by_thread::index_init(uint idx MY_ATTRIBUTE((unused)),
   return 0;
 }
 
-int table_variables_by_thread::index_next(void) {
-  if (m_context && !m_context->versions_match()) {
-    system_variable_warning();
-    return HA_ERR_END_OF_FILE;
-  }
-
+int table_variables_by_thread::index_next() {
   bool has_more_thread = true;
 
   for (m_pos.set_at(&m_next_pos); has_more_thread; m_pos.next_thread()) {
@@ -272,24 +247,24 @@ int table_variables_by_thread::read_row_values(TABLE *table, unsigned char *buf,
   Field *f;
 
   /* Set the null bits */
-  DBUG_ASSERT(table->s->null_bytes == 1);
+  assert(table->s->null_bytes == 1);
   buf[0] = 0;
 
   for (; (f = *fields); fields++) {
-    if (read_all || bitmap_is_set(table->read_set, f->field_index)) {
-      switch (f->field_index) {
+    if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
+      switch (f->field_index()) {
         case 0: /* THREAD_ID */
           set_field_ulonglong(f, m_row.m_thread_internal_id);
           break;
         case 1: /* VARIABLE_NAME */
-          set_field_varchar_utf8(f, m_row.m_variable_name.m_str,
-                                 m_row.m_variable_name.m_length);
+          set_field_varchar_utf8mb4(f, m_row.m_variable_name.m_str,
+                                    m_row.m_variable_name.m_length);
           break;
         case 2: /* VARIABLE_VALUE */
           m_row.m_variable_value.set_field(f);
           break;
         default:
-          DBUG_ASSERT(false);
+          assert(false);
       }
     }
   }
