@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,6 +22,7 @@
 
 #include "unittest/gunit/test_utils.h"
 
+#include <assert.h>
 #include <gtest/gtest.h>
 #include <atomic>
 #include <new>
@@ -31,7 +32,7 @@
 #include "gtest/gtest-message.h"
 #include "m_ctype.h"
 #include "m_string.h"
-#include "my_dbug.h"  // DBUG_ASSERT
+// assert
 #include "my_inttypes.h"
 #include "mysql_com.h"
 #include "sql/binlog.h"
@@ -50,7 +51,7 @@
 #include "sql/mysqld.h"  // set_remaining_args
 #include "sql/mysqld_thd_manager.h"
 #include "sql/opt_costconstantcache.h"  // optimizer cost constant cache
-#include "sql/opt_range.h"
+#include "sql/range_optimizer/range_optimizer.h"
 #include "sql/rpl_filter.h"
 #include "sql/rpl_handler.h"  // delegates_init()
 #include "sql/set_var.h"
@@ -59,7 +60,11 @@
 #include "sql/sql_locale.h"
 #include "sql/sql_plugin.h"
 #include "sql/xa.h"
+#include "sql/xa/transaction_cache.h"  // xa::Transaction_cache
 #include "unicode/uclean.h"
+#include "unittest/gunit/fake_table.h"
+
+void my_server_abort();
 
 namespace my_testing {
 
@@ -86,16 +91,19 @@ void setup_server_for_unit_tests() {
                   const_cast<char *>("--lc-messages-dir=" ERRMSG_DIR),
                   nullptr};
   set_remaining_args(6, argv);
-  system_charset_info = &my_charset_utf8_general_ci;
+  system_charset_info = &my_charset_utf8mb3_general_ci;
 
   mysql_mutex_init(PSI_NOT_INSTRUMENTED, &LOCK_plugin, MY_MUTEX_INIT_FAST);
   sys_var_init();
   init_common_variables();
   test_flags |= TEST_SIGINT;
+  test_flags |= TEST_NO_TEMP_TABLES;
   test_flags &= ~TEST_CORE_ON_SIGNAL;
   my_init_signals();
+  // Install server's abort handler to better represent server environment.
+  set_my_abort(my_server_abort);
   randominit(&sql_rand, 0, 0);
-  transaction_cache_init();
+  xa::Transaction_cache::initialize();
   delegates_init();
   gtid_server_init();
   error_handler_hook = test_error_handler_hook;
@@ -110,7 +118,7 @@ void teardown_server_for_unit_tests() {
   range_optimizer_free();
   sys_var_end();
   delegates_destroy();
-  transaction_cache_free();
+  xa::Transaction_cache::dispose();
   gtid_server_cleanup();
   query_logger.cleanup();
   item_func_sleep_free();
@@ -131,6 +139,8 @@ void teardown_server_for_unit_tests() {
   Global_THD_manager::destroy_instance();
   my_end(0);
   clean_up_mysqld_mutexes();
+  // Restore standard's abort.
+  set_my_abort(abort);
 }
 
 void Server_initializer::set_expected_error(uint val) { expected_error = val; }
@@ -145,11 +155,15 @@ void Server_initializer::SetUp() {
   m_thd->thread_stack = (char *)&stack_thd;
   m_thd->store_globals();
   lex_start(m_thd);
+  Fake_TABLE::reset_highest_table_id();
 }
 
 void Server_initializer::TearDown() {
-  m_thd->cleanup_after_query();
-  delete m_thd;
+  if (m_thd != nullptr) {
+    m_thd->cleanup_after_query();
+    delete m_thd;
+    m_thd = nullptr;
+  }
 }
 
 Mock_error_handler::Mock_error_handler(THD *thd, uint expected_error)
@@ -179,7 +193,7 @@ bool Mock_error_handler::handle_condition(THD *, uint sql_errno, const char *,
 
 void DD_initializer::SetUp() {
   /*
-    With WL#6599, SELECT_LEX::add_table_to_list() will invoke
+    With WL#6599, Query_block::add_table_to_list() will invoke
     dd::Dictionary::is_system_view_name() method. E.g., the unit
     test InsertDelayed would invoke above API. This requires us
     to have a instance of dictionary_impl. We do not really need
@@ -187,11 +201,11 @@ void DD_initializer::SetUp() {
     be future test cases that need the same.
   */
   dd::Dictionary_impl::s_instance = new (std::nothrow) dd::Dictionary_impl();
-  DBUG_ASSERT(dd::Dictionary_impl::s_instance != nullptr);
+  assert(dd::Dictionary_impl::s_instance != nullptr);
 }
 
 void DD_initializer::TearDown() {
-  DBUG_ASSERT(dd::Dictionary_impl::s_instance != nullptr);
+  assert(dd::Dictionary_impl::s_instance != nullptr);
   delete dd::Dictionary_impl::s_instance;
 }
 
